// DVDSubEditDlg.h : header file
//

#if !defined(AFX_DVDSubEditDLG_H__8849CA41_8885_4178_8047_BB28B35DE116__INCLUDED_)
#define AFX_DVDSubEditDLG_H__8849CA41_8885_4178_8047_BB28B35DE116__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Utils.h"
#include "SubPicEngine.h"
#include "DVDPreview.h"

class CScanDlg;
class CApplyToAllDlg;
class CMoveDlg;
class CSetupDlg;

typedef enum
{
	ChangeVMove = 0,
	ChangeHMove,
	ChangeBorder,
	ChangeTransp,
	ChangeColor,
	ChangeStart,
	ChangeDur,
	ChangeCrop,
	ChangeToInfinite,
	ChangeAddFadeIn,
	ChangeHide,
} Change;

// Enum for how to display 16:9 material.
typedef enum
{
	WSDispNormal = 0,	// Equivalent to the widescreen mode.
	WSDispLB,			// Letterboxed. Black mattes at top and bottom.
	WSDispPS,			// Pan&Scan, 3/4 of the width is shown.
} WSDisplayMode;

#define NUM_MRU 8

class Modif
{
public:
	// Reset all params.
	void	Reset();
	// Record a change made to TheSubpic. Which indicates what was changed, and Value1 and Value2
	// are parameters describing the change. If TheSubpic->m_SPUNumber is not m_WhichSPU, all values
	// are reset before the change is recorded.
	void	RecordChange(CSubPic* TheSubpic, Change Which, int Value1, int Value2=0);
	
	int		m_VMove;
	int		m_HMove;
	int		m_Border;
	uchar	m_Transparency[5];
	int		m_TranspPixel;
	int		m_StartCommand;
	int		m_Duration;
	uchar	m_Color[4];
	int		m_ColorPixel;
	int		m_Crop;
	int		m_MakeInfinite;
	int		m_AddFadeIn;
	int		m_Hide;
	uchar*	m_CLUT;
	int		m_WhichSPU;			// Which SPU were these modifications made on.
	int		m_ApplyToAllDCSQTs;
	int		m_DCSQT;			// Which DCSQT to modify.
};

/////////////////////////////////////////////////////////////////////////////
// CDVDSubEditDlg dialog

class CDVDSubEditDlg : public CDialog, public CSubPicEngine
{
// Construction
public:
	CDVDSubEditDlg(CWnd* pParent = NULL);	// standard constructor

// Dialog Data
	//{{AFX_DATA(CDVDSubEditDlg)
	enum { IDD = IDD_DVDSubEdit_DIALOG };
	CButton	m_ShowTransparent;
	CComboBox	m_AudioStreamSel;
	CButton	m_Loop;
	CButton	m_Play;
	CComboBox	m_CLUTSelect;
	CButton	m_ShowHighlights;
	CComboBox	m_ZoomSelect;
	CButton	m_MakeInfinite;
	CButton	m_ShowOriginalOCR;
	CButton	m_ShowEndFrame;
	CComboBox	m_DispModeSelect;
	CSliderCtrl	m_IFrameSelect;
	CButton	m_ShowForced;
	CButton	m_FlushBorders;
	CSliderCtrl	m_MoveHSlider;
	CButton	m_ShowSearched;
	CRichEditCtrl	m_OCROut;
	CComboBox	m_DCSQTSelect;
	CSliderCtrl	m_DurationSlider;
	CButton	m_ShowModified;
	CComboBox	m_BorderSelect;
	CSliderCtrl	m_BorderSlider;
	CComboBox	m_StartCommand;
	CComboBox	m_PixelSelect;
	CSliderCtrl	m_TransSlider;
	CButton	m_ShowVideo;
	CButton	m_SaveAll;
	CButton	m_Revert;
	CSliderCtrl	m_MoveVSlider;
	CComboBox	m_VCIDSelect;
	CSliderCtrl	m_SPUSlider;
	CComboBox	m_StreamSelect;
	CRichEditCtrl	m_CommandDisplay;
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CDVDSubEditDlg)
	public:
	virtual BOOL PreTranslateMessage(MSG* pMsg);
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support
	//}}AFX_VIRTUAL
	BOOL OnToolTipNotify( UINT id, NMHDR * pNMHDR, LRESULT * pResult );

public:
	CSettings			TheSettings;
	CScanDlg			*m_ScanDialog;
	CApplyToAllDlg		*m_ApplyAllDialog;
	CMoveDlg			*m_MoveDialog;
	CFindReplaceDialog	*m_FindDlg;
	CSetupDlg			*m_SetupDialog;
	CDialog				*m_ZoomDialog;
	CDVDPreview			*m_Preview;
	HWND				m_PreviewHandle;

	int					m_SubpicAtStartPlayback;	// Subpic where playback started.

	CMenu				m_Menu;
	bool				m_MenuEnabled;
	int					m_DisplayVideo;
	int					m_RealSizeDisplay;
	COLORREF			m_DefBackgroundColor;
	COLORREF			m_DefAutoPixColor;
	COLORREF			m_DefAutoBckgColor;
	COLORREF			m_DefAutoAA1Color;
	COLORREF			m_DefAutoAA2Color;
	bool				m_CMDOpenFullDomain;	// Open full domain for command-line input.
	int					m_LastBitmapSaveType;	// Bitmap saving type used last...
	int					m_AutoOpenVideo;		// It true, the video is displayed by default (if possible).

	int					m_NumMRU;
	void				AddFilesToMRU();
	int					FindFilesInMRU(bool IgnoreDate = false);
	void				UpdateMRUMenu();
	void				MoveMRUFileToTop(int i);

	
	Modif				m_ChangeRecord;

	void				OnDropFiles(HDROP hDropInfo) ;
	void				StartScan();
	virtual void		JustAddedSPU(int StreamIndex, int SPUIndex);
	virtual void		JustFoundStream(int StreamIndex);
	virtual void		JustFoundVCID(uint VCID);
	virtual void		Reset();
	virtual void		JustFoundVideoFormat(bool redraw = true);
						
	void				EnableUI(bool Enable, bool Full = false);
	int					m_SelectedStream;
	void				ShowCol(HDC TheDC, uint ID, uchar* Colormap);
	void				ViewSPU(int SPUIndex, bool RefreshVideo = true, bool RefreshCommands = true);

	int					m_Running;
	bool				m_QuitNow;
	int					m_Timer;
	HANDLE				m_ThreadHandle;
	int					m_DisplayHeight;
	WSDisplayMode		m_WSDisplayMode;
	int					m_WarnIfOnly4_3;		// Issue warning if DVD only has a 4/3 display mode.
	bool				m_4_3WarnedAlready;

	int					m_InitWindowWidth;		// Initial size of entire window.
	int					m_InitWindowHeight;		// Initial size of entire window.

	int					m_NextSPULba;			// Lba of navpack of next SPU in stream.
	int					m_LastDispLBA;			// LBA of last image displayed.

	BITMAPINFOHEADER	m_BitMapHeader;
	uchar*				m_OCRBitmap;
	float				m_VideoScalingX;
	float				m_VideoScalingY;
	float				m_SubPicScalingX;
	float				m_SubPicScalingY;
	CRect				m_VideoRect;

	// These strings are read from the ini file upon startup, to remember various dialog default paths.
	char*				m_LastScanFile;
	char*				m_LastPPMDir;
	char*				m_LastOpenSUPDir;
	char*				m_LastSaveSUPDir;
	char*				m_LastVOBDir;
	char*				m_LastSRTDir;
	char*				m_LogPath;
	char*				m_DefIFOPath;
	char*				m_CMDInputFile[10];	// Path of file passed as command line. Just a pointer.

	char				(*m_OCRLines)[128];		// Results of OCR;
	int					m_NumOCRLines;			// How many lines were found.
	char				m_SearchStr[64];

	uchar*				m_ColorMap;


	// Border values...
	int					m_Border4_3S_N;			// 4:3 safe, NTSC and PAL.
	int					m_Border4_3S_P;
	int					m_Border4_3LB_N;		// 4:3 letterboxed, NTSC and PAL.
	int					m_Border4_3LB_P;
	int					m_Border4_3LBS_N;		// 4:3 letterboxed safe, NTSC and PAL.
	int					m_Border4_3LBS_P;
	int					m_Border1_85_N;			// 1:1.85, NTSC and PAL.
	int					m_Border1_85_P;
	int					m_Border2_39_N;			// 1:2.39, NTSC and PAL
	int					m_Border2_39_P;

// Implementation
	HICON				m_hIcon;

	void				DispInfo(char* Info, int Color);
	void				DispOCR(CSubPic* ThisOne);
	void				UpdateSPUList(bool ForceDisplay = false);
	void				ShowBorders();
	void				DisplayBitmap(unsigned char* Bitmap, long Width, long Height, bool Black = false);
	void				SetMoveVSlider(int Pos);
	void				SetMoveHSlider(int Pos);
	void				SetTransSlider(int Pos);
	void				SetBorderSlider(int Pos);
	void				SetBorderSelect(int Selection);
	void				SetDurationSlider(int Pos);
	void				SetIFrameSlider(CSubPic* ThisOne);
	bool				WarnIfModifs();
	void				DoRunOCR(CSubPic* TheSubpic);
	void				OCRThisOne();
	void				OCRAll();
	void				FindNext(int Inc);
	void				ChangeColors(uchar* Colors, int Pixels, CSubPic* TheSubpic);
	void				ChangeTransparencies(uchar* Transparencies, int Pixels, CSubPic* TheSubpic);
	void				LoadLastFile(UINT Index);
	void				OnFileLoaded(const char* FileName);
	void				OnCropSubpic();
	void				OnOpenIFO();
	bool				HideHearingImpairedText(CSubPic* ThisOne);
	void				HideAllHearingImpairedText();
	void				HideHearingImpairedTextSingle();
	void				EraseSelectedText();
	void				OnOCRSetup();
	void				FillMissingPix(uchar* Bitmap, int Width, int Height, uchar Body, uchar Background, uchar FillPixel = 255);

	void				TestOCR();
	bool				OpenDomain(char* Path);

	bool				DlgPtInRect(UINT ID, CPoint point);
	uchar*				GetCLUT();
	void				AdjustDisplayMode();

	// WARNING: IF YOU REMOVE THIS ARRAY THE PROGRAM CRASHES UPON STARTUP! MAKES NO FUCKING SENSE.
	// ONLY IN DEV STUDIO THOUGH... NO! It crashes when invoked from the command line as well! AAAAAH!
	uchar				m_WhyDoINeedThisFuckingSHIT[323];

	// Generated message map functions
	//{{AFX_MSG(CDVDSubEditDlg)
	virtual BOOL OnInitDialog();
	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();
	afx_msg void OnDestroy();
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg HBRUSH OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor);
	afx_msg void OnSpur();
	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnSelchangeStreamselect();
	afx_msg void OnSpul();
	afx_msg void OnRevertsaved();
	afx_msg void OnPvcid();
	afx_msg void OnNvcid();
	afx_msg void OnPstream();
	afx_msg void OnNstream();
	afx_msg void OnApplytoall();
	afx_msg void OnRevertall();
	afx_msg void OnClose();
	afx_msg void OnSaveall();
	afx_msg void OnDefbutton();
	afx_msg void OnKillfocusMoveude();
	afx_msg void OnKillfocusMovelre();
	afx_msg void OnShowvideo();
	afx_msg void OnSelchangePixselect();
	afx_msg void OnSelchangeStart();
	afx_msg void OnSelchangeBorderselect();
	afx_msg void OnShowmod();
	afx_msg void OnKillfocusBordere();
	afx_msg void OnHelpAbout();
	afx_msg void OnHelpManual();
	afx_msg void OnHelpOntheweb();
	afx_msg void OnEditRedo();
	afx_msg void OnExit();
	afx_msg void OnEditApplyall();
	afx_msg void OnDeltaposMoveb(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnDeltaposMovelrb(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnDeltaposBorderb(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnOpenVob();
	afx_msg void OnOpenDomain();
	afx_msg void OnSelchangeDcsqtselect();
	afx_msg void OnDeltaposDscqtb(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnHidesubpic();
	afx_msg void OnHelpViewlog();
	afx_msg void OnFindLeft();
	afx_msg void OnFindRight();
	afx_msg void OnSavePPM();
	afx_msg void OnSaveBMP();
	afx_msg void OnLoadBMP();
	afx_msg void OnExportImportBMP();
	afx_msg void OnShowsearched();
	afx_msg void OnChangedSearchselect();
	afx_msg void OnKillfocusDuratione();
	afx_msg void OnSaveSup();
	afx_msg void OnFlushBorder();
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnKillfocusGoto();
	afx_msg void OnSelchangeDisplaymode();
	afx_msg void OnShoworigocr();
	afx_msg void OnOcrGuide();
	afx_msg void OnColorGuide();
	afx_msg void MakeInfinite();
	afx_msg void OnAddfadein();
	afx_msg void OnRemovefadein();
	afx_msg void OnContextMenu(CWnd* pWnd, CPoint point);
	afx_msg void OnCommandsSave();
	afx_msg void OnSaveVideoFrame();
	afx_msg void OnSelendokZoomsel();
	afx_msg void OnSelendokClutselect();
	afx_msg void OnPlay();
	afx_msg void OnLoop();
	afx_msg void OnSelendokAudiosel();
	//}}AFX_MSG
	void OnClutClick(UINT nID);
	afx_msg void OnSaveOCR();
	afx_msg void OnTimingSingle();
	afx_msg void OnTimingAll();
	afx_msg void OnOpenSup();
	afx_msg void OnGetMinMaxInfo( MINMAXINFO FAR* lpMMI );
	afx_msg void OnInitMenu( CMenu* pMenu );
	afx_msg void OnActivate( UINT nState, CWnd* pWndOther, BOOL bMinimized );
	void OnPlaybackStopped();
	void AbortPlayback(bool NoReturn = false);


	DECLARE_MESSAGE_MAP()
protected:
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_DVDSubEditDLG_H__8849CA41_8885_4178_8047_BB28B35DE116__INCLUDED_)
