/*
 *  DVDSubEdit and this code is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  DVDSubEdit and this code are distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
*/

#include "stdafx.h"
#include "DVDSubEdit.h"
#include "DVDSubEditDlg.h"
#include "Cell.h"
#include "Utils.h"
#include "SubPic.h"
#include "MiscDialogs.h"
#include <math.h>
#include <wingdi.h>

#define NO_CHANGE -123454321

#ifndef AC_SRC_ALPHA
#define AC_SRC_ALPHA                0x01
#endif

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define MAX_V_MOVE 600
#define MAX_H_MOVE 500
#define MAX_BORDER 120
#define MAX_DURATION 10000
#define GRAY RGB(100,100,100)
#if 0
#define BLACK RGB(0,0,0)
#define RED RGB(255,0,0)
#define GREEN RGB(0,134,37)
#define BLUE RGB(0,0,255)
#else
#define BLACK 0
#define RED 1
#define GREEN 2
#define BLUE 3
#endif

// Define this if you want the "hide" button to erase the subpic entirely, instead of turning it transparent.
#define ERASE_WHEN_HIDE

bool Annoy = false;

#define DispThis1(ID,format) {SetDlgItemText(ID,format);}
#define DispThis2(ID,format,val) {char Temp[512]; sprintf(Temp,format,val);SetDlgItemText(ID,Temp);}
#define DispThis3(ID,format,val1,val2) {char Temp[512]; sprintf(Temp,format,val1,val2);SetDlgItemText(ID,Temp);}

#define ENABLE(ID,En){CButton* Temp = (CButton*)GetDlgItem(ID);Temp->EnableWindow(En);}

void OutputError(char* AMessage, int Langage)
{
	if(Langage != ENGLISH) return;
	AfxMessageBox(AMessage,MB_ICONSTOP);
#if _DEBUG
	OutputDebugString(Message);
	OutputDebugString("\n");
#endif
}

// A function to prompt the user for a response.
int UserPrompt(const char* Prompt)
{
	return AfxMessageBox(Prompt,MB_YESNO) == IDYES;
}

FILE*	OutLogFd;

void OutputLog(char* AMessage, int Langage)
{
	if(Langage != ENGLISH) return;
	CTime CurrentTime = CTime::GetCurrentTime();
	CString Time = CurrentTime.Format("%H:%M:%S");
	if(Message[strlen(AMessage)-1]=='\n') AMessage[strlen(AMessage)-1] = 0;
	if(OutLogFd) {fprintf(OutLogFd,"%s -- %s\n",Time,AMessage);fflush(OutLogFd);}
	strcpy(Message,AMessage);
#if _DEBUG
	OutputDebugString(Message);
	OutputDebugString("\n");
#endif
}

enum
{
	BORDER_CUSTOM = 0,
	BORDER_SAFE,
	BORDER_4_3_L,
	BORDER_4_3_LS,
	BORDER_1_85_L,
	BORDER_2_39_L,
};

// This is for the prototype for RunOCR()
#include "pnmmem.h"

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

	CDVDSubEditDlg	*m_Edit;

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
	virtual BOOL OnInitDialog();
	afx_msg void OnShowhelp();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BOOL CAboutDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	SetDlgItemText(IDC_VERSIONSTRING,m_Edit->m_Version);
	return TRUE; 
}

void CAboutDlg::OnShowhelp() 
{
	OpenRSCFile(IDR_HELPFILE, ".pdf");
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
	ON_BN_CLICKED(IDC_SHOWHELP, OnShowhelp)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
class CScanDlg : public CDialog
{
public:
	CScanDlg();

	CSubPicEngine*	m_Engine;
	void			DisplayInfo(int Progress, char* Info, char* Message);

// Dialog Data
	//{{AFX_DATA(CScanDlg)
	enum { IDD = IDD_SCANDIALOG };
	CProgressCtrl	m_Progress;
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CScanDlg)
	public:
	virtual BOOL PreTranslateMessage(MSG* pMsg);
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CScanDlg)
	afx_msg void StartScan();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CScanDlg::CScanDlg() : CDialog(CScanDlg::IDD)
{
	//{{AFX_DATA_INIT(CScanDlg)
	//}}AFX_DATA_INIT
	m_Engine = 0;
}

void CScanDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CScanDlg)
	DDX_Control(pDX, IDC_PROGRESS, m_Progress);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CScanDlg, CDialog)
	//{{AFX_MSG_MAP(CScanDlg)
	ON_BN_CLICKED(IDC_RUN, StartScan)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


void CScanDlg::StartScan() 
{
	m_Engine->m_AbortNow = true;
}

void CScanDlg::DisplayInfo(int Progress, char* Info, char* AMessage)
{
	m_Progress.SetPos(Progress);
	SetWindowText(Info);
	SetDlgItemText(IDC_INFO,Info);
	if(AMessage[strlen(AMessage)-1]=='\n') AMessage[strlen(AMessage)-1] = 0;
	SetDlgItemText(IDC_MESSAGE,AMessage);
}

BOOL CScanDlg::PreTranslateMessage(MSG* pMsg) 
{
	if(pMsg->message == WM_KEYDOWN)
	{
		switch(pMsg->wParam)
		{
			// Must be all CAPS...
		case VK_ESCAPE: m_Engine->m_AbortNow = true;  break;
		default: return GetParent()->PreTranslateMessage(pMsg);  break;
		}
		return TRUE;
	}
	else return CDialog::PreTranslateMessage(pMsg);
}

/////////////////////////////////////////////////////////////////////////////
void Modif::Reset()
{
	m_VMove = m_HMove = m_StartCommand = m_Duration = m_Crop = m_MakeInfinite = m_AddFadeIn = m_Hide = NO_CHANGE;
	// These are special. I'm resetting them to 0, because they will be or'ed when modified.
	m_ColorPixel = 0; memset(m_Color,0,4);
	m_TranspPixel = 0; memset(m_Transparency,0,5);
	m_Border = 0;
	m_ApplyToAllDCSQTs = 0;
	m_DCSQT = 0;
}

void Modif::RecordChange(CSubPic* TheSubpic, Change Which, int Value1, int Value2)
{
	// If this modification is applied to a "new" subpic, let's reset all the previously recorded changes.
	if(TheSubpic->m_SPUNumber != m_WhichSPU) Reset();
	// Register the changes.
	int Pixel;
	switch(Which)
	{
	case ChangeVMove : m_VMove = Value1; break;
	case ChangeHMove : m_HMove = Value1; break;
	case ChangeBorder : m_Border = Value1; break;
	case ChangeTransp : 
		Pixel = ((int)1)<<Value2;
		m_Transparency[Value2] = Value1; m_TranspPixel |= Pixel; 
		m_ApplyToAllDCSQTs = TheSubpic->m_ChangeAllDCSQT;
		break;
	// Color is special: m_ColorPixel has bit 0 set for a change to pixel 0 etc. m_Color has 1 byte
	// storing the color per pixel.
	case ChangeColor : 
		Pixel = ((int)1)<<Value2;
		m_Color[Value2] = Value1; m_ColorPixel |= Pixel; 
		m_ApplyToAllDCSQTs = TheSubpic->m_ChangeAllDCSQT;
		break;
	case ChangeStart : m_StartCommand = Value1; break;
	case ChangeDur : m_Duration = Value1; break;
	case ChangeCrop : m_Crop = Value1; break;
	case ChangeToInfinite : m_MakeInfinite = 1; break;
	case ChangeAddFadeIn : m_AddFadeIn = Value1; break;
	case ChangeHide : m_Hide = 1; break;
	}
	// Remember the CLUT, for changes to color, and the SPUNumber the changes came from.
	m_CLUT = TheSubpic->m_CLUT;
	m_WhichSPU = TheSubpic->m_SPUNumber;
	m_DCSQT = TheSubpic->m_SelectedDCSQT;
}


/////////////////////////////////////////////////////////////////////////////
// CDVDSubEditDlg dialog

CDVDSubEditDlg::CDVDSubEditDlg(CWnd* pParent /*=NULL*/)
//	: m_InitWindowWidth (-1)
	: CDialog(CDVDSubEditDlg::IDD, pParent)
{

//Create(CDVDSubEditDlg::IDD, pParent);
if(Annoy) AfxMessageBox("Creator0");
	//{{AFX_DATA_INIT(CDVDSubEditDlg)
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
if(Annoy) AfxMessageBox("Creator1");
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
if(Annoy) AfxMessageBox("Creator2");
	pErrorFunction = OutputError;
	pMessageFunction = OutputLog;
	m_OCRBitmap = 0;
	m_OCRLines = 0;
	m_InitWindowWidth = m_InitWindowHeight = -1;
	m_RealSizeDisplay = 0;

	m_LastScanFile = new char[1024];
	m_LastPPMDir = new char[1024];
	m_LastOpenSUPDir = new char[1024];
	m_LastSaveSUPDir = new char[1024];
	m_LastVOBDir = new char[1024];
	m_LastSRTDir = new char[1024];
	m_LogPath = new char[1024];
	m_DefIFOPath = new char[512];

	m_CMDInputFile[0] = NULL;
	m_CMDOpenFullDomain = false;
	m_UserPromptFunction = UserPrompt;


if(Annoy) AfxMessageBox("Creator3");
}

void CDVDSubEditDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDVDSubEditDlg)
	DDX_Control(pDX, IDC_SHOWTRANSP, m_ShowTransparent);
	DDX_Control(pDX, IDC_AUDIOSEL, m_AudioStreamSel);
	DDX_Control(pDX, IDC_LOOP, m_Loop);
	DDX_Control(pDX, IDC_PLAY, m_Play);
	DDX_Control(pDX, IDC_CLUTSELECT, m_CLUTSelect);
	DDX_Control(pDX, IDC_SHOWHIGHLIGHTS, m_ShowHighlights);
	DDX_Control(pDX, IDC_ZOOMSEL, m_ZoomSelect);
	DDX_Control(pDX, IDC_NOSTOP, m_MakeInfinite);
	DDX_Control(pDX, IDC_SHOWORIGOCR, m_ShowOriginalOCR);
	DDX_Control(pDX, IDC_SHOWENDFRAME, m_ShowEndFrame);
	DDX_Control(pDX, IDC_DISPLAYMODE, m_DispModeSelect);
	DDX_Control(pDX, IDC_VIDEOSELECT, m_IFrameSelect);
	DDX_Control(pDX, IDC_SHOWFORCED, m_ShowForced);
	DDX_Control(pDX, IDC_FLUSH, m_FlushBorders);
	DDX_Control(pDX, IDC_MOVELR, m_MoveHSlider);
	DDX_Control(pDX, IDC_SHOWSEARCHED, m_ShowSearched);
	DDX_Control(pDX, IDC_OCROUT, m_OCROut);
	DDX_Control(pDX, IDC_DCSQTSELECT, m_DCSQTSelect);
	DDX_Control(pDX, IDC_DURATIONS, m_DurationSlider);
	DDX_Control(pDX, IDC_SHOWMOD, m_ShowModified);
	DDX_Control(pDX, IDC_BORDERSELECT, m_BorderSelect);
	DDX_Control(pDX, IDC_BORDERS, m_BorderSlider);
	DDX_Control(pDX, IDC_START, m_StartCommand);
	DDX_Control(pDX, IDC_PIXSELECT, m_PixelSelect);
	DDX_Control(pDX, IDC_TRANSPARENCYS, m_TransSlider);
	DDX_Control(pDX, IDC_SHOWVIDEO, m_ShowVideo);
	DDX_Control(pDX, IDC_SAVEALL, m_SaveAll);
	DDX_Control(pDX, IDC_REVERTSAVED, m_Revert);
	DDX_Control(pDX, IDC_MOVEUDS, m_MoveVSlider);
	DDX_Control(pDX, IDC_VCIDSELECT, m_VCIDSelect);
	DDX_Control(pDX, IDC_SPUSLIDER, m_SPUSlider);
	DDX_Control(pDX, IDC_STREAMSELECT, m_StreamSelect);
	DDX_Control(pDX, IDC_COMMANDS, m_CommandDisplay);
	//}}AFX_DATA_MAP
}

static UINT FindDialogMessage = ::RegisterWindowMessage(FINDMSGSTRING);

BEGIN_MESSAGE_MAP(CDVDSubEditDlg, CDialog)
	//{{AFX_MSG_MAP(CDVDSubEditDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_DESTROY()
	ON_WM_TIMER()
	ON_WM_CTLCOLOR()
	ON_BN_CLICKED(IDC_SPUR, OnSpur)
	ON_WM_HSCROLL()
	ON_CBN_SELCHANGE(IDC_STREAMSELECT, OnSelchangeStreamselect)
	ON_BN_CLICKED(IDC_SPUL, OnSpul)
	ON_BN_CLICKED(IDC_REVERTSAVED, OnRevertsaved)
	ON_BN_CLICKED(IDC_PVCID, OnPvcid)
	ON_BN_CLICKED(IDC_NVCID, OnNvcid)
	ON_BN_CLICKED(IDC_PSTREAM, OnPstream)
	ON_BN_CLICKED(IDC_NSTREAM, OnNstream)
	ON_BN_CLICKED(IDC_APPLYTOALL, OnApplytoall)
	ON_BN_CLICKED(IDC_REVERTALL, OnRevertall)
	ON_WM_CLOSE()
	ON_BN_CLICKED(IDC_SAVEALL, OnSaveall)
	ON_BN_CLICKED(IDC_DEFBUTTON, OnDefbutton)
	ON_EN_KILLFOCUS(IDC_MOVEUDE, OnKillfocusMoveude)
	ON_EN_KILLFOCUS(IDC_MOVELRE, OnKillfocusMovelre)
	ON_BN_CLICKED(IDC_SHOWVIDEO, OnShowvideo)
	ON_CBN_SELCHANGE(IDC_PIXSELECT, OnSelchangePixselect)
	ON_CBN_SELCHANGE(IDC_START, OnSelchangeStart)
	ON_CBN_SELCHANGE(IDC_BORDERSELECT, OnSelchangeBorderselect)
	ON_BN_CLICKED(IDC_SHOWMOD, OnShowmod)
	ON_BN_CLICKED(IDC_SHOWTRANSP, OnShowmod)
	ON_EN_KILLFOCUS(IDC_BORDERE, OnKillfocusBordere)
	ON_COMMAND(ID_HELP_ABOUT, OnHelpAbout)
	ON_COMMAND(ID_HELP_MANUAL, OnHelpManual)
	ON_COMMAND(ID_HELP_ONTHEWEB, OnHelpOntheweb)
	ON_COMMAND(ID_EDIT_REDO, OnEditRedo)
	ON_COMMAND(ID_FILE_EXIT, OnExit)
	ON_COMMAND(ID_EDIT_APPLYALL, OnEditApplyall)
	ON_NOTIFY(UDN_DELTAPOS, IDC_MOVEB, OnDeltaposMoveb)
	ON_NOTIFY(UDN_DELTAPOS, IDC_MOVELRB, OnDeltaposMovelrb)
	ON_NOTIFY(UDN_DELTAPOS, IDC_BORDERB, OnDeltaposBorderb)
	ON_COMMAND(ID_FILE_OPENVOB, OnOpenVob)
	ON_COMMAND(ID_FILE_OPENDOMAIN, OnOpenDomain)
	ON_CBN_SELCHANGE(IDC_DCSQTSELECT, OnSelchangeDcsqtselect)
	ON_NOTIFY(UDN_DELTAPOS, IDC_DSCQTB, OnDeltaposDscqtb)
	ON_BN_CLICKED(IDC_HIDESUBPIC, OnHidesubpic)
	ON_COMMAND(ID_HELP_VIEWLOG, OnHelpViewlog)
	ON_BN_CLICKED(IDC_FINDL, OnFindLeft)
	ON_BN_CLICKED(IDC_FINDR, OnFindRight)
	ON_COMMAND(ID_FILE_SAVEPPM, OnSavePPM)
	ON_COMMAND(ID_FILE_SAVEBMP, OnSaveBMP)
	ON_COMMAND(ID_FILE_LOADBMP, OnLoadBMP)
	ON_COMMAND(ID_FILE_EXPORTIMPORTBMP, OnExportImportBMP)
	ON_BN_CLICKED(IDC_SHOWSEARCHED, OnShowsearched)
	ON_EN_CHANGE(IDC_SEARCHSELECT, OnChangedSearchselect)
	ON_EN_KILLFOCUS(IDC_DURATIONE, OnKillfocusDuratione)
	ON_COMMAND(ID_FILE_SAVESUP, OnSaveSup)
	ON_BN_CLICKED(IDC_FLUSH, OnFlushBorder)
	ON_WM_SIZE()
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	ON_EN_KILLFOCUS(IDC_GOTO, OnKillfocusGoto)
	ON_CBN_SELCHANGE(IDC_DISPLAYMODE, OnSelchangeDisplaymode)
	ON_BN_CLICKED(IDC_SHOWORIGOCR, OnShoworigocr)
	ON_COMMAND(ID_OCR_GUIDE, OnOcrGuide)
	ON_COMMAND(ID_OCR_GUIDE2, OnColorGuide)
	ON_BN_CLICKED(IDC_NOSTOP, MakeInfinite)
	ON_BN_CLICKED(IDC_ADDFADEIN, OnAddfadein)
	ON_BN_CLICKED(IDC_REMOVEFADE, OnRemovefadein)
	ON_WM_CONTEXTMENU()
	ON_COMMAND(IDC_CONTEXT_COMMANDS_SAVE, OnCommandsSave)
	ON_COMMAND(ID_FILE_SAVEBACK, OnSaveVideoFrame)
	ON_CBN_SELENDOK(IDC_ZOOMSEL, OnSelendokZoomsel)
	ON_CBN_SELENDOK(IDC_CLUTSELECT, OnSelendokClutselect)
	ON_BN_CLICKED(IDC_PLAY, OnPlay)
	ON_BN_CLICKED(IDC_LOOP, OnLoop)
	ON_WM_SYSCOMMAND()
	ON_BN_CLICKED(IDC_SHOWFORCED, OnShowmod)
	ON_CBN_SELCHANGE(IDC_VCIDSELECT, OnSelchangeStreamselect)
	ON_WM_SHOWWINDOW()
	ON_CBN_SELENDOK(IDC_AUDIOSEL, OnSelendokAudiosel)
	//}}AFX_MSG_MAP
	ON_WM_INITMENU()
	ON_COMMAND(ID_EDIT_REVERTTHIS, OnRevertsaved)
	ON_COMMAND(ID_EDIT_REVERTALL, OnRevertall)
	ON_COMMAND(ID_FILE_SAVE, OnSaveall)
//	ON_COMMAND(ID_FILE_OPENLAST, LoadLastFile)
	ON_COMMAND(ID_FILE_OPENSUP, OnOpenSup)
	ON_COMMAND(ID_FILE_OPENIFO, OnOpenIFO)
	ON_COMMAND(ID_OCR_ONE, OCRThisOne)
	ON_COMMAND(ID_OCR_ALL, OCRAll)
	ON_COMMAND(ID_OCR_REMOVEH, HideAllHearingImpairedText)
	ON_COMMAND(ID_OCR_REMOVEHSINGLE, HideHearingImpairedTextSingle)
	ON_COMMAND(ID_OCR_SETUP, OnOCRSetup)
	ON_WM_DROPFILES()
	ON_WM_GETMINMAXINFO()
	ON_NOTIFY_EX( TTN_NEEDTEXT, 0, OnToolTipNotify )
	ON_BN_CLICKED(IDC_RUNOCR, OCRAll)
	ON_BN_CLICKED(IDC_SAVEOCR, OnSaveOCR)
	ON_BN_CLICKED(IDC_ERASESELECTION, EraseSelectedText)
	ON_BN_CLICKED(IDC_RETIMEALL, OnTimingAll)
	ON_BN_CLICKED(IDC_RETIMEONE, OnTimingSingle)
	ON_BN_CLICKED(IDC_CROP, OnCropSubpic)
	ON_BN_CLICKED(IDC_SHOWENDFRAME, OnShowvideo)
	ON_BN_CLICKED(IDC_SHOWHIGHLIGHTS, OnShowvideo)
	ON_CONTROL_RANGE(BN_CLICKED, IDC_COLB, IDC_COLE2, OnClutClick )
	ON_COMMAND_RANGE(ID_FILE_MRU0, ID_FILE_MRU7, LoadLastFile )
	ON_WM_ACTIVATE()
END_MESSAGE_MAP()

BOOL CDVDSubEditDlg::OnToolTipNotify( UINT id, NMHDR * pNMHDR, LRESULT * pResult )
{
	// Multiline tooltips: http://www.thecodeproject.com/miscctrl/tooltipex.asp
    TOOLTIPTEXT *pTTT = (TOOLTIPTEXT *)pNMHDR;
    UINT nID =pNMHDR->idFrom;
	char* PixNamesClut[4]={"'b' (background)","'p' (pattern)","'e1' (emphasis 1)","'e2' (emphasis 2)"};
	char* PixNamesAuto[4]={"background","text","anti-alias (1)","anti-alias (2)"};
	static char Mess[128];
	int i,j;
    if (pTTT->uFlags & TTF_IDISHWND)
    {
        // idFrom is actually the HWND of the tool
        nID = ::GetDlgCtrlID((HWND)nID);
		switch(nID)
		{
		case IDC_COLB : 
		case IDC_COLP : 
		case IDC_COLE1 : 
		case IDC_COLE2 : 
		{
			// Tortuous... show help message as a function of the auto-clut selector...
			j=nID-IDC_COLB;
			i=4;
			if(m_CLUTSelect.GetCurSel() == 0 || m_SelectedSubPic == 0)
				i=j;
			else
			{
				if(m_SelectedSubPic->m_BackgroundPix == j) i=0;
				else if(m_SelectedSubPic->m_BodyPix == j) i=1;
				else if(m_SelectedSubPic->m_AntiAliasPixel1 == j) i=2;
				else if(m_SelectedSubPic->m_AntiAliasPixel2 == j) i=3;
			}
			char** PixNames = m_CLUTSelect.GetCurSel() == 0 || m_SelectedSubPic == 0 ? PixNamesClut : PixNamesAuto;
			if(i<4)
				sprintf(Message,"Shows color used for %s pixels. Click on letter to see only these pixels",PixNames[i]);
			else sprintf(Message,"Pixel not used in subpic");
			pTTT->lpszText = Message;
			break;
		}
		case IDC_SHOWORIGOCR : pTTT->lpszText = "Show OCR of saved subtitle"; break; 		
		case IDC_ERASESELECTION : pTTT->lpszText = "Erase selected text from OCR output and bitmap. (shortcut: Del or Backspace or Ctlr-X)"; break; 		
		case IDC_SHOWVIDEO : pTTT->lpszText = "Display or hide the video frame."; break; 		
		case IDC_ZOOMSEL : pTTT->lpszText = "Display or hide the bitmap zoom window and select zoom factor"; break; 		
		case IDC_GOTO : pTTT->lpszText = "Go straight to selected subpic number."; break; 		
		case IDC_DISPLAYMODE : pTTT->lpszText = "Select the aspect ratio and for 16:9 the desired display mode."; break; 		
		case IDC_SHOWENDFRAME : pTTT->lpszText = "Show I-frame nearest to the selected subpic end time."; break; 		
		case IDC_SHOWFORCED : pTTT->lpszText = "Only show subpics which have a forced-start command (forced subpics)."; break; 
		case IDC_CROP : pTTT->lpszText = "Crop subpic background to allow adjustment of horizontal position in all cases."; break; 
		case IDC_STREAMSELECT : pTTT->lpszText = "Filter subpics based on stream ID"; break; 
		case IDC_SPUL : pTTT->lpszText = "Go to previous subpic"; break; 
		case IDC_SPUR : pTTT->lpszText = "Go to next subpic"; break; 
		case IDC_REVERTSAVED : pTTT->lpszText = "Reloads this subpic from disk"; break; 
		case IDC_VCIDSELECT : pTTT->lpszText = "Filter subpics based on V/CID"; break; 
		case IDC_PVCID : pTTT->lpszText = "Go to previous V/CID"; break; 
		case IDC_NVCID : pTTT->lpszText = "Go to next V/CID"; break; 
		case IDC_PSTREAM : pTTT->lpszText = "Go to previous subpic stream"; break; 
		case IDC_NSTREAM : pTTT->lpszText = "Go to next subpic stream"; break; 
		case IDC_APPLYTOALL : pTTT->lpszText = "Apply these modifications to all selected subpics"; break; 
		case IDC_REVERTALL : pTTT->lpszText = "Reloads all selected subpics from disk"; break; 
		case IDC_SPUSLIDER : pTTT->lpszText = "Picks a subpic in the current selection"; break; 
		case IDC_MOVEUDS : pTTT->lpszText = "Moves the subpic currently displayed up and down"; break; 
		case IDC_SAVEALL : pTTT->lpszText = "Saves all the changes back to file."; break; 
		case IDC_MOVEUDE : pTTT->lpszText = "Enter amount to move the subpic here."; break; 
		case IDC_PIXSELECT : pTTT->lpszText = "Selects of which pixel to change the transparency. All means all non-background pixels."; break; 
		case IDC_TRANSPARENCYS : pTTT->lpszText = "Adjusts transparency for selected pixels."; break; 
		case IDC_START : pTTT->lpszText = "Select the subpic start command. Forced means the subpic will be on if the stream is selected, even if subtitles are off."; break; 
		case IDC_BORDERS : pTTT->lpszText = "Adjusts the vertical margin between subtitles and image borders. Only available if there are no buttons"; break; 
		case IDC_BORDERSELECT : pTTT->lpszText = "Pick standard values for vertical borders. Only available if there are no buttons"; break; 
		case IDC_SHOWMOD : pTTT->lpszText = "Only show modified subpics"; break; 
		case IDC_DURATIONS : pTTT->lpszText = "Adjusts the subpic duration"; break; 
		case IDC_DCSQTSELECT : pTTT->lpszText = "Selects the DCSQT (set of commands) to visualize and modify"; break; 
		case IDC_HIDESUBPIC : pTTT->lpszText = "Completely hide this subpic (make it transparent)"; break; 
		case IDC_RUNOCR : pTTT->lpszText = "Run Optical Character Recognition on selected subpics"; break; 
		case IDC_SAVEOCR : pTTT->lpszText = "Save subpics as .srt file"; break; 
		case IDC_RETIMEALL : pTTT->lpszText = "Modify start time of all selected subpics"; break; 
		case IDC_RETIMEONE : pTTT->lpszText = "Modify start time of current subpic"; break; 
		case IDC_FINDR : pTTT->lpszText = "Find next occurence of search string in subpics"; break; 
		case IDC_FINDL : pTTT->lpszText = "Find previous occurence of search string in subpics"; break; 
		case IDC_SHOWSEARCHED : pTTT->lpszText = "Only show subpics containing one of the following characters"; break; 
		case IDC_SEARCHSELECT : pTTT->lpszText = "Type characters to be found in subpics"; break; 
		case IDC_SEARCH : pTTT->lpszText = "Type search string"; break; 
		case IDC_MOVELR : pTTT->lpszText = "Moves subpic left or right. If disabled, crop the background (button below)."; break; 
		case IDC_FLUSH : pTTT->lpszText = "Moves subpic flush with the nearest border, if borders are used"; break; 
		case IDC_NOSTOP : pTTT->lpszText = "Make the subpic display indefinitely (or until the next one) - removes the STP_DSP command"; break; 
		case IDC_ADDFADEIN : pTTT->lpszText = "Add Fade-In/Out effect to subpic"; break; 
		case IDC_REMOVEFADE : pTTT->lpszText = "Remove Fade-In/Out effect from subpic"; break; 
		case IDC_SHOWHIGHLIGHTS : pTTT->lpszText = "Ignore pixel transparencies, except for background pixel. Useful to see menu highlights"; break; 
		case IDC_CLUTSELECT : pTTT->lpszText = "Use CLUT found in IFO, or use custom CLUT adjusted to best show text"; break; 
		case IDC_PLAY : pTTT->lpszText = "Start/Stop video playback with subtitles."; break; 
		case IDC_LOOP : pTTT->lpszText = "Start/Stop looped video playback with subtitles."; break; 
		case IDC_AUDIOSEL : pTTT->lpszText = "Selects which audio stream to play."; break; 

		}
		pTTT->hinst = AfxGetResourceHandle();
		return(TRUE);
    }
    return(FALSE);
}

// This function will be called after the display of each video frame.
void PostVideoDisplayFunc(HDC hDC,void* Param)
{
	CDVDSubEditDlg* DVDSubEdit = (CDVDSubEditDlg*) Param;
	// We simply call our ShowBorders function;
	DVDSubEdit->ShowBorders();
}

// The preview will call BitmapMixFunction both before and after the rescaling of 
// the video image. In all display modes we will mix the subtpics *after* the video image has been
// rescaled to the proper target size. This means we call our function when WichCall is 1.
extern void BitmapMixFunction(unsigned char* Bitmap, int Width, int Height, void* Param, int WhichCall)
{
	if(WhichCall != 1) return;
	static CSubPic* LastSubpic = 0;

	CDVDSubEditDlg* DVDSubEdit = (CDVDSubEditDlg*) Param;
	if(DVDSubEdit->m_Preview->IsVideoPlaying())
	{
		int VideoPTS = DVDSubEdit->m_Preview->GetCurrentPTS();
		int VobID, CellID;
		DVDSubEdit->m_Preview->GetVobCellID(VobID, CellID);

//		TRACE("VideoPTS = %d (frame %d)\n",VideoPTS,VideoPTS/3003);
		int Index;
		CSubPic* DisplayThis = DVDSubEdit->FindSubpicForPTS(VideoPTS,VobID, CellID, &Index);
		if(DisplayThis) 
		{
//			TRACE("   VideoPTS = %d (frame %d) SUB PTS %d\n",VideoPTS,VideoPTS/3003,DisplayThis->m_PTS);
			// Now would be a good time to only select that stream!
			DVDSubEdit->m_SelectedSubPic = DisplayThis;
			DisplayThis->DecodeSubpic();
			DVDSubEdit->DisplayBitmap(Bitmap, Width, Height);
			if(LastSubpic != DisplayThis)
			{
				DVDSubEdit->m_SPUSlider.SetPos(Index);
				DVDSubEdit->ViewSPU(Index,!DVDSubEdit->m_ShowVideo.GetCheck(),1);
			}
			LastSubpic = DisplayThis;
		}
		DVDSubEdit->m_LastDispLBA = -1;
	}
	else
	{
		DVDSubEdit->DisplayBitmap(Bitmap, Width, Height);
		LastSubpic = 0;
	}
}



/////////////////////////////////////////////////////////////////////////////
// CDVDSubEditDlg message handlers

#define LOADCOLOR(Array,Col){*(Array)=GetRValue(Col);*(Array+1)=GetGValue(Col);*(Array+2)=GetBValue(Col);}

BOOL CDVDSubEditDlg::OnInitDialog()
{
	CDialog::OnInitDialog();
if(Annoy) AfxMessageBox("OnInitDialog1");

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

    m_Menu.LoadMenu (IDR_MENU);
	SetMenu(&m_Menu);

	
if(Annoy) AfxMessageBox("OnInitDialog2");
	// TODO: Add extra initialization here
	DragAcceptFiles();
	EnableToolTips(true);

	// Make a note of initial window width and height.
	CRect TempRect;
	GetWindowRect(&TempRect);
	m_InitWindowWidth = TempRect.Width();
	m_InitWindowHeight = TempRect.Height();

if(Annoy) AfxMessageBox("OnInitDialog3");
	int Resize = TheSettings.LoadSettings("NoSize",0);
	if(!Resize)
		TheSettings.LoadSettings(m_hWnd,"MainWindowPos",0);
	
if(Annoy) AfxMessageBox("OnInitDialog4");
	m_QuitNow = 0;
	m_Running = 0;
	m_ThreadHandle = 0;
	strcpy(Message,"");
	m_ScanDialog = new CScanDlg;
	m_ScanDialog->CDialog::Create(IDD_SCANDIALOG);
	m_ScanDialog->m_Engine = this;
	m_SubpicAtStartPlayback = -1;
	TheSettings.LoadSettings(m_ScanDialog->m_hWnd,"ScanWindowPos",1);

	// Load memory of last directories used for loading/saving.
	TheSettings.LoadSettings(m_LastPPMDir,"LastPPMDir","");
	TheSettings.LoadSettings(m_LastOpenSUPDir,"LastSUPDir","");
	TheSettings.LoadSettings(m_LastSaveSUPDir,"LastSaveSUPDir","");
	TheSettings.LoadSettings(m_LastVOBDir,"LastVOBDir","");
	TheSettings.LoadSettings(m_LastSRTDir,"LastSRTDir","");

	// Load border settings.
	// 480*(1- 1/(OrigAR)*16/9)/2 .. Safe values, add 10% of target size...
	TheSettings.LoadSettings(&m_Border4_3S_N,"Border4_3S_N",48);
	TheSettings.LoadSettings(&m_Border4_3S_P,"Border4_3S_P",58);
	TheSettings.LoadSettings(&m_Border4_3LB_N,"Border4_3LB_N",60);
	TheSettings.LoadSettings(&m_Border4_3LB_P,"Border4_3LB_P",72);
	TheSettings.LoadSettings(&m_Border4_3LBS_N,"Border4_3LBS_N",89);
	TheSettings.LoadSettings(&m_Border4_3LBS_P,"Border4_3LBS_P",106);
	TheSettings.LoadSettings(&m_Border1_85_N,"Border1_85_N",10);
	TheSettings.LoadSettings(&m_Border1_85_P,"Border1_85_P",12);
	TheSettings.LoadSettings(&m_Border2_39_N,"Border2_39_N",62);
	TheSettings.LoadSettings(&m_Border2_39_P,"Border2_39_P",74);
	TheSettings.LoadSettings(&CSubPic::m_ForceEvenMove,"ForceEvenMoves",0);
	TheSettings.LoadSettings(&m_LastBitmapSaveType,"BitmapSaveType",0);

	TheSettings.LoadSettings(&m_AutoOpenVideo,"AutoForceDisplayVideo",1);
	TheSettings.LoadSettings(&m_WarnIfOnly4_3,"WarnIfOnly4_3",0);

	TheSettings.LoadSettings(&m_NumMRU,"NumMRU",0);
	UpdateMRUMenu();

	// Create all dialogs.
	m_ApplyAllDialog = new CApplyToAllDlg;
	m_ApplyAllDialog->m_Engine = this;
	m_MoveDialog = new CMoveDlg;
	m_MoveDialog->m_Engine = this;
	m_SetupDialog = new CSetupDlg(true); m_SetupDialog->m_Engine = this;
	char* BMPEditPath;
	BMPEditPath = TheSettings.LoadSettings("BMPEditPath","");
	m_SetupDialog->m_BitmapEditTool = BMPEditPath;
	BMPEditPath = TheSettings.LoadSettings("BMPEditPathArgs","");
	m_SetupDialog->m_BitmapEditToolArgs = BMPEditPath;
	m_ZoomDialog = new CDialog(IDD_ZOOMDLG,this);
	m_ZoomDialog->Create(IDD_ZOOMDLG);
	m_ZoomDialog->SetWindowText("Bitmap zoom");
	// Remember positions.
	TheSettings.LoadSettings(m_ApplyAllDialog->m_hWnd,"ApplyAllWindowPos",1);
	TheSettings.LoadSettings(m_ZoomDialog->m_hWnd,"ZoomWindowPos",1);

	// Load OCR settings.
	TheSettings.LoadSettings(&m_SetupDialog->m_Recenter,"OCRRecenter",1);
	TheSettings.LoadSettings(&m_SetupDialog->m_AlignBottom,"OCRAlignBottom",1);
	TheSettings.LoadSettings(&m_SetupDialog->m_BracketSquare,"OCRBracketSquare",1);
	TheSettings.LoadSettings(&m_SetupDialog->m_BracketParen,"OCRBracketParen",1);
	TheSettings.LoadSettings(&m_SetupDialog->m_BracketNotes,"OCRBracketNotes",0);
	TheSettings.LoadSettings(&m_SetupDialog->m_Italics,"OCRItalics",1);
	TheSettings.LoadSettings(&m_SetupDialog->m_RemoveColumns,"OCRRemoveColumns",0);
	TheSettings.LoadSettings(&m_SetupDialog->m_SpaceSelect,"OCRSpaceSelect",0);
	TheSettings.LoadSettings(&m_SetupDialog->m_CaseSensitive,"OCRCaseSensitive",0);
	TheSettings.LoadSettings(&m_SetupDialog->m_CropBorderPixelsX,"CropBackgroundBorderX",1);
	TheSettings.LoadSettings(&m_SetupDialog->m_FPS,"FrameDownSampling",0);
	m_Preview->SetFrameDownSample(m_SetupDialog->m_FPS);
	CSubPic::m_CropBackBorderX = m_SetupDialog->m_CropBorderPixelsX;
	TheSettings.LoadSettings(&m_SetupDialog->m_CropBorderPixelsY,"CropBackgroundBorderY",1);
	CSubPic::m_CropBackBorderY = m_SetupDialog->m_CropBorderPixelsY;
	// This is needed so the setups above are reflected in the brackets used.
	m_SetupDialog->SetBrackets();
	TheSettings.LoadSettings(&m_SetupDialog->m_AutoDisplay,"AutoDisplayMode",1);

	// Setting for closed captions.
	TheSettings.LoadSettings(&m_AlertIfCC,"AlertIfClosedCaptions",0);
	m_SetupDialog->m_AlertClosedCaptions = m_AlertIfCC;

	// Misc settings.
//	TheSettings.LoadSettings(&m_DisplayVideo,"ShowVideo",1);
//	m_ShowVideo.SetCheck(m_DisplayVideo);
	m_ShowVideo.EnableWindow(false);
	TheSettings.LoadSettings(&m_SetupDialog->m_AskForIFO,"AskForIFOs",1);

	TheSettings.LoadSettings(&m_SetupDialog->m_RetimeSliderRange,"RetimingSliderRange",10.0);
	m_MoveDialog->m_SliderRangeS = m_SetupDialog->m_RetimeSliderRange;

	TheSettings.LoadSettings(&m_SetupDialog->m_EditBitmapAs24,"EditBitmapAs24BitBMP",0);

	m_ZoomSelect.SetCurSel(m_RealSizeDisplay);
	
	// Load default background color.
	TheSettings.LoadSettings((int*)&m_SetupDialog->m_DefBackgroundColor,"DefaultBackgdColor",(long)RGB(50,50,50));
	LOADCOLOR(CSubPic::m_BackColor,m_SetupDialog->m_DefBackgroundColor);
	// Load default "auto-clut" colors.
	TheSettings.LoadSettings((int*)&m_DefAutoPixColor,"DefaultAutoPixColor",(long)RGB(255,255,255));
	LOADCOLOR(CSubPic::m_DefColors,m_DefAutoPixColor);
	TheSettings.LoadSettings((int*)&m_DefAutoBckgColor,"DefaultAutoBkgColor",(long)RGB(0,0,0));
	LOADCOLOR(CSubPic::m_DefColors+3,m_DefAutoBckgColor);
	TheSettings.LoadSettings((int*)&m_DefAutoAA1Color,"DefaultAutoAA1Color",(long)RGB(0,0,0));
	LOADCOLOR(CSubPic::m_DefColors+6,m_DefAutoAA1Color);
	TheSettings.LoadSettings((int*)&m_DefAutoAA2Color,"DefaultAutoAA2Color",(long)RGB(255,255,255));
	LOADCOLOR(CSubPic::m_DefColors+9,m_DefAutoAA2Color);

	CRect Rect;
	GetDlgItem(IDC_VIDEOBACK)->GetWindowRect(&Rect);
	m_DisplayHeight = Rect.Height();

	// This will simply erase the display.
	JustFoundVideoFormat();
	Reset();

	// Bitmap structure used to display the subpics.
	ZeroMemory(&m_BitMapHeader, sizeof(BITMAPINFOHEADER));
	m_BitMapHeader.biSize = sizeof(BITMAPINFOHEADER);
	m_BitMapHeader.biPlanes = 1;
	m_BitMapHeader.biBitCount = 24;
	m_BitMapHeader.biCompression = BI_RGB;

	// Create preview.
	m_Preview = new CDVDPreview;
	m_Preview->SetDrawButtons(true);
	// This is to conserve memory.
	m_Preview->SetMaxVideoScaling(1.5);
	m_PreviewHandle = m_Preview->InitPreview(m_hWnd,1,1,100,100,SW_HIDE,true,WS_CHILD,true);

	// Now setup the preview so our PostVideoDisplayFunc function is called after each drawn video
	// frame...
	m_Preview->SetBitmapMixFunction(BitmapMixFunction,this);
	m_Preview->SetPostDisplayFunc(PostVideoDisplayFunc,this);

	m_Play.SetIcon(AfxGetApp()->LoadIcon(IDI_PLAY));
	m_Loop.SetIcon(AfxGetApp()->LoadIcon(IDI_LOOP));
	EnableUI(false);

	// Set path to log file.
	if(getenv("TEMP") != 0)
		sprintf(m_LogPath,"%s\\DVDSubEdit.log",getenv("TEMP"));
	OutLogFd = 0;

#if 0
	// This would be for profiling the OCR code.
//	LoadSUPFile("E:/2_TESTVOBS/in_her_shoes.sup");
	LoadSUPFile("C:/temp/VIDEOS/in_her_shoes_engsubs_r1_first_vob_only.sup");
	int k;
	for(k=0;k<40;k++)
	{
		m_SelectedSubPic = m_MasterList[k];
		m_SelectedSubPic->DecodeSubpic();
	OCRThisOne();
	}
	exit(0);
#endif

	m_ShowVideo.SetCheck(0);m_DisplayVideo=0;

	m_CLUTSelect.SetCurSel(0);
	if(m_CMDInputFile[0] != NULL)
	{
		// Open file passed as argument...
		if(strcmpi(GetExt(m_CMDInputFile[0]),".sup") != 0 && strcmpi(GetExt(m_CMDInputFile[0]),".vob") != 0)
		{
			ERROR1("Error, DVDSubEdit can only open sup or vob files");
		}
		else if(m_CMDOpenFullDomain && strcmpi(GetExt(m_CMDInputFile[0]),".sup") != 0)
			OpenDomain(m_CMDInputFile[0]);
		else
		{
			Reset();
			if(strcmpi(GetExt(m_CMDInputFile[0]),".sup") == 0)
			{
				LoadSUPFile(m_CMDInputFile[0]);
				OnSelchangeStreamselect();
				EnableUI(true);
				SetWindowText(m_CMDInputFile[0]);
				m_CLUTSelect.SetCurSel(1);
			}
			else
			{
				int j;
				for(j=0;j<10 && m_CMDInputFile[j] != 0;j++)
				{
					AddFile(m_CMDInputFile[j]);
				}
				EnableUI(true);
				// I need to do that so preview indeed opens the first file!
				SortInputFiles();
				// Now scan input file...
				m_ScanDialog->DisplayInfo(0,"","");
				m_ScanDialog->ShowWindow(SW_SHOW);
				if(m_AutoOpenVideo) {m_ShowVideo.SetCheck(1);m_DisplayVideo=1;}
				::ShowWindow(m_PreviewHandle,SW_HIDE);
				StartScan();
				m_Preview->LoadFile(m_FileInfos[0].m_FileName);
				strcpy(m_LastVOBDir,m_CMDInputFile[0]);
			}
		}
	}

	return TRUE;  // return TRUE  unless you set the focus to a control
}

// IMPORTANT: Use a floor() not a round, otherwise, the sucpics might go beyond the video (which uses a 
// floor, not a round).
#define RESCALEVIDEOX(x) ((int)((x)*m_VideoScalingX+0.0f))
#define RESCALEVIDEOY(x) ((int)((x)*m_VideoScalingY+0.0f))

void CDVDSubEditDlg::JustFoundVideoFormat(bool redraw)
{
	GetDlgItem(IDC_VIDEOBACK)->GetWindowRect(&m_VideoRect);
	ScreenToClient(&m_VideoRect);
	int VideoHeight = m_CodedHeight;

	// In case the display mode hasn't been selected, select something that makes sense.
	if(m_DispModeSelect.GetCurSel() == -1)
	{
		if(m_AspectRatio == 1 || m_AspectRatio == 2) 
			m_DispModeSelect.SetCurSel(0) ;
		else m_DispModeSelect.SetCurSel(1);
		m_WSDisplayMode = WSDispNormal;
		m_DispModeSelect.RedrawWindow();
	}

	switch(m_AspectRatio)
	{
	case 1: m_VideoScalingX = 1; break;
	case 2: m_VideoScalingX = 4.f/3/m_CodedWidth*VideoHeight; break;
	case 3: m_VideoScalingX = 16.f/9/m_CodedWidth*VideoHeight; break;
	case 4: m_VideoScalingX = 2.21f/m_CodedWidth*VideoHeight; break;
	}
	// Tricky! We need to make sure both the width and the height fit our window!
	m_VideoScalingY = min((float)m_DisplayHeight / VideoHeight, (float)m_VideoRect.Width() / (m_VideoScalingX*m_CodedWidth));
	m_VideoScalingX *= m_VideoScalingY;

	// Calculate borders needed to center the video in m_VideoRect.
	int temp = m_VideoRect.Width() - RESCALEVIDEOX(m_CodedWidth);
	m_VideoRect.left += temp/2;
	m_VideoRect.right -= temp/2;
	m_VideoRect.bottom = m_VideoRect.top + RESCALEVIDEOY(VideoHeight);

	if(!m_NoVideo)
	{
		// Set video display mode, scaling and upper left position of video display.
		m_Preview->SetWSDisplayMode((int)m_WSDisplayMode);
		m_Preview->SetVideoScaling(m_VideoScalingX,m_VideoScalingY,0);
		m_Preview->SetVideoPosition(m_VideoRect.left,m_VideoRect.top,0);
	}
	// The flickering happens if you erase the background and there's no video. If you don't erase
	// no flicker if on video. Flickering IF video.
	// Question: I disabled the drawing in store.c but still when I resize, a rectangle is filled
	// with black when showvideo is on! HOW CAN THIS BE!!!! NOT in displaybitmap at least.
	// Not, this happens in ResizeWindow in gui.cpp. The MoveWindow() somehow must paint the window
	// background black, but I'm not sure how.
	// OK: The flickering ONLY OCCURS when accessing a new video frame! If it's the same
	// video frame, it's immediate, even when rescaling etc...
//	if(redraw) InvalidateRect(&m_VideoRect);
	if(redraw) InvalidateRect(0);
	GetDlgItem(IDC_SHOWVIDEO)->EnableWindow(!m_NoVideo);
}

void CDVDSubEditDlg::ShowCol(HDC TheDC, uint ID, uchar* Colormap)
{
	// This shows the colors currently used for each pixels.
	CRect Temp;
	GetDlgItem(ID)->GetWindowRect(&Temp);
	ScreenToClient(&Temp);
	HBRUSH Color = (HBRUSH)CreateSolidBrush(RGB(Colormap[0],Colormap[1],Colormap[2]));
	FillRect(TheDC,Temp,Color);
	HBRUSH Black = (HBRUSH)CreateSolidBrush(BLACK);
	FrameRect(TheDC,&Temp,Black);
	// Don't forget this! If you don't release these resources, the app crashes after a while!
	DeleteObject(Color);
	DeleteObject(Black);
}

uchar* CDVDSubEditDlg::GetCLUT()
{
	uchar* CLUT = 0;
	if(m_SelectedSubPic == 0 || m_CLUTSelect.GetCurSel() == 1) return 0;
	CLUT = FindCLUT(m_SelectedSubPic->m_VOBID,m_SelectedSubPic->m_CELLID);
	if(!CLUT) {CLUT = m_SelectedSubPic->m_CLUT; if(!CLUT) m_CLUTSelect.SetCurSel(1);}
	return CLUT;
}

void CDVDSubEditDlg::ShowBorders()
{
	// Displays the top and bottom borders, the last display step.
	if(m_SelectedSubPic == NULL)
		return;

	// Now draw the borders if needed.
	if(m_SelectedSubPic->m_VerticalBorders != 0)
	{
		CDC *TheDC = GetDC();
		HDC hDC = TheDC->m_hDC;
		HPEN hpen;
		hpen = CreatePen(PS_SOLID, 1, RGB(255, 0, 0));
		SelectObject(hDC, hpen);
		SelectObject(hDC, GetStockObject(HOLLOW_BRUSH)); 
		POINT Points[2];
		int VBorder = RESCALEVIDEOY(m_SelectedSubPic->m_VerticalBorders);
		Points[0].x = m_VideoRect.left; Points[1].x = m_VideoRect.right-1; 
		Points[0].y = m_VideoRect.top+VBorder; Points[1].y = m_VideoRect.top+VBorder;
		Polyline(hDC,Points,2);
		Points[0].y = m_VideoRect.bottom-VBorder-1; Points[1].y = m_VideoRect.bottom-VBorder-1;
		Polyline(hDC,Points,2);
		DeleteObject(hpen);
		ReleaseDC(TheDC);
	}
}


void CDVDSubEditDlg::DisplayBitmap(unsigned char* Bitmap, long Width, long Height, bool Black)
{
	CDC *TheDC = GetDC();
//	if(!TheDC) return;	// Should NEVER happen!
	HDC hDC = TheDC->m_hDC;

	// I'm keeping a local copy here to avoid m_SelectedSubPic becoming 0 under me, which sometimes happens,
	// when the video display is on, and I've no idea why >:(
	CSubPic* SelectedSubPic = m_SelectedSubPic;
	if(Black || SelectedSubPic == 0)
	{
		// Simply erasing the video area.
		CRect Temp = m_VideoRect;
		// If I paint all the way to the right, the video might not cover the black and that's ugly.
		Temp.right --;
		HBRUSH Black = (HBRUSH)CreateSolidBrush(m_SetupDialog->m_DefBackgroundColor);
		FillRect(hDC, &Temp, Black);
		DeleteObject(Black);
		ReleaseDC(TheDC);
		return;
	}

	if(m_ShowHighlights.GetCheck()) CSubPic::m_PixTempAlpha |= 0x10;
	else CSubPic::m_PixTempAlpha &= 0x0F;

	// Determine the CLUT, language and display mode(s) for this subpic.
	uchar* CLUT = 0;
	char Lang[32][16];
	uchar DispModes[32];
	CLUT = FindCLUT(SelectedSubPic->m_VOBID,SelectedSubPic->m_CELLID,Lang,DispModes);
	if(CLUT && !m_Preview->IsVideoPlaying())
	{
		bool Only4_3 = true;
		// Set language strings...
		m_StreamSelect.ResetContent();
		m_StreamSelect.AddString("All Streams");
		int j;
		for(j=0;j<=m_TopStreamID;j++)
		{
			CString Temp = Lang[j];
			Temp += " ";
			int k,b,l;
			char* DS[4]; DS[0]="4:3";DS[1]="WS";DS[2]="LB";DS[3]="P&S";
			for(k=l=0,b=1;k<4;k++,b*=2)
			{
				if(DispModes[j] & b) 
				{
					if(l) Temp += "+";
					Temp+=DS[k];
					l=1;
					if(k>0) Only4_3 = false;
				}
			}
			m_StreamSelect.AddString(Temp);
		}
		m_StreamSelect.SetCurSel(m_SelectedStream);
		if(Only4_3 && m_WarnIfOnly4_3 != 0 && m_4_3WarnedAlready == false)
		{
			m_4_3WarnedAlready = true;
			AfxMessageBox("Warning! This VTS can only be displayed as a 4:3 aspect ratio");
		}

		// Audio.
		static PGC_CLUT_Info* LastClutInf = NULL;
		PGC_CLUT_Info* ClutInf = FindCLUTInfo(SelectedSubPic->m_VOBID,SelectedSubPic->m_CELLID);
		if(ClutInf != LastClutInf)
		{
			int PrevSel = m_AudioStreamSel.GetCurSel();
			m_AudioStreamSel.ResetContent();
			if(ClutInf)
			{
				for(j=0;j<ClutInf->m_NumAudioStreams;j++)
				{
					uchar StreamID = ClutInf->m_AudioStreamIDs[j];
					// Preview can only play AC3 and lpcm.
					if((StreamID < 0x80 || StreamID >= 0x88) && (StreamID < 0xA0 || StreamID >= 0xA8)) continue;
					char Temp[64];
					sprintf(Temp,"0x%x (%s)",ClutInf->m_AudioStreamIDs[j],ClutInf->m_AudioLang[j]);
					m_AudioStreamSel.AddString(Temp);
					m_AudioStreamSel.SetItemData(m_AudioStreamSel.GetCount()-1,ClutInf->m_AudioStreamIDs[j]);
				}
				m_AudioStreamSel.EnableWindow(ClutInf->m_NumAudioStreams > 0);
			}
			m_AudioStreamSel.SetCurSel(PrevSel == -1 ? 0 : PrevSel);
			LastClutInf = ClutInf;
		}
	}
	// This will handle all cases (the clut selectors, whether or not IFOs are present etc.
	CLUT = GetCLUT();

	if(Bitmap != 0)
	{
		// Mix subpic into video bitmap with transparency
		m_ColorMap = SelectedSubPic->MixScaleIntoBitmap(Bitmap, Width, Height,CLUT, 0, 0, m_WSDisplayMode);
	}
	else
	{
		// Not mixing into an existing bitmap. We have to create one, and then display it.
		// FIX THIS: shouldn't allocate and deallocate!
		long NewWidth = 0;
		long NewHeight = 0;
		int ScaledWidth, ScaledHeight,ScaledStartX, ScaledStartY;
		uchar* TempBitmap;
		ScaledWidth = 4+RESCALEVIDEOX(SelectedSubPic->m_SubPicWidth) ;
		ScaledHeight = 1+RESCALEVIDEOY(SelectedSubPic->m_SubPicHeight);
		ScaledStartX = m_VideoRect.left+RESCALEVIDEOX(SelectedSubPic->m_StartX);
		ScaledStartY = m_VideoRect.top+RESCALEVIDEOY(SelectedSubPic->m_StartY);	
		// Create bitmap, save subpic into it.
		TempBitmap = new uchar[ScaledWidth*ScaledHeight*3];
		m_ColorMap = SelectedSubPic->SaveToBitmap(TempBitmap, NewWidth, NewHeight, 1/m_VideoScalingX, 1/m_VideoScalingY, CLUT, 0, 0, m_WSDisplayMode);
		m_BitMapHeader.biWidth = NewWidth;
		m_BitMapHeader.biHeight = NewHeight;
		m_BitMapHeader.biSizeImage = m_BitMapHeader.biWidth * m_BitMapHeader.biHeight * m_BitMapHeader.biBitCount / 8;
		// Erase video screen,
		HBRUSH Black = (HBRUSH)CreateSolidBrush(m_SetupDialog->m_DefBackgroundColor);
		FillRect(hDC, &m_VideoRect, Black);
		DeleteObject(Black);
		// And display bitmap.
		SetDIBitsToDevice(hDC, ScaledStartX, ScaledStartY, NewWidth, NewHeight,
			0, 0, 0, NewHeight, TempBitmap, (LPBITMAPINFO)&m_BitMapHeader, DIB_RGB_COLORS);
		delete[] TempBitmap;
		// Now show borders.
		ShowBorders();
	}
	if(m_RealSizeDisplay && m_ZoomDialog->IsWindowVisible())
	{
		// This shows the subpic in 1:1 magnification.
		long NewWidth = 0;
		long NewHeight = 0;
		uchar* TempBitmap;
		// Using a factor 4 because the final bitmap can be larger than m_SubPicWidth*m_SubPicHeight*3.
		float InvZoom = m_RealSizeDisplay == 2 ? .5f : 1;
		TempBitmap = new uchar[(SelectedSubPic->m_SubPicWidth+4)*(SelectedSubPic->m_SubPicHeight+4)*3/InvZoom/InvZoom];
		m_ColorMap = SelectedSubPic->SaveToBitmap(TempBitmap, NewWidth, NewHeight, InvZoom, InvZoom, CLUT, 0,true, m_WSDisplayMode);
		// Adjust the window to exactly fit the size of the bitmap in its client area.
		RECT WinRect; m_ZoomDialog->GetWindowRect(&WinRect);
		RECT CliRect; m_ZoomDialog->GetClientRect(&CliRect); m_ZoomDialog->ClientToScreen(&CliRect);
		int TopBotBorder = (CliRect.top - WinRect.top) + (WinRect.bottom - CliRect.bottom);
		int LeftBorder = CliRect.left - WinRect.left;
		// Resize window, get DC, etc...
		m_ZoomDialog->SetWindowPos(NULL,0,0,NewWidth+2*LeftBorder,NewHeight+TopBotBorder,SWP_NOMOVE|SWP_NOOWNERZORDER|SWP_NOACTIVATE );
		HDC hDC2 = m_ZoomDialog->GetDC()->m_hDC;
		m_BitMapHeader.biWidth = NewWidth;
		m_BitMapHeader.biHeight = NewHeight;
		m_BitMapHeader.biSizeImage = m_BitMapHeader.biWidth * m_BitMapHeader.biHeight * m_BitMapHeader.biBitCount / 8;
		static RECT LastRect = {0,0,0,0};
		RECT TempRect;
		m_ZoomDialog->GetClientRect(&TempRect);
		if(TempRect.bottom != LastRect.bottom || TempRect.right != LastRect.right)
		{
			// Only erase the background if the window changed size. This improves flickering.
			HBRUSH Black = (HBRUSH)CreateSolidBrush(m_SetupDialog->m_DefBackgroundColor);
			FillRect(hDC2, &TempRect, Black);
			DeleteObject(Black);
		}
		LastRect = TempRect;
		// Now display our bitmap.
		SetDIBitsToDevice(hDC2, 0, 0, NewWidth, NewHeight, 0, 0, 0, NewHeight, TempBitmap, (LPBITMAPINFO)&m_BitMapHeader, DIB_RGB_COLORS);
		::ReleaseDC(m_ZoomDialog->m_hWnd,hDC2);
		delete[] TempBitmap;
	}
	// In case the zoom window was closed. This is ugly though...
	else m_RealSizeDisplay = 0;
	if(!m_RealSizeDisplay && !m_Preview->IsVideoPlaying())
		m_ZoomSelect.SetCurSel(0);

	// Show current CLUT colors... 
	ShowCol(hDC, IDC_COLB,m_ColorMap); ShowCol(hDC,IDC_COLP,m_ColorMap+3);
	ShowCol(hDC, IDC_COLE1,m_ColorMap+6); ShowCol(hDC,IDC_COLE2,m_ColorMap+9);
	ReleaseDC(TheDC);
}

void CDVDSubEditDlg::Reset()
{
	// This is called everytime a new set of vobs or a sup file is loaded.
	CSubPicEngine::Reset();
	SetWindowText(m_Version);
	m_SelectedStream = 0;
	m_PercentDone = 0;
	m_NumOCRLines = 0;
	strcpy(m_SearchStr,"");
	m_CommandDisplay.SetWindowText("");
	m_OCROut.SetWindowText("");
	m_StreamSelect.ResetContent();
	m_StreamSelect.AddString("All Streams");
	m_StreamSelect.SetCurSel(0);
	m_VCIDSelect.ResetContent();
	m_VCIDSelect.AddString("All VCID");
	m_VCIDSelect.SetCurSel(0);
	m_SPUSlider.SetRange(0,0);
	m_VideoScalingY = m_VideoScalingX = 1;
	m_SPUSlider.ShowWindow(SW_HIDE);
	m_IFrameSelect.ShowWindow(SW_HIDE);
	m_MoveVSlider.SetRange(0,MAX_V_MOVE*2);
	m_MoveHSlider.SetRange(0,MAX_H_MOVE*2);
	m_TransSlider.SetRange(0,15);
	m_BorderSlider.SetRange(0,MAX_BORDER);
	m_DurationSlider.SetRange(0,MAX_DURATION);
	m_FlushBorders.SetCheck(0);
	m_ShowModified.SetCheck(0);
	m_ShowSearched.SetCheck(0);
	m_ShowForced.SetCheck(0);
	m_ShowTransparent.SetCheck(0);
	m_DispModeSelect.SetCurSel(-1);
	SetBorderSelect(-1);
	SetMoveVSlider(MAX_V_MOVE);
	m_PixelSelect.SetCurSel(4);
	m_ApplyAllDialog->ResetAllCheck();
	DisplayBitmap(0,0,0);
	GetDlgItem(IDC_REVERTALL)->EnableWindow(false);
	GetDlgItem(IDC_REVERTSAVED)->EnableWindow(false);
	m_MenuEnabled = true;
	m_Preview->CloseFile();
	strcpy(m_DefIFOPath,"");
	m_WSDisplayMode = WSDispNormal;
	m_NextSPULba = 0;
	m_LastDispLBA = -1;
	m_ShowEndFrame.SetCheck(false);
	m_ColorMap = 0;
	GetDlgItem(IDC_SEARCHSELECT)->SetWindowText("");
	m_4_3WarnedAlready = false;

	m_ChangeRecord.Reset();
}


// Found here: http://www.codeguru.com/Cpp/controls/richedit/article.php/c2413/
// Function used to display the SPU info. It's much much faster than using MFC calls.
static DWORD CALLBACK EditStreamCallBack(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb)
{
	CString *pstr = (CString *)dwCookie;
	if( pstr->GetLength() < cb )
	{
		*pcb = pstr->GetLength();
		memcpy(pbBuff, (LPCSTR)*pstr, *pcb );
		pstr->Empty();
	}
	else
	{
		*pcb = cb;
		memcpy(pbBuff, (LPCSTR)*pstr, *pcb );
		*pstr = pstr->Right( pstr->GetLength() - cb );
	}
	return 0;
}

CString rtfString;
void CDVDSubEditDlg::DispInfo(char* Info, int Color)
{
	CString Temp; 
	if(Info[1] == 'C')
		Temp.Format("{\\ul\\cf%d %s}",Color,Info);
	else
		Temp.Format("\\cf%d %s",Color,Info);
	rtfString += Temp;
	rtfString += "\\par ";
}

char* PrintPTS(int PTS, int durMs)
{
	static char Temp[128];
	float Seconds1 = (float)PTS/(VideoFormat == FORMAT_PAL ? PAL_TICS_PER_SEC : NTSC_TICS_PER_SEC);;
	float Seconds2 = (float)Seconds1+(float)durMs/1000;
	if(durMs >= 0)
		sprintf(Temp,"%02d:%02d:%02d,%03d --> %02d:%02d:%02d,%03d",
			(int) Seconds1 / 3600,
			((int) Seconds1 % 3600)/60,
			((int) Seconds1 % 3600)%60,
			(int)((Seconds1-(int)Seconds1)*1000),
			(int) Seconds2 / 3600,
			((int) Seconds2 % 3600)/60,
			((int) Seconds2 % 3600)%60,
			(int)((Seconds2-(int)Seconds2)*1000));
	else
		sprintf(Temp,"%02d:%02d:%02d,%03d --> ?",
			(int) Seconds1 / 3600,
			((int) Seconds1 % 3600)/60,
			((int) Seconds1 % 3600)%60,
			(int)((Seconds1-(int)Seconds1)*1000));
	return Temp;
}

void CDVDSubEditDlg::ViewSPU(int SPUIndex, bool RefreshVideo, bool RefreshCommands)
{
	static int LastSPU = -1;
	if(SPUIndex < m_PruneLength)
	{
		char Temp[1024*16];
		int SelectedPixel;
		m_SelectedSubPic = m_PrunedList[SPUIndex];
if(m_SelectedSubPic && m_SetupDialog->m_AutoDisplay)
{
	AdjustDisplayMode();
}
		// I'm keeping a local copy here to avoid m_SelectedSubPic becoming 0 under me, which sometimes happens,
		// when the video display is on, and I've no idea why >:(
		CSubPic* SelectedSubPic = m_SelectedSubPic;
		ASSERT(SelectedSubPic != 0);
		if(RefreshCommands)
		{
			// Now show commands!
			// http://netghost.narod.ru/gff/graphics/summary/micrtf.htm
			CString rtfPrefix, rtfPostfix;
			rtfPrefix = "{\\rtf1\\ansi\\deff0\\deftab720{\\fonttbl{\\f0\\froman "
//				"Times New Roman;}}\n{\\colortbl\\red0\\green0\\blue0;\\red255\\green0\\blue0;\\red0\\green134\\blue37;\\red0\\green0\\blue255;}\n"
				"Tahoma;}}\n{\\colortbl\\red0\\green0\\blue0;\\red255\\green0\\blue0;\\red0\\green134\\blue37;\\red0\\green0\\blue255;}\n"
				"\\deflang1033\\pard\\tx360\\tx720\\tx1080\\tx1440\\tx1800"
				"\\tx2160\\tx2520\\tx2880\\tx3240\\tx3600\\tx3960\\tx4320"
				"\\tx4680\\tx5040\\tx5400\\tx5760\\tx6120"
				"\\tx6480\\plain\\f3\\fs20 ";
			rtfString = rtfPrefix ;
			// Display various info.
			sprintf(Temp,"\\fs16\\b\\ul Subpic# = %d (%d in stream)\\b0\\ul0\n",SelectedSubPic->m_SPUNumber,SelectedSubPic->m_SPUNumberInStream); DispInfo(Temp,BLUE);
			sprintf(Temp,"StreamID = 0x%x\n",SelectedSubPic->m_StreamID+0x20); DispInfo(Temp,BLUE);
			sprintf(Temp,"LBA = %d\n",SelectedSubPic->m_LBAFrom[0]); DispInfo(Temp,BLUE);
			sprintf(Temp,"V/CID = %d/%d\n",SelectedSubPic->m_VOBID,SelectedSubPic->m_CELLID); DispInfo(Temp,BLUE);
			sprintf(Temp,"PTS = %d\n",SelectedSubPic->m_PTS); DispInfo(Temp,BLUE);
			if(SelectedSubPic->m_DCSQT.size() == 1)
			{
				SetDurationSlider(0);
				sprintf(Temp,"Duration = infinite\n"); DispInfo(Temp,BLUE);
				sprintf(Temp,"TimeCode: %s\n",PrintPTS(SelectedSubPic->m_PTS,-1)); DispInfo(Temp,BLUE);
			}
			else
			{
				int DurationMs = SelectedSubPic->GetDCSQTDelay();
				SetDurationSlider(DurationMs);
				sprintf(Temp,"Duration = %.3fs\n",(float)DurationMs/1000); DispInfo(Temp,BLUE);
				sprintf(Temp,"TimeCode: %s\n",PrintPTS(SelectedSubPic->m_PTS,DurationMs)); DispInfo(Temp,BLUE);
			}
			// Find LBA of I-frame corresponding to next subpic.
			CSubPic* Next = FindNextSPUInStream(SelectedSubPic->m_SPUNumber);
			if(Next)
				m_NextSPULba = Next->m_NavPackLBA;
			SetIFrameSlider(SelectedSubPic);
			if(LastSPU != SPUIndex)
			{
				CSubPic::m_SelButton = 0;
				m_IFrameSelect.SetPos(0);
			}
			LastSPU = SPUIndex;
			if(SelectedSubPic->m_Modified)
				DispInfo("Modified\n",GREEN);
			// Now display all commands.
			while(RetreiveNextCommand(Temp) == 0)
				DispInfo(Temp,(Temp[0] != ' ')?RED:BLACK);
			rtfString += "\n\\par }";
			EDITSTREAM es = {(DWORD)&rtfString, 0, EditStreamCallBack};
			m_CommandDisplay.StreamIn(SF_RTF, es);
		}
		// Now decode subpic "bitmap" and update the sliders.
		SelectedSubPic->DecodeSubpic();
		SetMoveVSlider(SelectedSubPic->m_LastVMove+MAX_V_MOVE);
		SetMoveHSlider(SelectedSubPic->m_LastHMove+MAX_H_MOVE);
		SelectedPixel = m_PixelSelect.GetCurSel();
		// The alphas are between 0 and 255;
		if(SelectedPixel < 4)
			SetTransSlider(SelectedSubPic->m_AlphaPtr[SelectedPixel]);
		else
			SetTransSlider(SelectedSubPic->GetMeanTransparency());
		// Start command.
		m_StartCommand.SetCurSel(SelectedSubPic->FindCommand(STA_DSP) != -1 ? 0 : 1);

		// Borders.
		m_BorderSlider.EnableWindow(!SelectedSubPic->m_NumButtons);
		m_BorderSelect.EnableWindow(!SelectedSubPic->m_NumButtons);
		GetDlgItem(IDC_BORDERB)->EnableWindow(!SelectedSubPic->m_NumButtons);

		SetBorderSlider(SelectedSubPic->m_VerticalBorders);
		if(SelectedSubPic->m_VerticalBorders == (VideoFormat == FORMAT_NTSC ? m_Border4_3LBS_N : m_Border4_3LBS_P))
			SetBorderSelect(BORDER_4_3_LS);
		else if(SelectedSubPic->m_VerticalBorders == (VideoFormat == FORMAT_NTSC ? m_Border4_3LB_N : m_Border4_3LB_P))
			SetBorderSelect(BORDER_4_3_L);
		else if(SelectedSubPic->m_VerticalBorders == (VideoFormat == FORMAT_NTSC ? m_Border4_3S_N : m_Border4_3S_P))
			SetBorderSelect(BORDER_SAFE);
		else if(SelectedSubPic->m_VerticalBorders == (VideoFormat == FORMAT_NTSC ? m_Border1_85_N : m_Border1_85_P))
			SetBorderSelect(BORDER_1_85_L);
		else if(SelectedSubPic->m_VerticalBorders == (VideoFormat == FORMAT_NTSC ? m_Border2_39_N : m_Border2_39_P))
			SetBorderSelect(BORDER_2_39_L);
		else if(SelectedSubPic->m_VerticalBorders)
			SetBorderSelect(BORDER_CUSTOM);
		else 
			SetBorderSelect(-1);

		// DCSQT:
		m_DCSQTSelect.ResetContent();
		for(int i=0;i<SelectedSubPic->m_DCSQT.size()-1;i++)
		{
			sprintf(Temp,"DCSQT %d",i);
			m_DCSQTSelect.AddString(Temp);
		}
		sprintf(Temp,"DCSQT %d",i);
		if(SelectedSubPic->m_DCSQT.back().m_Commands.size() > 2)
			m_DCSQTSelect.AddString(Temp);
		m_DCSQTSelect.AddString("All DCSQTs");
		m_DCSQTSelect.SetCurSel(!SelectedSubPic->m_ChangeAllDCSQT ? 
		SelectedSubPic->m_SelectedDCSQT : m_DCSQTSelect.GetCount()-1);

		// Enable revert button (no longer displayed...)
		m_Revert.EnableWindow(SelectedSubPic->m_Modified && !m_Running);

		// Enable horizontal move slider if possible.
		bool CanHMove = SelectedSubPic->CanHMove();
		m_MoveHSlider.EnableWindow(CanHMove);
		GetDlgItem(IDC_MOVELRE)->EnableWindow(CanHMove);
		GetDlgItem(IDC_MOVELRB)->EnableWindow(CanHMove);

		// If we're not refreshing the video, we're done.
		if(!RefreshVideo)
			return;
		// Display OCR output.
		DispOCR(SelectedSubPic);

		// Now's the time to display the video if needed, then the subpics.
		if(!m_DisplayVideo || m_NoVideo)
		{
			// No background video display. Simply display the bitmap.
			::ShowWindow(m_PreviewHandle,SW_HIDE);
			DisplayBitmap(0,0,0);
		}
		else
		{
			int DispLBA = SelectedSubPic->m_NavPackLBA;
			// If m_ShowEndFrame, we display the I-Frame right after the end of the subpic...
			if(m_ShowEndFrame.GetCheck())
//				DispLBA = DispLBA + m_DurationSlider.GetPos()  * (m_NextSPULba-DispLBA) / m_DurationSlider.GetRangeMax();
				m_IFrameSelect.SetPos(m_IFrameSelect.GetRangeMax());
			DispLBA = m_IFrameSelect.GetPos();
			// Only call GoToSector() if we need to, since this means decoding the mpeg frame etc.
			if(m_LastDispLBA != DispLBA)
			{
				// When showing the video, the preview is responsible for drawing
				// everything. ShowBorders() will be called after the frame is drawn.
				m_Preview->SetStopSector(-1);
				m_Preview->SetStopPTS(-1);
				m_Preview->GoToSector(DispLBA);
				m_Preview->WaitForThread();
				m_LastDispLBA = DispLBA;
			}
			else
			{
				// Same image, just repaint the frame! This will display our subpic.
				m_Preview->RePaintVideo();
			}
			::ShowWindow(m_PreviewHandle,SW_SHOW);
		}
	}
	else
	{
		m_SelectedSubPic = 0;
		// Just to erase the background...
		DisplayBitmap(0,0,0);
	}
}

void CDVDSubEditDlg::OnDestroy() 
{
	KillTimer(m_Timer);
	// In case another thread is running, tell it to abort!
	m_AbortNow = m_QuitNow = true;
	WaitForSingleObject(m_ThreadHandle,1000);

#if 0
	// Save info... This will enable a fast-reload the next time around.
	char Temp[1024];
	sprintf(Temp,"%s/DVDSubEditLastFile.txt",getenv("APPDATA"));
	SaveSubpicInfo(Temp);
#else
#endif

	// Destroy all dialogs.
	m_ScanDialog->DestroyWindow();
	delete m_ScanDialog;
	m_ApplyAllDialog->DestroyWindow();
	delete m_ApplyAllDialog;
	m_MoveDialog->DestroyWindow();
	delete m_MoveDialog;
	m_SetupDialog->DestroyWindow();
	delete m_SetupDialog;
	m_ZoomDialog->DestroyWindow();
	delete m_ZoomDialog;
	m_Preview->CloseFile();
	delete m_Preview;
	m_Menu.DestroyMenu();
	if(OutLogFd) fclose(OutLogFd);
	delete[] m_OCRBitmap;
	delete[] m_OCRLines;
	delete[] m_LastScanFile;
	delete[] m_LastPPMDir;
	delete[] m_LastOpenSUPDir;
	delete[] m_LastSaveSUPDir;
	delete[] m_LastVOBDir;
	delete[] m_LastSRTDir;
	delete[] m_LogPath;
	delete[] m_DefIFOPath;
	CDialog::OnDestroy();
}

void CDVDSubEditDlg::DispOCR(CSubPic* ThisOne)
{
	if(!ThisOne)
		return;
	long start,end;
	// Erase OCR output.
	m_OCROut.GetSel(start,end);
	m_OCROut.SetWindowText("");
	int i;
	// Select whether to show the original or the modified OCR.
	uchar (*OCR)[128] = m_ShowOriginalOCR.GetCheck() ? ThisOne->m_OCRLinesOrig : ThisOne->m_OCRLines;
	for(i=0;;i++)
	{
		if(OCR[i][0] == '\0') break;
		// This is messy! I need to remove \n from the OCR display, otherwise the OCR and the SUBPIC are
		// not synchronized...
		char Temp[128];
		strcpy(Temp,(char*)OCR[i]);
		int j;
		for(j=0;j<strlen(Temp);j++)
		{
			if(Temp[j] == '\n' )
			{
				Temp[j]=0; break;
			}
		}

		m_OCROut.SetSel(1000,1000);
		if(OCR[i+1][0] != '\0')
			m_OCROut.ReplaceSel(CatStr(Temp,"\n"));
		else
			m_OCROut.ReplaceSel(Temp);
	}
	// Change font.
	if(i)
	{
		CHARFORMAT cf;
		cf.dwMask = CFM_FACE;
//			strcpy(cf.szFaceName,"Times New Roman");
//		strcpy(cf.szFaceName,"Courier New");
//			strcpy(cf.szFaceName,"Arial");
		strcpy(cf.szFaceName,"Tahoma");
		m_OCROut.SetSel(0,-1);
		m_OCROut.SetSelectionCharFormat(cf);
	}
	m_OCROut.SetSel(start,end);
}

void BeforeFunc(LPVOID pParam)
{
	// This function is called at the start of all threaded operations.
	CDVDSubEditDlg* Dialog = (CDVDSubEditDlg*) pParam;
	// Start timer... Disable drag/drop, indicate we're running, show the scan dialog...
	Dialog->m_Timer = Dialog->SetTimer(0,200,NULL);
	Dialog->DragAcceptFiles(FALSE);
	Dialog->m_Running = 1;
	Dialog->m_ScanDialog->ShowWindow(SW_SHOW);
	Dialog->m_AbortNow = 0;
	Dialog->m_Preview->Stop(false);
	Dialog->m_Preview->WaitForThread();
	// Disable the UI and the menus.
	Dialog->EnableUI(false);
	Dialog->m_MenuEnabled = false;
	// This will return the keyboard focus to the main window, instead of stealing it from it.
	::PostMessage(Dialog->m_CLUTSelect.m_hWnd,WM_ACTIVATE,WA_ACTIVE,0);
}

void AfterFunc(LPVOID pParam)
{
	// This is called at the end of every threaded operations.
	CDVDSubEditDlg* Dialog = (CDVDSubEditDlg*) pParam;
	// Don't do any of that if you're quitting because you might lock up (in case you're closing while running).
	if(!Dialog->m_QuitNow)
	{
		// Kill timer, enable drag/drop, enable UI, hide dialog, reset move slider, display SPU...
		Dialog->KillTimer(Dialog->m_Timer);
		Dialog->DragAcceptFiles(TRUE);
		Dialog->EnableUI(true);
		Dialog->m_ScanDialog->ShowWindow(SW_HIDE);
		Dialog->m_ScanDialog->DisplayInfo(0,"","");
		Dialog->m_Phase = Processing_Idle;
		Dialog->SetMoveVSlider(MAX_V_MOVE);
		Dialog->ViewSPU(Dialog->m_SPUSlider.GetPos());
		Dialog->m_MenuEnabled = true;
		Dialog->m_PercentDone = 0;
	}
	Dialog->m_Running = 0;
	Dialog->m_ThreadHandle = 0;
}

UINT DVDSubEditLoadLastFileFunc( LPVOID pParam )
{
	// Threaded function to reload the last file.
	CDVDSubEditDlg* Dialog = (CDVDSubEditDlg*) pParam;
	BeforeFunc(pParam);
	Dialog->LoadSubpicInfo(Dialog->m_LastScanFile);
	AfterFunc(pParam);
	if(!Dialog->m_QuitNow)
	{
		char Temp[512];
		sprintf(Temp,"%s ... -- %d Subpics found",Dialog->m_FileInfos[0].m_FileName,Dialog->m_TotNumSPU);
		Dialog->SetWindowText(Temp);
		Dialog->OnSelchangeStreamselect();
		Dialog->EnableUI(true);
	}
	return 0;
}

UINT DVDSubEditScanFunc( LPVOID pParam )
{
	// Threaded function to scan a set of Vob files.
	CDVDSubEditDlg* Dialog = (CDVDSubEditDlg*) pParam;
	BeforeFunc(pParam);
	Dialog->ScanFilesForSPUs();
	// Waiting one second in case the scan was too fast. That was crashing blutach's machine.
	Sleep(1000);
	AfterFunc(pParam);
	if(!Dialog->m_QuitNow)
	{
		if(Dialog->m_ScanComplete)
		{
			// Add files to MRU... 
			Dialog->AddFilesToMRU();
			Dialog->UpdateMRUMenu();
		}
		char Temp[512];
		sprintf(Temp,"%s ... -- %d Subpics found",Dialog->m_FileInfos[0].m_FileName,Dialog->m_TotNumSPU);
		Dialog->SetWindowText(Temp);
		Dialog->OnSelchangeStreamselect();
		Dialog->SetForegroundWindow();
		Sleep(1500);
	}
	return 0;
}

void CDVDSubEditDlg::UpdateMRUMenu()
{
    m_Menu.DestroyMenu();
    m_Menu.LoadMenu(IDR_MENU);
	SetMenu(&m_Menu);
	CMenu* FileMenu = m_Menu.GetSubMenu(0);
	int i,j;
	char FileName[1024];
	int NumFiles = 0;
	for(i=j=0;i<NUM_MRU;i++)
	{
		sprintf(m_LastScanFile,"%s/DVDSubEditLastFile%d.txt",getenv("APPDATA"),i);
		if(CheckSubpicInfo(m_LastScanFile, FileName, &NumFiles) == 0)
		{
			if(i!=j)
			{
				char Temp2[1024];
				sprintf(Temp2,"%s/DVDSubEditLastFile%d.txt",getenv("APPDATA"),j);
				remove(Temp2);
				rename(m_LastScanFile,Temp2);
			}
			char Temp[1024];
			if(NumFiles>1)
				sprintf(Temp,"%s - %d files",FileName,NumFiles);
			else
				sprintf(Temp,"%s ",FileName);
			FileMenu->ModifyMenu(ID_FILE_MRU0+j,MF_BYCOMMAND|MF_STRING,ID_FILE_MRU0+j,Temp);
			j++;
		}
	}
	m_NumMRU = j;
	for(;j<NUM_MRU;j++)
	{
		FileMenu->RemoveMenu(ID_FILE_MRU0+j,MF_BYCOMMAND);
	}
	DrawMenuBar();
}

void CDVDSubEditDlg::MoveMRUFileToTop(int i)
{
	if(i >= m_NumMRU || i == 0) return;
	char Temp[1024];
	char Temp2[1024];
	sprintf(Temp,"%s/DVDSubEditLastFile%d.txt",getenv("APPDATA"),i);
	sprintf(Temp2,"%s/DVDSubEditLastFileTemp.txt",getenv("APPDATA"));
	remove(Temp2);
	rename(Temp,Temp2);

	int j;
	for(j=i;j>0;j--)
	{
		sprintf(Temp2,"%s/DVDSubEditLastFile%d.txt",getenv("APPDATA"),j);
		sprintf(Temp,"%s/DVDSubEditLastFile%d.txt",getenv("APPDATA"),j-1);
		rename(Temp,Temp2);
	}
	sprintf(Temp2,"%s/DVDSubEditLastFile%d.txt",getenv("APPDATA"),0);
	sprintf(Temp,"%s/DVDSubEditLastFileTemp.txt",getenv("APPDATA"));
	rename(Temp,Temp2);
	UpdateMRUMenu();
}


int CDVDSubEditDlg::FindFilesInMRU(bool IgnoreDate)
{
	int i;
	SortInputFiles();
	char FileName[1024];
	int NumFiles = 0;
	for(i=0;i<m_NumMRU;i++)
	{
		sprintf(m_LastScanFile,"%s/DVDSubEditLastFile%d.txt",getenv("APPDATA"),i);
		if(CheckSubpicInfo(m_LastScanFile, FileName, &NumFiles, IgnoreDate) == 0 && 
			strcmpi(m_FileInfos[0].m_FileName,FileName) == 0 && 
			NumFiles >= m_NumVOBFiles)
			break;
	}
	if(i<m_NumMRU) 
		return i;	
	return -1;
}

void CDVDSubEditDlg::AddFilesToMRU()
{
	int i;
	if(m_TotNumSPU == 0 || m_NumVOBFiles == 0) return;
	// First, check whether this is already in the MRU.
	SortInputFiles();
	char Temp[1024];
	// Wheh adding a file, we only want to find the file in the MRU, regardless of date.
	if((i=FindFilesInMRU(true)) == -1)
	{
		// Not found. Add it!
		if(m_NumMRU < NUM_MRU)
			m_NumMRU++;
		else
		{
			// Remove last file.
			sprintf(Temp,"%s/DVDSubEditLastFile%d.txt",getenv("APPDATA"),NUM_MRU-1);
			remove(Temp);
		}
		i = m_NumMRU-1;
	}
	// Save info.
	sprintf(Temp,"%s/DVDSubEditLastFile%d.txt",getenv("APPDATA"),i);
	SaveSubpicInfo(Temp);
	// Put it in top position.
	MoveMRUFileToTop(i);
}


UINT ApplyToAllFunc(LPVOID pParam)
{
	// Threaded function to apply modifications to all selected subpics.
	CDVDSubEditDlg* Dialog = (CDVDSubEditDlg*) pParam;
	BeforeFunc(pParam);
	Dialog->m_Phase = Processing_Applying;
	// Get all modification parameters from our ChangeRecord class.
	int VMoveby = Dialog->m_ChangeRecord.m_VMove;
	int HMoveby = Dialog->m_ChangeRecord.m_HMove;
	int Border = Dialog->m_ChangeRecord.m_Border;
	int FlushIt = Dialog->m_FlushBorders.GetCheck();
	uchar* Transparency = Dialog->m_ChangeRecord.m_Transparency;
	int PixSelectT = Dialog->m_ChangeRecord.m_TranspPixel;
	uchar* Color = Dialog->m_ChangeRecord.m_Color;
	int PixSelectC = Dialog->m_ChangeRecord.m_ColorPixel;
	uchar* CLUT = Dialog->m_ChangeRecord.m_CLUT;
	int Duration = Dialog->m_ChangeRecord.m_Duration;
	int DurMinMax = Dialog->m_ApplyAllDialog->m_MinMax;
	int StartSelect = Dialog->m_ChangeRecord.m_StartCommand;
	int ChangeAllDCSQT = Dialog->m_ChangeRecord.m_ApplyToAllDCSQTs;
	int DCSQT = Dialog->m_ChangeRecord.m_DCSQT;
	int Fade = Dialog->m_ChangeRecord.m_AddFadeIn;
	int i;
	for(i=0;i<Dialog->m_PruneLength;i++)
	{
		if(Dialog->m_AbortNow)
			break;
		// Check which of the modifications should be applied to all..
		if(Dialog->m_ApplyAllDialog->m_ApplyVMove)
			Dialog->m_PrunedList[i]->VMoveSubPic(VMoveby);
		// Applying borders after move, so if "flushing", it will be applied!
		if(Dialog->m_ApplyAllDialog->m_ApplyBorders)
			Dialog->m_PrunedList[i]->VMoveSubPicInFrame(Border,FlushIt);
		// Make sure to crop the background BEFORE you try to move horizontally.
		if(Dialog->m_ApplyAllDialog->m_ApplyCrop)
			Dialog->m_PrunedList[i]->CropBackground();
		if(Dialog->m_ApplyAllDialog->m_ApplyHMove)
			Dialog->m_PrunedList[i]->HMoveSubPic(HMoveby);
		if(Dialog->m_ApplyAllDialog->m_ApplyFade)
		{
			if(Fade == 1)
				Dialog->m_PrunedList[i]->AddFadeIn(true);
			else if(Fade == -1) 
				Dialog->m_PrunedList[i]->RemoveFadeIn();
		}
		if(Dialog->m_ApplyAllDialog->m_ApplyTransp)
		{
			Dialog->m_PrunedList[i]->m_ChangeAllDCSQT = ChangeAllDCSQT != 0;
			if(ChangeAllDCSQT == 0 && DCSQT >= Dialog->m_PrunedList[i]->m_DCSQT.size())
				continue;
			if(ChangeAllDCSQT == 0) Dialog->m_PrunedList[i]->SelectDCSQT(DCSQT);
			Dialog->ChangeTransparencies(Transparency,PixSelectT,Dialog->m_PrunedList[i]);
		}
		if(Dialog->m_ApplyAllDialog->m_ApplyHide)
		{
			Dialog->m_PrunedList[i]->EraseAll();
		}
#if 0
#ifdef ERASE_WHEN_HIDE
		if(Dialog->m_ChangeRecord.m_Transparency[0] == 0 && Dialog->m_ChangeRecord.m_Transparency[1] == 0 && 
			Dialog->m_ChangeRecord.m_Transparency[2] == 0 && Dialog->m_ChangeRecord.m_Transparency[3] == 0)
			Dialog->m_PrunedList[i]->EraseAll();
#endif
#endif
//		if(Dialog->m_ApplyAllDialog->m_ApplyColor && CLUT == Dialog->m_PrunedList[i]->m_CLUT)
		if(Dialog->m_ApplyAllDialog->m_ApplyColor)
		{
			Dialog->m_PrunedList[i]->m_ChangeAllDCSQT = ChangeAllDCSQT != 0;
			if(ChangeAllDCSQT == 0 && DCSQT >= Dialog->m_PrunedList[i]->m_DCSQT.size())
				continue;
			if(ChangeAllDCSQT == 0) Dialog->m_PrunedList[i]->SelectDCSQT(DCSQT);
			Dialog->ChangeColors(Color,PixSelectC,Dialog->m_PrunedList[i]);
		}
		if(Dialog->m_ApplyAllDialog->m_ApplyDuration)
		{
			if(DurMinMax == 0) // Set exact duration.
				Dialog->m_PrunedList[i]->ChangeDCSQTDelay(Duration);
			else if(DurMinMax == 1 && Dialog->m_PrunedList[i]->GetDCSQTDelay() < Duration) // At least.
				Dialog->m_PrunedList[i]->ChangeDCSQTDelay(Duration);
			else if(DurMinMax == 2 && Dialog->m_PrunedList[i]->GetDCSQTDelay() > Duration) // At most.
				Dialog->m_PrunedList[i]->ChangeDCSQTDelay(Duration);
		}
		if(Dialog->m_ApplyAllDialog->m_ApplyStart)
		{
			if(StartSelect == 0)
				Dialog->m_PrunedList[i]->ChangeCommand(FSTA_DSP,STA_DSP);
			else
				Dialog->m_PrunedList[i]->ChangeCommand(STA_DSP,FSTA_DSP);
		}
		Dialog->m_PercentDone = i*100/(1+Dialog->m_PruneLength);
	}
//	Dialog->m_ChangeRecord.Reset();
	AfterFunc(pParam);
	return 0;
}

UINT RevertAllFunc(LPVOID pParam)
{
	CDVDSubEditDlg* Dialog = (CDVDSubEditDlg*) pParam;
	BeforeFunc(pParam);

	Dialog->m_Phase = Processing_Reverting;
	Dialog->m_ScanDialog->SetWindowText("Reverting changes");
	sprintf(Message,"Reverting changes");
	int i;
	for(i=0;i<Dialog->m_PruneLength;i++)
	{
		Dialog->m_PrunedList[i]->ReloadSPU();
		Dialog->m_PercentDone = i*100/Dialog->m_PruneLength;
	}
	Dialog->UpdateSPUList();
	AfterFunc(pParam);
	return 0;
}

UINT SaveAllFunc(LPVOID pParam)
{
	CDVDSubEditDlg* Dialog = (CDVDSubEditDlg*) pParam;
	BeforeFunc(pParam);

	Dialog->m_Phase = Processing_Saving;
	Dialog->m_ScanDialog->SetWindowText("Saving changes");
	sprintf(Message,"Saving changes");
	int i;
	for(i=0;i<Dialog->m_TotNumSPU;i++)
	{
		if(Dialog->m_MasterList[i]->m_Modified)
			Dialog->m_MasterList[i]->SaveSPU();
		Dialog->m_PercentDone = i*100/Dialog->m_TotNumSPU;
	}
	AfterFunc(pParam);
	if(Dialog->m_ScanComplete)
	{
		Dialog->CloseOpenAllFiles();
		// Re-save file info.
		Dialog->AddFilesToMRU();
	}
	return 0;
}

UINT RetimeAllFunc(LPVOID pParam)
{
	CDVDSubEditDlg* Dialog = (CDVDSubEditDlg*) pParam;
	BeforeFunc(pParam);

	Dialog->m_Phase = Processing_ReTiming;
	Dialog->m_ScanDialog->SetWindowText("ReTiming subpics");
	sprintf(Message,"ReTiming subpics");
	Dialog->MoveSelection(Dialog->m_MoveDialog->m_ChangeTimingBy);
	Dialog->UpdateSPUList();
	AfterFunc(pParam);
	Dialog->CloseOpenAllFiles();
	Dialog->AddFilesToMRU();
	return 0;
}

UINT RunOCRFunc(LPVOID pParam)
{
	CDVDSubEditDlg* Dialog = (CDVDSubEditDlg*) pParam;
	BeforeFunc(pParam);

	Dialog->m_ScanDialog->SetWindowText("Running OCR");
	int i;
	for(i=0;i<Dialog->m_PruneLength;i++)
	{
		TRACE("i=%d\n",i); 
		Dialog->m_PercentDone = 100*i/Dialog->m_PruneLength;
		if(Dialog->m_AbortNow) break;
		Dialog->m_PrunedList[i]->DecodeSubpic();
		Dialog->DoRunOCR(Dialog->m_PrunedList[i]);
	}
	// This to update the OCR status...
	Dialog->UpdateSPUList();
	AfterFunc(pParam);
	return 0;
}

UINT HideAllHearingImpaired(LPVOID pParam)
{
	CDVDSubEditDlg* Dialog = (CDVDSubEditDlg*) pParam;
	BeforeFunc(pParam);
	Dialog->m_ScanDialog->SetWindowText("Erasing Hearing-Impaired subpics");
	for(int i=0;i<Dialog->m_PruneLength;i++)
	{
		TRACE("PL %d i=%d SPU %d \n",Dialog->m_PruneLength,i,Dialog->m_PrunedList[i]->m_SPUNumber); 
		if(Dialog->m_AbortNow) break;
		Dialog->m_PercentDone = 100*i/Dialog->m_PruneLength;
		if(Dialog->m_PrunedList[i]->m_NumChars == 0)
		{
			Dialog->m_PrunedList[i]->DecodeSubpic();
			Dialog->DoRunOCR(Dialog->m_PrunedList[i]);
		}
		Dialog->HideHearingImpairedText(Dialog->m_PrunedList[i]);
	}
	AfterFunc(pParam);
	return 0;
}

void CDVDSubEditDlg::HideAllHearingImpairedText()
{
	if(m_PruneLength == 0 || m_Running)
		return;
#if 0
	if(!m_ListHasOCR)
	{
		if(AfxMessageBox("You must run the OCR function on all subpics before you can use this feature.\nRun it now?",MB_YESNO) == IDYES)
			OCRAll();
		return;
	}
#else
#endif

	EnableUI(false,true);
	m_MenuEnabled = false;
	m_Running = true;
	MESSAGE1("");
	m_Phase = Processing_ErasingHearingImpaired;
	CWinThread *Temp = AfxBeginThread( HideAllHearingImpaired, this);
	m_ThreadHandle = Temp->m_hThread;
}



void CDVDSubEditDlg::StartScan() 
{
	int i=-1;
	if(m_Running == 0)
	{
		m_AbortNow = 0;
		EnableUI(false);
		OutLogFd = fopen(m_LogPath,"w");
		if((i=FindFilesInMRU()) != -1)
		{
			if(AfxMessageBox("The file(s) you want to open was previously analyzed,\n\
and you have the possibility to do a \"fast open\" (which is a lot faster than rescanning).\nYou won't be able to retime subpics, \
but everything else will work normally.\nDo you want to do a fast open?",MB_YESNO) == IDYES)
			{
				LoadLastFile(ID_FILE_MRU0+i);
				return;
			}
		}
		CWinThread *Temp = AfxBeginThread( DVDSubEditScanFunc, this);
		m_ThreadHandle = Temp->m_hThread;
		m_Running = 1;
	}
	else
		m_AbortNow = 1;
}

void CDVDSubEditDlg::OCRThisOne()
{
	if(!m_SelectedSubPic)
		return;
	m_SelectedSubPic->DecodeSubpic();
	DoRunOCR(m_SelectedSubPic); 
	DispOCR(m_SelectedSubPic);
}

void CDVDSubEditDlg::OCRAll()
{
	if(m_Running)
		return;
	EnableUI(false,true);
	m_MenuEnabled = false;
	m_Running = true;
	MESSAGE1("");
	m_Phase = Processing_RunningOCR;
	CWinThread *Temp = AfxBeginThread( RunOCRFunc, this);
	m_ThreadHandle = Temp->m_hThread;
}

void CDVDSubEditDlg::LoadLastFile(UINT Index)
{
	if(WarnIfModifs() == false || m_Running)
		return;
	Index -= ID_FILE_MRU0;
	MoveMRUFileToTop(Index);
	Index = 0;
	sprintf(m_LastScanFile,"%s/DVDSubEditLastFile%d.txt",getenv("APPDATA"),Index);
	m_AbortNow = 0;
	EnableUI(false);
	OutLogFd = fopen(m_LogPath,"w");
	if(m_AutoOpenVideo) {m_ShowVideo.SetCheck(1);m_DisplayVideo=1;}
	CWinThread *Temp = AfxBeginThread( DVDSubEditLoadLastFileFunc, this);
	m_ThreadHandle = Temp->m_hThread;
	m_Running = 1;
}

void CDVDSubEditDlg::OnOpenIFO()
{
	if(!m_SupFileMode)
		return;

	// Don't make it static otherwise, lpstrInitialDir is simply ignored... 
	CFileDialog FileDialog(TRUE, CString("*.IFO"), "", OFN_FILEMUSTEXIST, CString("IFO Files (*.IFO)|*.IFO||"));
	FileDialog.m_ofn.lpstrTitle = "Load IFO file";
	FileDialog.m_ofn.lpstrInitialDir = m_DefIFOPath;

	if (FileDialog.DoModal() == IDOK)
	{
		OpenIFO((LPCTSTR)FileDialog.GetPathName());
		m_CLUTSelect.SetCurSel(0);
		ViewSPU(m_SPUSlider.GetPos());
	}
}


void CDVDSubEditDlg::OnApplytoall() 
{
	if(m_Running) return;
	if(m_ApplyAllDialog->DoModal() != IDOK)
		return;
	CWinThread *Temp = AfxBeginThread( ApplyToAllFunc, this);
	m_ThreadHandle = Temp->m_hThread;
	m_Running = 1;
}

void CDVDSubEditDlg::OnSaveall() 
{
	if(m_Running || !m_CanSave) return;
	if(AfxMessageBox("This will save all your changes to file. No undo...\nProceed?",MB_OKCANCEL) != IDOK)
		return;
	if(!m_SupFileMode)
	{
		// Normal VOB file mode. First save the IFO...
		SaveIFO();
		CWinThread *Temp = AfxBeginThread( SaveAllFunc, this);
		m_ThreadHandle = Temp->m_hThread;
		m_Running = 1;
	}
	else
	{
		// We're dealing with a sup file...
		SaveSelectedAsSupFile(0);
		if(m_SupIFOFileName[0] != 0)
			SaveIFO(m_SupIFOFileName);
	}
	// Uncheck this since no subpic is modified now, and update the list.
	m_ShowModified.SetCheck(0);
	// Make sure we display the subpic to show it's no longer modified.
	UpdateSPUList(true);
}


void CDVDSubEditDlg::OnRevertall() 
{
	if(m_Running || !m_CanReload) return;
	if(AfxMessageBox("This will undo all your changes... Proceed?",MB_OKCANCEL) != IDOK)
		return;
	CWinThread *Temp = AfxBeginThread( RevertAllFunc, this);
	m_ThreadHandle = Temp->m_hThread;
	m_ShowModified.SetCheck(false);
	m_ChangeRecord.Reset();
	m_Running = 1;
}

void CDVDSubEditDlg::OnTimingAll() 
{
	static bool First = true;
	m_MoveDialog->m_RetimeAll = true;
	if(!m_CanMove)
	{
		AfxMessageBox("Retiming subtitles is only possible in the title domain\nand if the entire domain is loaded.\
\nNot available for .sup files");
		return;
	}
	m_ShowEndFrame.SetCheck(0);
	if(m_SelectedSubPic == 0 || m_MoveDialog->DoModal() != IDOK)
		return;
	if(First && AfxMessageBox("There is NO undo for this\nThe VOB file will be modified. Proceed?",MB_OKCANCEL) != IDOK)
		return;
	First = false;
	EnableUI(false,true);
	CWinThread *Temp = AfxBeginThread(RetimeAllFunc, this);
	m_ThreadHandle = Temp->m_hThread;
	m_Running = 1;
}

void CDVDSubEditDlg::OnFileLoaded(const char* FileName)
{
	m_Preview->LoadFile(FileName);
}

void CDVDSubEditDlg::HideHearingImpairedTextSingle()
{
	if(!m_SelectedSubPic)
		return;
	HideHearingImpairedText(m_SelectedSubPic);
	ViewSPU(m_SPUSlider.GetPos());
}

bool CDVDSubEditDlg::HideHearingImpairedText(CSubPic* ThisOne)
{
	if(!ThisOne) return false;
	if(ThisOne->m_NumChars == 0)
		DoRunOCR(ThisOne);
	if(ThisOne->IsTextBracketed((LPCSTR)m_SetupDialog->m_Brackets))
	{
		ThisOne->EraseAll();
		DoRunOCR(ThisOne);
	}
	else 
	{
		// A negative m_NumChars means something was changed.
		if(m_SetupDialog->m_RemoveColumns)
		{
			ThisOne->EraseColumns(m_SetupDialog->m_Recenter!=0,m_SetupDialog->m_AlignBottom!=0);
			if(ThisOne->m_NumChars < 0)
				DoRunOCR(ThisOne);
		}
		ThisOne->EraseBrackets((LPCSTR)m_SetupDialog->m_Brackets,
			m_SetupDialog->m_Recenter!=0,m_SetupDialog->m_AlignBottom!=0);
		if(ThisOne->m_NumChars < 0)
			DoRunOCR(ThisOne);
	}
	return false;
}

void CDVDSubEditDlg::EraseSelectedText()
{
	if(m_SelectedSubPic == 0 || m_ShowOriginalOCR.GetCheck())
		return;
	long Start,End;
	m_OCROut.GetSel(Start,End);
	End = min(End,m_SelectedSubPic->m_NumChars);	// DAMN microsoft!
	if(Start == End) Start = End-1;

#if 0
	int i;
	for(i=0;i<End;i++)
	{
		m_OCROut.SetSel(i,i+1);
		char Fuck[128];
		m_OCROut.GetSelText(Fuck);
		TRACE("Char %d %c m_OCROut %c\n",i,m_SelectedSubPic->m_CharCoords[i].c,Fuck[0]);
	}
#endif

	if(Start < 0)
		return;
	// It seems that End actually points to the next character after the end of the selection.
	m_SelectedSubPic->EraseSubPic(Start,End-1,m_SetupDialog->m_Recenter != 0,m_SetupDialog->m_AlignBottom != 0);
	m_OCROut.SetSel(Start,Start);
	OCRThisOne();
	ViewSPU(m_SPUSlider.GetPos());
}

void CDVDSubEditDlg::OnOCRSetup()
{
	m_SetupDialog->DoModal();
}

void CDVDSubEditDlg::OnOcrGuide() 
{
	ShellExecute(NULL, "open", "http://www.videohelp.com/~DVDSubEdit/Guides/HearingImpaired/guide.htm", NULL, NULL, SW_SHOWNORMAL);
}

void CDVDSubEditDlg::OnColorGuide() 
{
	ShellExecute(NULL, "open", "http://www.videohelp.com/~DVDSubEdit/Guides/ChangingColors/Guide.htm", NULL, NULL, SW_SHOWNORMAL);
}



void CDVDSubEditDlg::EnableUI(bool Enable, bool Full)
{
	GetDlgItem(IDC_APPLYTOALL)->EnableWindow(Enable);
	GetDlgItem(IDC_REVERTALL)->EnableWindow(Enable);
	GetDlgItem(IDC_REVERTSAVED)->EnableWindow(Enable);
	GetDlgItem(IDC_SAVEALL)->EnableWindow(Enable);
	GetDlgItem(IDC_RUNOCR)->EnableWindow(Enable);
	GetDlgItem(IDC_SAVEOCR)->EnableWindow(Enable && m_ListHasOCR);
	GetDlgItem(IDC_ERASESELECTION)->EnableWindow(Enable && m_ListHasOCR);
	GetDlgItem(IDC_RETIMEONE)->EnableWindow(Enable);
	GetDlgItem(IDC_RETIMEALL)->EnableWindow(Enable);
	GetDlgItem(IDC_FINDL)->EnableWindow(Enable && m_ListHasOCR);
	GetDlgItem(IDC_FINDR)->EnableWindow(Enable && m_ListHasOCR);
//	GetDlgItem(IDC_SHOWVIDEO)->EnableWindow(!m_NoVideo);
	GetDlgItem(IDC_PLAY)->EnableWindow(Enable);
	GetDlgItem(IDC_LOOP)->EnableWindow(Enable);
	GetDlgItem(IDC_AUDIOSEL)->EnableWindow(Enable);
	if(Full || Enable)
	{
		GetDlgItem(IDC_STREAMSELECT)->EnableWindow(Enable);
		GetDlgItem(IDC_PSTREAM)->EnableWindow(Enable);
		GetDlgItem(IDC_NSTREAM)->EnableWindow(Enable);
		GetDlgItem(IDC_VCIDSELECT)->EnableWindow(Enable);
		GetDlgItem(IDC_PVCID)->EnableWindow(Enable);
		GetDlgItem(IDC_NVCID)->EnableWindow(Enable);
		GetDlgItem(IDC_SHOWMOD)->EnableWindow(Enable);
		GetDlgItem(IDC_SHOWFORCED)->EnableWindow(Enable);
		GetDlgItem(IDC_SHOWSEARCHED)->EnableWindow(Enable && m_ListHasOCR);
		GetDlgItem(IDC_SEARCHSELECT)->EnableWindow(Enable);
		GetDlgItem(IDC_DURATIONS)->EnableWindow(Enable);
		GetDlgItem(IDC_DURATIONE)->EnableWindow(Enable);
	}
}

void CDVDSubEditDlg::OnTimer(UINT nIDEvent) 
{
	static int Count = 0;

	if(m_QuitNow == 1)
		return;

	// Update progress bar... TODO: FIX THIS MESS!
	char Temp[512];
	switch(m_Phase)
	{
	case Processing_Idle : sprintf(Temp,"");break;
	case Processing_Scanning_VOB : 	sprintf(Temp,"Scanning VOB file(s)"); break;
	case Processing_Done : sprintf(Temp,"Done...");break;
	case Processing_Aborted : sprintf(Temp,"Aborted");break;
	case Processing_Applying : sprintf(Temp,"Applying modifications");break;
	case Processing_Reverting : sprintf(Temp,"Reverting");break;
	case Processing_Saving : sprintf(Temp,"Saving modifications");break;
	case Processing_ReTiming : sprintf(Temp,"Re-timing");break;
	case Processing_UpdatingSRI : sprintf(Temp,"Updating trick-play pointers");break;
	case Processing_RunningOCR : sprintf(Temp,"Running OCR... (based on GOCR by Joerg Schulenburg)");break;
	case Processing_FastReloading : sprintf(Temp,"Loading file(s) from MRU");break;
	case Processing_ErasingHearingImpaired : sprintf(Temp,"Erasing Hearing-Impaired text");break;
	case Processing_Cropping : sprintf(Temp,"Cropping subpic background"); break;
	}
	if(m_Phase != Processing_Idle)
	{
		if(m_Phase != Processing_Scanning_VOB)
			MESSAGE2("%d%% done",m_PercentDone);
		m_ScanDialog->DisplayInfo(m_PercentDone,Temp,Message);
	}
	if((m_Phase == Processing_Scanning_VOB || m_Phase == Processing_FastReloading) && Count++ % 5 == 0)
	{
		// When scanning the the vob, update the SPU list as we detect more spus...
		Count = 1;
		if(m_TotNumSPU < 2)
			OnSelchangeStreamselect();
		else
			UpdateSPUList();
	}

	CDialog::OnTimer(nIDEvent);
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CDVDSubEditDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
		return;
	}
	CPaintDC dc(this);
	if(!m_DisplayVideo || m_NoVideo)
	{
		// We're not displaying any video, so we have to draw our bitmap ourself!
		// Only do it if it's needed!
		if(RectVisible(dc.m_hDC,&m_VideoRect))
		{
			TRACE("ONPAINT\n");
			CDialog::OnPaint();
			PAINTSTRUCT Ps;
			BeginPaint(&Ps);
			DisplayBitmap(0,0,0);
			EndPaint(&Ps);
			return;
		}
	}
	if(m_ColorMap)
	{
		// We need to display at least the clut tiles...
		PAINTSTRUCT Ps;
		BeginPaint(&Ps);
		ShowCol(dc.m_hDC, IDC_COLB,m_ColorMap); ShowCol(dc.m_hDC,IDC_COLP,m_ColorMap+3);
		ShowCol(dc.m_hDC, IDC_COLE1,m_ColorMap+6); ShowCol(dc.m_hDC,IDC_COLE2,m_ColorMap+9);
		EndPaint(&Ps);
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CDVDSubEditDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}


#define MAX_NUM_FILES 64
void CDVDSubEditDlg::OnDropFiles(HDROP hDropInfo) 
{
	SetForegroundWindow();      // activate us first !
	POINT ThePoint;
	DragQueryPoint(hDropInfo,&ThePoint);

	UINT nFiles = ::DragQueryFile(hDropInfo, (UINT)-1, NULL, 0);
	CWinApp* pApp = AfxGetApp();
	ASSERT(pApp != NULL);
	char FileNames[MAX_NUM_FILES][512];
	int i;
	for(i=0;i<MAX_NUM_FILES;i++)
		strcpy(FileNames[i],"");
	int VTSNum = 0;
	m_CLUTSelect.SetCurSel(0);
	for (UINT iFile = 0; iFile < nFiles; iFile++)
	{
		TCHAR szFileName[_MAX_PATH];
		::DragQueryFile(hDropInfo, iFile, szFileName, _MAX_PATH);
		if(strcmpi(GetExt(szFileName),".sup") == 0)
		{
			if(WarnIfModifs() == false)
				return;
			// Special code for sup files.
			if(LoadSUPFile(szFileName) != 0)
				return;
			SetWindowText(szFileName);
			::DragFinish(hDropInfo);
			strcpy(m_DefIFOPath,GetDirPath(szFileName));
			if(m_SetupDialog->m_AskForIFO)
				OnOpenIFO();
			m_CLUTSelect.SetCurSel(1);
			m_DispModeSelect.SetCurSel(0);
			OnSelchangeDisplaymode();
			OnSelchangeStreamselect();
			EnableUI(true);
			return;
		}
		if(strcmpi(GetExt(szFileName),".bmp") == 0 || strcmpi(GetExt(szFileName),".ppm") == 0)
		{
			if(m_SelectedSubPic == 0)
				return;
			m_SelectedSubPic->LoadBMP((LPCTSTR)szFileName,GetCLUT(), 0);
			strcpy(m_LastPPMDir,GetDirPath((LPCTSTR)szFileName));
			ViewSPU(m_SPUSlider.GetPos());
			return;
		}
		if(strcmpi(GetExt(szFileName),".vob") != 0)
		{
			AfxMessageBox("Error: I can only open VOB or SUP files");
			::DragFinish(hDropInfo);
			return;
		}
		if(iFile == 0 && WarnIfModifs() == false)
			return;
		int Index = -1;
		int VTS = -1;
		// Get VTS number and index.
		if(sscanf(GetFname(szFileName),"%*[^_]_%d_%d",&VTS,&Index) != 2)
		{
			// Probably not a "vts_xx_xx", let's just add to our list at the next index...
			Index = (int)iFile;
			VTSNum = 0;
		}
		if(VTSNum == 0) VTSNum = VTS;
		else if(VTSNum != VTS)
		{
			AfxMessageBox("All the VOB files must come from the same VTS");
			::DragFinish(hDropInfo);
			return;
		}
		ASSERT(Index < 10 && Index >= 0);
		strcpy(FileNames[Index],szFileName);
	}
	::DragFinish(hDropInfo);
	
	// Now put these file names in order...
	Reset();
	for(i=0;i<MAX_NUM_FILES;i++)
	{
		if(strcmp(FileNames[i],"") == 0)
			continue;
		if(AddFile(FileNames[i]) == -1)
		{
			char Temp[512];
			sprintf(Temp,"Error opening %s\nIs the file writeable?",FileNames[i]);
			AfxMessageBox(Temp);
			return;
		}
		else
		{
			SetWindowText(FileNames[i]);
		}
	}
	EnableUI(true);

	// Now scan input files...
	m_ScanDialog->DisplayInfo(0,"","");
	m_ScanDialog->ShowWindow(SW_SHOW);
	if(m_AutoOpenVideo) {m_ShowVideo.SetCheck(1);m_DisplayVideo=1;}
	// Loading file first in case StartScan finishes too fast!
	::ShowWindow(m_PreviewHandle,SW_HIDE);
	m_Preview->LoadFile(m_FileInfos[0].m_FileName);
	StartScan();
}


HBRUSH CDVDSubEditDlg::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) 
{
	HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);
	int id = pWnd->GetDlgCtrlID();
	//COLORREF bkcolor = pDC->GetBkColor();
	if (id == IDC_INFO || id == IDC_MESSAGE)
	{
		// Set the text color to black
		pDC->SetTextColor(RGB(0, 0, 0)); 
	}	
	// TODO: Return a different brush if the default is not desired
	return hbr;
}

void CDVDSubEditDlg::OnSpur() 
{
	if(m_Preview->IsVideoPlaying()) AbortPlayback(1);
	m_SPUSlider.SetPos(m_SPUSlider.GetPos()+1);
	ViewSPU(m_SPUSlider.GetPos()); 
}

void CDVDSubEditDlg::OnSpul() 
{
	if(m_Preview->IsVideoPlaying())  AbortPlayback(1);
	m_SPUSlider.SetPos(m_SPUSlider.GetPos()-1);
	ViewSPU(m_SPUSlider.GetPos()); 
}

void CDVDSubEditDlg::OnCropSubpic()
{
	if(!m_SelectedSubPic)
		return;
	m_SelectedSubPic->CropBackground();
	ViewSPU(m_SPUSlider.GetPos());
	m_ChangeRecord.RecordChange(m_SelectedSubPic,ChangeCrop,1);
}

void CDVDSubEditDlg::JustAddedSPU(int StreamIndex, int SPUIndex)
{
	if(m_QuitNow)
		return;
	if(m_TotNumSPU == 1)
	{
		OnSelchangeStreamselect();
	}
}

void CDVDSubEditDlg::JustFoundStream(int StreamIndex)
{
	if(m_QuitNow)
		return;
	if(StreamIndex+1 < m_StreamSelect.GetCount())
		return;
	// If we're in VOB mode, the stream selection will be updated in ViewSPU.
	if(!m_SupFileMode)
	{
		ViewSPU(m_SPUSlider.GetPos());
		return;
	}
	// In .sup mode, we need to do this by hand.
	m_StreamSelect.ResetContent();
	m_StreamSelect.AddString("All Streams");
	int i;
	for(i=0;i<=m_TopStreamID;i++)
	{
		char Temp[16];
		sprintf(Temp,"0x%x",i+32);
		m_StreamSelect.AddString(Temp);
	}
	m_StreamSelect.SetCurSel(m_SupFileMode?1:m_SelectedStream);
	m_StreamSelect.RedrawWindow();
}

void CDVDSubEditDlg::JustFoundVCID(uint VCID)
{
	if(m_QuitNow)
		return;
	char Temp[64];
	if(VCID%256 != 0)
		sprintf(Temp,"%d/%d",VCID/256,VCID%256);
	else
		sprintf(Temp,"%d/*",VCID/256);
	m_VCIDSelect.AddString(Temp);
}

void CDVDSubEditDlg::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	// At the end of the scroll, we always get a SB_ENDSCROLL and a SB_THUMBPOSITION message. Ignore them!
	bool EndScroll = nSBCode == SB_ENDSCROLL || nSBCode == SB_THUMBPOSITION;
	switch(pScrollBar->GetDlgCtrlID())
	{
	case IDC_VIDEOSELECT : {
		int IframeLBA = m_IFrameSelect.GetPos();
		m_ShowEndFrame.SetCheck(0);
TRACE("IFRAME %d\n",IframeLBA);
		m_Preview->GoToSector(IframeLBA);
		break;}
	case IDC_SPUSLIDER : {
		if(EndScroll) break;
		static int LastPos = 0;
		if(LastPos == m_SPUSlider.GetPos())
			break;
		if(m_Preview->IsVideoPlaying())  AbortPlayback(1);
		LastPos = m_SPUSlider.GetPos();
		ViewSPU(LastPos); 
		break;}
	case IDC_MOVEUDS : 
		if(EndScroll) break;
		DispThis2(IDC_MOVEUDE,"%d",m_MoveVSlider.GetPos()-MAX_V_MOVE);
		if(m_SelectedSubPic)// && nSBCode == SB_ENDSCROLL) 
		{
			m_SelectedSubPic->VMoveSubPic(m_MoveVSlider.GetPos()-MAX_V_MOVE);
			// Be smart: if we move the move slider, we probably want to apply that!
			m_ChangeRecord.RecordChange(m_SelectedSubPic, ChangeVMove, m_MoveVSlider.GetPos()-MAX_V_MOVE);
			ViewSPU(m_SPUSlider.GetPos());
		}
		break;
	case IDC_MOVELR : 
		if(EndScroll) break;
		DispThis2(IDC_MOVELRE,"%d",m_MoveHSlider.GetPos()-MAX_H_MOVE);
		if(m_SelectedSubPic)// && nSBCode == SB_ENDSCROLL) 
		{
			m_SelectedSubPic->HMoveSubPic(m_MoveHSlider.GetPos()-MAX_H_MOVE);
			// Be smart: if we move the move slider, we probably want to apply that!
			m_ChangeRecord.RecordChange(m_SelectedSubPic, ChangeHMove, m_MoveHSlider.GetPos()-MAX_H_MOVE);
			ViewSPU(m_SPUSlider.GetPos());
		}
		break;
	case IDC_TRANSPARENCYS : 
		if(EndScroll) break;
		if(m_SelectedSubPic)// && nSBCode == SB_ENDSCROLL) 
		{
			int SelectedPixel = m_PixelSelect.GetCurSel();
			int CurTrans = m_TransSlider.GetPos();
			// Be smart: if we move the transparency slider, we probably want to apply that!
			m_ChangeRecord.RecordChange(m_SelectedSubPic, ChangeTransp, CurTrans,SelectedPixel);
			if(m_SelectedSubPic->ChangeTransparency(CurTrans,SelectedPixel) == 0)
				ViewSPU(m_SPUSlider.GetPos());
		}
		break;
	case IDC_BORDERS : 
		if(EndScroll) break;
		if(m_SelectedSubPic)
		{
			int TopBorder = m_BorderSlider.GetPos();
			DispThis2(IDC_BORDERE,"%d",TopBorder);
			m_SelectedSubPic->VMoveSubPicInFrame(TopBorder,m_FlushBorders.GetCheck());
			ViewSPU(m_SPUSlider.GetPos());
			if(TopBorder == 0)
				SetBorderSelect(-1);
			else 
				SetBorderSelect(BORDER_CUSTOM);
			m_ChangeRecord.RecordChange(m_SelectedSubPic, ChangeBorder, TopBorder);

		}
		break;
	case IDC_DURATIONS : 
		// Discarding these events because of the feedback between setting the duration and the fact that it's rounded
		// to the nearest frame.
		if(m_SelectedSubPic && !EndScroll)
		{
			int Duration = m_DurationSlider.GetPos();
			m_SelectedSubPic->ChangeDCSQTDelay(Duration);
			m_ChangeRecord.RecordChange(m_SelectedSubPic, ChangeDur, m_SelectedSubPic->GetDCSQTDelay());
			SetIFrameSlider(m_SelectedSubPic);
			ViewSPU(m_SPUSlider.GetPos(),m_ShowEndFrame.GetCheck() != 0);
		}
		break;
	default: break;
	}
	CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
}

void CDVDSubEditDlg::SetMoveVSlider(int Pos)
{
	m_MoveVSlider.SetPos(Pos);
	DispThis2(IDC_MOVEUDE,"%d",Pos-MAX_V_MOVE);
}

void CDVDSubEditDlg::SetMoveHSlider(int Pos)
{
	m_MoveHSlider.SetPos(Pos);
	DispThis2(IDC_MOVELRE,"%d",Pos-MAX_H_MOVE);
}

void CDVDSubEditDlg::SetTransSlider(int Pos)
{
	m_TransSlider.SetPos(Pos);
	DispThis2(IDC_TRANSPARENCYE,"%d",Pos);
}

void CDVDSubEditDlg::SetBorderSlider(int Pos)
{
	m_BorderSlider.SetPos(Pos);
	DispThis2(IDC_BORDERE,"%d",Pos);
}

void CDVDSubEditDlg::SetDurationSlider(int Pos)
{
	if(m_SelectedSubPic && m_SelectedSubPic->m_DCSQT.size() > 1  && 
		m_SelectedSubPic->FindCommand(STP_DSP,m_SelectedSubPic->m_DCSQT.size()-1) != -1)
	{
		// Find timing of next to last DCSQT
		int MinDurMs, MaxDurMs;
		m_SelectedSubPic->GetMinMaxDCSQTDelay(MinDurMs, MaxDurMs);
		m_DurationSlider.SetRange(MinDurMs,MaxDurMs,true);
		m_DurationSlider.EnableWindow(true);
		m_MakeInfinite.EnableWindow(true);
		DispThis2(IDC_DURATIONE,"%.3f",(float)Pos/1000);
	}
	else
	{
		m_DurationSlider.EnableWindow(false);
		m_MakeInfinite.EnableWindow(false);
		DispThis1(IDC_DURATIONE,"infinite");
	}
	m_DurationSlider.SetPos(Pos);
}

void CDVDSubEditDlg::SetIFrameSlider(CSubPic* ThisOne)
{
	if(!m_SelectedSubPic)
		return;
	// Find LBA of I-frame corresponding to next subpic.
	CSubPic* Next = FindNextSPUInStream(ThisOne->m_SPUNumber);
	if(!Next || m_SelectedSubPic->m_DCSQT.size() == 1)
	{
		m_IFrameSelect.SetRange(ThisOne->m_NavPackLBA,ThisOne->m_NavPackLBA,true);
		return;
	}
	m_NextSPULba = Next->m_NavPackLBA;
	int DispLBA;
	if(m_DurationSlider.GetRangeMax() != 0 && m_SelectedSubPic->m_DCSQT.size() != 1)
		DispLBA = ThisOne->m_NavPackLBA + m_DurationSlider.GetPos()  * (m_NextSPULba-ThisOne->m_NavPackLBA) / m_DurationSlider.GetRangeMax();
	else 
		DispLBA = m_NextSPULba;
	m_IFrameSelect.SetRange(ThisOne->m_NavPackLBA,DispLBA,true);
}

void CDVDSubEditDlg::SetBorderSelect(int Selection)
{
	if(Selection == 0)
	{
		// Custom borders.
		m_BorderSelect.DeleteString(0);
		m_BorderSelect.InsertString(0,"Custom Vertical Borders");
		m_BorderSelect.SetCurSel(Selection);
	}
	else if(Selection > 0)
	{
		// Standard borders.
		m_BorderSelect.DeleteString(0);
		m_BorderSelect.InsertString(0,"No Vertical Borders");
		m_BorderSelect.SetCurSel(Selection);
	}
	else
	{
		// -1 means, no border. Change the string name!
		m_BorderSelect.DeleteString(0);
		m_BorderSelect.InsertString(0,"No Vertical Borders");
		m_BorderSelect.SetCurSel(0);
	}
}

void CDVDSubEditDlg::OnSelchangeStreamselect() 
{
	m_ShowSearched.SetCheck(false);
	UpdateSPUList();
	if(!m_PruneLength)
		m_CommandDisplay.SetWindowText("");
}

void CDVDSubEditDlg::OnShowmod() 
{
	UpdateSPUList();
}

void CDVDSubEditDlg::OnShowsearched() 
{
	UpdateSPUList();
}

void CDVDSubEditDlg::OnChangedSearchselect() 
{
	CString Temp;
	GetDlgItemText(IDC_SEARCHSELECT,Temp);

	bool Active =  Temp != "";
	m_ShowSearched.SetCheck(Active);
	UpdateSPUList();
}

void CDVDSubEditDlg::UpdateSPUList(bool ForceDisplay)
{
	// User changed the selected stream. Create the pruned list.
	int ThisPos = m_SPUSlider.GetPos();
	int SPUIndex = m_SelectedSubPic ? m_SelectedSubPic->m_SPUNumber : -1;
	int NextPos = 0;
	// Don't do anything if the drop down menu is down!
	if(m_StreamSelect.GetDroppedState()) return;
	m_SelectedStream = m_StreamSelect.GetCurSel();
	int CurrentVCID = m_VCIDSelect.GetCurSel();
	char SearchStr[128];
	GetDlgItemText(IDC_SEARCHSELECT,SearchStr,128);
	if(CurrentVCID == 0)
		NextPos = PruneSPUList(ThisPos,m_SelectedStream-1,-1,-1,0,0,m_ShowModified.GetCheck(),
		m_ShowSearched.GetCheck()?SearchStr: 0, m_ShowForced.GetCheck()?FSTA_DSP:-1, m_ShowTransparent.GetCheck());
	else
		NextPos = PruneSPUList(ThisPos,m_SelectedStream-1,-1,-1,m_VCIDList[CurrentVCID-1]/256,
			m_VCIDList[CurrentVCID-1]%256,m_ShowModified.GetCheck(),
			m_ShowSearched.GetCheck()?SearchStr : 0, m_ShowForced.GetCheck()?FSTA_DSP:-1, m_ShowTransparent.GetCheck());

	m_SPUSlider.SetRange(0,m_PruneLength-1,true);
	m_SPUSlider.ShowWindow((m_PruneLength > 1)?SW_SHOW:SW_HIDE);
	m_IFrameSelect.ShowWindow((m_PruneLength > 1 && m_ShowVideo)?SW_SHOW:SW_HIDE);
	if(NextPos != -1)
		m_SPUSlider.SetPos(NextPos);
	GetDlgItem(IDC_SPUL)->ShowWindow((m_PruneLength > 1)?SW_SHOW:SW_HIDE);
	GetDlgItem(IDC_SPUR)->ShowWindow((m_PruneLength > 1)?SW_SHOW:SW_HIDE);
	ENABLE(IDC_FINDL,m_ListHasOCR);
	ENABLE(IDC_FINDR,m_ListHasOCR);
	ENABLE(IDC_SHOWSEARCHED,m_ListHasOCR && !m_Running);
	ENABLE(IDC_SAVEOCR,m_ListHasOCR && !m_Running);
	ENABLE(IDC_ERASESELECTION,m_ListHasOCR && !m_Running);
	char Temp[64];
	sprintf(Temp,"%d SPUs",m_PruneLength);
	SetDlgItemText(IDC_NUMPRUNED,Temp);
	if(m_PruneLength != 0 && !m_SupFileMode && m_SetupDialog->m_AutoDisplay && NextPos != -1)
	{
		m_SelectedSubPic = m_PrunedList[NextPos];
		AdjustDisplayMode();
	}
	if(ForceDisplay || NextPos == -1 || m_PrunedList[NextPos]->m_SPUNumber != SPUIndex)
	{
		ViewSPU(m_SPUSlider.GetPos());
	}
}

void CDVDSubEditDlg::AdjustDisplayMode()
{
	// Select the display mode as a function of the stream selected.
	if(m_SelectedSubPic == 0 || (m_SupFileMode && m_SupIFOFileName[0] == 0)) return;
	char Lang[32][16];
	uchar DispModes[32];
	FindCLUT(m_SelectedSubPic->m_VOBID,m_SelectedSubPic->m_CELLID,Lang,DispModes);
	uchar Disp = DispModes[m_SelectedSubPic->m_StreamID];
	// Tricky. Disp has bit 0 set for 4:3, 1 for 16:9, 2 for LB and 3 for PS.
	// Let's reflect the current display mode in the same manner:
	uchar CurrentDispMode = (uchar) (1 << m_DispModeSelect.GetCurSel());
	// Then check that the bit set in CurrentDispMode is also set in Disp (meaning,
	// the current display is OK for the current subpic).
	if((Disp & CurrentDispMode) == 0)
	{
		// Not correct. Change display mode!
		for(int i=0;i<4;i++,Disp >>= 1)
			if(Disp & 0x01) break;
		m_DispModeSelect.SetCurSel(min(i,3));
		// This calls JustFoundVideoFormat which causes a refresh of the display.
		OnSelchangeDisplaymode();
	}
}


// This is really a kludge, to "reconnect" very thin vertical lines that can be messed up by the OCR. It's mainly
// intended for music notes (lots of them have a very thin vertical stroke.
// TODO: Detect situations like b-a-a-a-b ?
void CDVDSubEditDlg::FillMissingPix(uchar* Bitmap, int Width, int Height, uchar Body, uchar Background, uchar FillPixel)
{
	//return;
	int i,j;
	if(FillPixel == 255) FillPixel = Body;
	for(i=1;i<Height-1;i++)
	{
		uchar* CurL = Bitmap+i*Width;
		uchar* PreL = Bitmap+(i-1)*Width;
		uchar* NexL = Bitmap+(i+1)*Width;
		for(j=2;j<Width-2;j++)
		{
			// Connect up with down, but only if the line is very thin.
			// Do nothing if already body, or background, or the one left of it is pixel.
			if(CurL[j] == Body || CurL[j-1] == Body || CurL[j] == Background) continue;
			// Do nothing if pixel right above isn't body, or if up-left IS body!.
			if(PreL[j] != Body || PreL[j-1] == Body) continue;
			// Line below must be thin (CountBelow must be 1), and the pixel below right must be clear.
			int CountBelow = (NexL[j] == Body) + (NexL[j-1] == Body) + (NexL[j+1] == Body);
			if(NexL[j+1] != Body && NexL[j-2] != Body && CountBelow == 1)
				CurL[j] = 6;
		}
	}
	int N = Width * Height;
	for(i=0;i<N;i++)
		if(Bitmap[i] == 6) 
			Bitmap[i] = FillPixel;
}


#define OVERSIZE 30
void CDVDSubEditDlg::DoRunOCR(CSubPic* TheSubpic)
{
	// Don't run OCR on button highlights. This is not a 
	if(!TheSubpic || TheSubpic->ContainsHighlights()) return;
	// Allocate bitmap and lines of characters for results if needed.
	if(m_OCRBitmap == 0) m_OCRBitmap = new uchar[(720+OVERSIZE)*576];
	if(m_OCRLines == 0) m_OCRLines = new char[32][128];
	long NewWidth, NewHeight;
	static int busy = 0;
	while(busy) Sleep(10);

	int i,j;
	// Get the body pixel from the subpic. Use the previous one if it couldn't be detected...
	static int BodyPixel = -1;
	if(TheSubpic->m_BodyPix != -1)
		BodyPixel = TheSubpic->m_BodyPix;
	if(BodyPixel == -1)
		return;

	busy = 1;


	// Colors for each pixel types. Black for everything but the character body pixels.
	uchar Colors[4] ={255,255,255,255};
	Colors[BodyPixel]=0;
	// Now copy the pixel data into our m_OCRBitmap array. Oversizing to allow straightening characters.
	NewWidth = TheSubpic->m_SubPicWidth ;
	NewHeight = TheSubpic->m_BottomMostPixel - TheSubpic->m_TopMostPixel;

	FillMissingPix(TheSubpic->m_Pixels+TheSubpic->m_TopMostPixel*NewWidth,
		NewWidth,NewHeight,
		BodyPixel,TheSubpic->m_BackgroundPix);

	uchar *Ptr = m_OCRBitmap;
	for(i=0;i<NewHeight;i++)
	{
		uchar* src = TheSubpic->m_Pixels + 
			(TheSubpic->m_TopMostPixel+TheSubpic->m_VerticalAdjust+i) * NewWidth;
		for(j=0;j<NewWidth;j++)
			*Ptr++ = Colors[src[j]] ;
		for(;j<NewWidth+OVERSIZE;j++)
			*Ptr++ = 255 ;
	}
	NewWidth += OVERSIZE;

#if 0
	// Use this to take a look at the bitmap.
	FILE* fd = fopen("/temp/t2.ppm","wb");
	fprintf (fd,"P6\n%d %d\n255\n", NewWidth, NewHeight);
	uchar* Foo = new uchar[3*NewWidth*NewHeight];
	memset(Foo,255,3*NewWidth*NewHeight);
	for(i=0;i<NewWidth*NewHeight;i++)
	{
		if(m_OCRBitmap[i] == 0)
			Foo[3*i] = Foo[3*i+1] = Foo[3*i+2] = 0;
	}
	fwrite(Foo,sizeof(char),3*NewWidth*NewHeight,fd);
	fclose(fd);
	delete[] Foo;
#endif

	// Coord will contain the coordinates of each character box..
	CharCoord Coord[1024];
	int NumChars=0;
	RunOCR(m_OCRBitmap,NewWidth,NewHeight,m_OCRLines,&m_NumOCRLines,Coord,&NumChars,
		m_SetupDialog->m_Italics, 2*m_SetupDialog->m_SpaceSelect);

#if 0
	// Use this to take a look at the bitmap.
	FILE* fd = fopen("/temp/t2.ppm","wb");
	fprintf (fd,"P6\n%d %d\n255\n", NewWidth, NewHeight);
	uchar* Foo = new uchar[3*NewWidth*NewHeight];
	memset(Foo,255,3*NewWidth*NewHeight);
	for(i=0;i<NewWidth*NewHeight;i++)
	{
		if(m_OCRBitmap[i] == 0)
			Foo[3*i] = Foo[3*i+1] = Foo[3*i+2] = 0;
	}
	fwrite(Foo,sizeof(char),3*NewWidth*NewHeight,fd);
	fclose(fd);
	delete[] Foo;
#endif

	if(DoJustItalics)
	{
		// If you wanted to see the straightened bitmap.
		Ptr = TheSubpic->m_Pixels+TheSubpic->m_TopMostPixel*TheSubpic->m_SubPicWidth;
		for(i=0;i<NewHeight;i++)
		{
			uchar* src = m_OCRBitmap + i * NewWidth;
			for(j=0;j<TheSubpic->m_SubPicWidth;j++)
				*Ptr++ = (src[j] == 0 ? BodyPixel : TheSubpic->m_BackgroundPix) ;
		}
		DisplayBitmap(0,0,0);
	}

	delete[] TheSubpic->m_CharCoords;
	TheSubpic->m_CharCoords = new CSubPic::CharCoord[NumChars];
	for(i=0;i<NumChars;i++)
	{
		memcpy(&TheSubpic->m_CharCoords[i],&Coord[i],sizeof(CharCoord));
		TheSubpic->m_CharCoords[i].y0 += TheSubpic->m_TopMostPixel;
		TheSubpic->m_CharCoords[i].y1 += TheSubpic->m_TopMostPixel;
		TheSubpic->m_CharCoords[i].LineTops += TheSubpic->m_TopMostPixel;
	}
	TheSubpic->m_NumChars = NumChars;

	for(i=0;i<m_NumOCRLines;i++)
		strcpy((char*)TheSubpic->m_OCRLines[i],m_OCRLines[i]);
	TheSubpic->m_OCRLines[i][0]='\0';
	// We also keep a copy of the original OCR output...
	if(TheSubpic->m_OCRLinesOrig[0][0] == '\0')
	{
		for(i=0;i<m_NumOCRLines;i++)
			strcpy((char*)TheSubpic->m_OCRLinesOrig[i],m_OCRLines[i]);
		TheSubpic->m_OCRLinesOrig[i][0]='\0';
	}
//	m_OCROut.SetWindowText(m_OCRLines[0]);

	busy = 0;
	// Debug stuff
	OCRVerbose = 0;
	DoJustItalics = 0;
}


BOOL CDVDSubEditDlg::PreTranslateMessage(MSG* pMsg) 
{

	// These are messages sent by the preview at the end of the playback.
	// I don't understand why, but if you wiggle the window, these messages disappear from the message
	// queue and are never received by the dialog, even though they're sent by the Preview thread. :(
	if(pMsg->message == WM_USER && pMsg->wParam == ID_PLAYBACK_STOPPED)
	{
		// Playback just stopped.
		m_Preview->WaitForThread();
		OutputDebugString("ID_PLAYBACK_STOPPED\n");
		OnPlaybackStopped();
		return true;
	}

	if(pMsg->message == WM_USER && pMsg->wParam == ID_SECTOR_HIT)
	{
		// Playback reached sector or PTS.
		m_Preview->WaitForThread();
		OutputDebugString("ID_SECTOR_HIT\n");
		OnLoop();
		return true;
	}

	// This will capture all messages arriving at the app before they're translated and dispatched.
	// Don't process keydown messages coming from the search or searchselect forms.
	if(pMsg->message == WM_KEYDOWN && pMsg->hwnd != GetDlgItem(IDC_SEARCH)->m_hWnd && 
		pMsg->hwnd != GetDlgItem(IDC_SEARCHSELECT)->m_hWnd)
	{
		unsigned short Mask = 0x8000;
		BOOL ShiftOn = (GetAsyncKeyState(VK_SHIFT) & Mask);
		BOOL CtrlOn = (GetAsyncKeyState(VK_CONTROL) & Mask);
		BOOL AltOn = (GetAsyncKeyState(VK_MENU) & Mask);
		BOOL InFormWin = pMsg->hwnd == GetDlgItem(IDC_OCROUT)->m_hWnd;
		InFormWin |= pMsg->hwnd == GetDlgItem(IDC_GOTO)->m_hWnd;

		// Handle these no matter what.
		switch(pMsg->wParam)
		{
		case VK_F1:	OnHelpManual();return true;
		case VK_F5:	InvalidateRect(0,0);return true;
		case VK_F3:	FindNext(ShiftOn?-1:1);return true;
		case VK_ESCAPE: m_AbortNow = true;  return true;
		}
		// Handle these if not in an input form:
		if(!InFormWin)
		{
			switch(pMsg->wParam)
			{
				case VK_RIGHT: 	OnSpur(); return true;
				case VK_LEFT: 	OnSpul(); return true;
				case VK_HOME: m_SPUSlider.SetPos(0); ViewSPU(0); return true;
				case VK_END: m_SPUSlider.SetPos(m_PruneLength-1); ViewSPU(m_PruneLength-1);return true;
				case VK_SPACE: OnPlay(); return true;
			}
		}
		// Special case: OCR output window.
		if(pMsg->hwnd == GetDlgItem(IDC_OCROUT)->m_hWnd)
		switch(pMsg->wParam)
		{
			case VK_BACK:	EraseSelectedText();return true;
//			case VK_DELETE:	long start,end; m_OCROut.GetSel(start,end); m_OCROut.SetSel(start+1,end+1);
//				EraseSelectedText();return true;
			case VK_DELETE:	EraseSelectedText();return true;
			case 'X': if(CtrlOn) {EraseSelectedText(); return true;} break;
		}

		if(!CtrlOn)
			return CDialog::PreTranslateMessage(pMsg); 

		switch(pMsg->wParam)
		{
		case 'V': OnEditRedo(); break;
		case 'W': if(ShiftOn) HideAllHearingImpairedText(); else HideHearingImpairedTextSingle(); break;
		case 'O': LoadLastFile(ID_FILE_MRU0+0); break;
		case 'E': OnCropSubpic(); break;
		case 'U': OnOpenSup(); break;
		case 'H': OnHidesubpic(); break;
//		case 'I': m_SetupDialog->m_Italics = !m_SetupDialog->m_Italics; break;
		case 'A': OnEditApplyall(); break;
		case 'Q': 
			{
				if(CSubPic::m_ShowState == 0) CSubPic::m_ShowState = 1; 
				else if(CSubPic::m_ShowState == 1) CSubPic::m_ShowState = 2;
				else if (CSubPic::m_ShowState == 2) CSubPic::m_ShowState = 1; 
				ViewSPU(m_SPUSlider.GetPos());
				break;
			}
		case 'T': if(ShiftOn) OCRAll(); else OCRThisOne(); break;
		case 'Z': OnRevertsaved(); break;
		case 'J': OnOpenIFO(); break;
		case 'S': if(ShiftOn) OnSavePPM() ; else OnSaveall(); break;
#if 0
		case 'R': 
			m_SelectedSubPic->m_Modified=TRUE;ViewSPU(m_SPUSlider.GetPos()); 
			{
				FILE* fd = fopen("C:\\temp\\origencoded.bin","wb");
				fwrite(&m_SelectedSubPic->m_TopFieldOffset,sizeof(ushort),1,fd);
				fwrite(&m_SelectedSubPic->m_BotFieldOffset,sizeof(ushort),1,fd);
#if 0
				uchar* Ptr = m_SelectedSubPic->m_Data + m_SelectedSubPic->m_TopFieldOffset;
				fwrite(Ptr,m_SelectedSubPic->m_OffsetToCmd - m_SelectedSubPic->m_TopFieldOffset,1,fd);
#else
				uchar* Ptr = m_SelectedSubPic->m_Data;
				fwrite(Ptr,m_SelectedSubPic->m_SubPicSize,1,fd);
#endif
				fclose(fd);
			}			
			m_SelectedSubPic->DecodeOrig(); m_SelectedSubPic->RLEEncode(0); m_SelectedSubPic->RLEEncode(1); 
			m_SelectedSubPic->m_Modified=TRUE;ViewSPU(m_SPUSlider.GetPos()); 
			{
				FILE* fd = fopen("C:\\temp\\dvdsubencoded.bin","wb");
				fwrite(&m_SelectedSubPic->m_TopFieldOffset,sizeof(ushort),1,fd);
				fwrite(&m_SelectedSubPic->m_BotFieldOffset,sizeof(ushort),1,fd);
#if 0
				uchar* Ptr = m_SelectedSubPic->m_Data + m_SelectedSubPic->m_TopFieldOffset;
				fwrite(Ptr,m_SelectedSubPic->m_OffsetToCmd - m_SelectedSubPic->m_TopFieldOffset,1,fd);
#else
				uchar* Ptr = m_SelectedSubPic->m_Data;
				fwrite(Ptr,m_SelectedSubPic->m_SubPicSize,1,fd);
#endif
				fclose(fd);
			}			
			break;
#else
#endif
		case 'D': 
			m_ZoomSelect.SetCurSel(m_ZoomSelect.GetCurSel() > 0 ? 0 : 1);
			OnSelendokZoomsel(); break;
		case 'G': GetDlgItem(IDC_GOTO)->SetFocus(); GetDlgItem(IDC_GOTO)->SetWindowText("");break;
		case 'N': {if(m_SelectedSubPic == 0) break; CSubPic::m_SelButton++; if(CSubPic::m_SelButton >= m_SelectedSubPic->m_NumButtons) CSubPic::m_SelButton = 0; ViewSPU(m_SPUSlider.GetPos()); break;}
		case 'M': m_Preview->SetPlayAudioStream(m_Preview->GetPlayAudioStream()+ (ShiftOn ? -1 : 1)); break;
		case 'P': {if(m_SelectedSubPic == 0) break; CSubPic::m_SelButton--; if(CSubPic::m_SelButton < 0) CSubPic::m_SelButton = m_SelectedSubPic->m_NumButtons-1; ViewSPU(m_SPUSlider.GetPos()); break;}
#if 1
		case '0': CSubPic::m_SelButton = 9; ViewSPU(m_SPUSlider.GetPos());break;
		case '1': CSubPic::m_SelButton = 0; ViewSPU(m_SPUSlider.GetPos());break;
		case '2': CSubPic::m_SelButton = 1; ViewSPU(m_SPUSlider.GetPos());break;
		case '3': CSubPic::m_SelButton = 2; ViewSPU(m_SPUSlider.GetPos());break;
		case '4': CSubPic::m_SelButton = 3; ViewSPU(m_SPUSlider.GetPos());break;
		case '5': CSubPic::m_SelButton = 4; ViewSPU(m_SPUSlider.GetPos());break;
		case '6': CSubPic::m_SelButton = 5; ViewSPU(m_SPUSlider.GetPos());break;
		case '7': CSubPic::m_SelButton = 6; ViewSPU(m_SPUSlider.GetPos());break;
		case '8': CSubPic::m_SelButton = 7; ViewSPU(m_SPUSlider.GetPos());break;
		case '9': CSubPic::m_SelButton = 8; ViewSPU(m_SPUSlider.GetPos());break;
#endif
#ifdef _DEBUG
//		case 'Q': OCRVerbose = 8; OCRThisOne(); break;
		case 'B':
			{
			CommandID Foo[5] = {SET_COLOR,SET_CONTR,CMD_END,STP_DSP,STP_DSP};
//			CommandID Foo[5] = {STP_DSP,CMD_END,STP_DSP,SET_COLOR,SET_CONTR};
			ushort (*Foo2)[4] = new ushort[5][4]; memset(Foo2,0,3*4*2);
			Foo2[0][0] = 3;Foo2[0][1]=2;Foo2[0][2]=0;Foo2[0][3]=1;
			Foo2[1][1]=Foo2[1][2]=Foo2[1][3]=15;
			if(!ShiftOn)
				m_SelectedSubPic->AddDCSQT(0,400,Foo,Foo2,3);
			else
				m_SelectedSubPic->RemoveDCSQT(m_SelectedSubPic->m_SelectedDCSQT);
			ViewSPU(m_SPUSlider.GetPos()); 
			delete[] Foo2; 
			break;
			}
		case 'K': m_SelectedSubPic->AddFadeIn(); ViewSPU(m_SPUSlider.GetPos());break;
//		case 'K': m_SelectedSubPic->AddStop(); ViewSPU(m_SPUSlider.GetPos());break;
		case 'I': DoJustItalics = 1; OCRThisOne(); break;
//		case 'I': m_SelectedSubPic->MoveData(m_SelectedSubPic->m_SpaceBeforeCmd); ViewSPU(m_SPUSlider.GetPos());break;
//		case 'L': SaveIFO(); break;
		case 'F': TheSettings.OpenIniFile(); break;
		case 'L': m_SelectedSubPic->LoadBMP("C:\\temp\\foo.bmp"); ViewSPU(m_SPUSlider.GetPos());break;
#endif
		default: return CDialog::PreTranslateMessage(pMsg);
		}
		return TRUE;
	}
	else return CDialog::PreTranslateMessage(pMsg);
}

void CDVDSubEditDlg::OnRevertsaved() 
{
	if(!m_CanReload || m_Running)
		return;
	long start,end;
	m_OCROut.GetSel(start,end);
	if(m_SelectedSubPic != 0)
		m_SelectedSubPic->ReloadSPU();
	m_LastDispLBA = -1;			// This will force a redisplay of the image.
	SetMoveVSlider(MAX_V_MOVE);
	if(m_SelectedSubPic->m_OCRLines[0][0] != 0)
		OCRThisOne();
	UpdateSPUList(true);
	m_ChangeRecord.Reset();
	m_OCROut.SetSel(start,end);
}


void CDVDSubEditDlg::OnPvcid() 
{
	if(m_VCIDSelect.GetCurSel() < 1) return;
	m_VCIDSelect.SetCurSel(m_VCIDSelect.GetCurSel()-1);
	OnSelchangeStreamselect();
}

void CDVDSubEditDlg::OnNvcid() 
{
	if(m_VCIDSelect.GetCurSel() >= m_VCIDLength) return;
	m_VCIDSelect.SetCurSel(m_VCIDSelect.GetCurSel()+1);
	OnSelchangeStreamselect();
}

void CDVDSubEditDlg::OnPstream() 
{
	if(m_StreamSelect.GetCurSel() < 1) return;
	m_StreamSelect.SetCurSel(m_StreamSelect.GetCurSel()-1);
	OnSelchangeStreamselect();
}

void CDVDSubEditDlg::OnNstream() 
{
	if(m_StreamSelect.GetCurSel() >= m_TopStreamID+1) return;
	m_StreamSelect.SetCurSel(m_StreamSelect.GetCurSel()+1);
	OnSelchangeStreamselect();
}



void CDVDSubEditDlg::OnShowvideo() 
{
	m_DisplayVideo = m_ShowVideo.GetCheck();
	m_IFrameSelect.ShowWindow(m_DisplayVideo ? SW_SHOW : SW_HIDE);
	// Hide the preview window!
	::ShowWindow(m_PreviewHandle,m_DisplayVideo?SW_SHOW:SW_HIDE);
	// Show the current SPU.
	m_LastDispLBA = -1;
	ViewSPU(m_SPUSlider.GetPos());	
}


void CDVDSubEditDlg::OnShoworigocr() 
{
	if(m_SelectedSubPic == 0)
		return;
	DispOCR(m_SelectedSubPic);	
}

void CDVDSubEditDlg::OnClose() 
{
	if(WarnIfModifs() == false)
		return;
	// Saving preferences here because ScanDialog is a child and will be destroyed before OnDestroy.
	TheSettings.SaveSettings();
	char BMPEditPath[512];
	strcpy(BMPEditPath,(LPCTSTR) m_SetupDialog->m_BitmapEditTool);
	TheSettings.SaveSettings("BMPEditPath",BMPEditPath);
	strcpy(BMPEditPath,(LPCTSTR) m_SetupDialog->m_BitmapEditToolArgs);
	TheSettings.SaveSettings("BMPEditPathArgs",BMPEditPath);
	AbortPlayback(1);
	CDialog::OnClose();
}

bool CDVDSubEditDlg::WarnIfModifs()
{
	if(!m_CanSave)
		return true;
	if(CheckForModifications() && 
		AfxMessageBox("You have modified some of the subpics or some of the CLUTs... \nDo you want to close without saving?",MB_YESNO) != IDYES)
		return false;
	return true;
}

void CDVDSubEditDlg::OnDefbutton() 
{
	CWnd* Ptr = SetFocus();
	if(Ptr == GetDlgItem(IDC_SEARCH))
	{
		GetDlgItem(IDC_FINDR)->SetFocus();
		OnFindRight();
		return;
	}
	if(Ptr == GetDlgItem(IDC_SEARCHSELECT))
	{
		CString Temp;
		GetDlgItemText(IDC_SEARCHSELECT,Temp);
		bool Active =  Temp != "";
		m_ShowSearched.SetCheck(Active);
		UpdateSPUList();
		return;
	}
}

void CDVDSubEditDlg::OnKillfocusMoveude() 
{
	int move = GetDlgItemInt(IDC_MOVEUDE);
	move = min(MAX_V_MOVE,move);
	move = max(-MAX_V_MOVE,move);
	SetMoveVSlider(MAX_V_MOVE+move);
	if(!m_SelectedSubPic)
		return ;
	m_SelectedSubPic->VMoveSubPic(m_MoveVSlider.GetPos()-MAX_V_MOVE); 
	m_ChangeRecord.RecordChange(m_SelectedSubPic, ChangeVMove, m_MoveVSlider.GetPos()-MAX_V_MOVE);
	ViewSPU(m_SPUSlider.GetPos());
}

void CDVDSubEditDlg::OnKillfocusMovelre() 
{
	int move = GetDlgItemInt(IDC_MOVELRE);
	move = min(MAX_H_MOVE,move);
	move = max(-MAX_H_MOVE,move);
	SetMoveHSlider(MAX_H_MOVE+move);
	if(!m_SelectedSubPic)
		return ;
	m_SelectedSubPic->HMoveSubPic(m_MoveHSlider.GetPos()-MAX_H_MOVE); 
	m_ChangeRecord.RecordChange(m_SelectedSubPic, ChangeHMove, m_MoveHSlider.GetPos()-MAX_H_MOVE);
	ViewSPU(m_SPUSlider.GetPos());
}

void CDVDSubEditDlg::OnKillfocusBordere() 
{
	int border = GetDlgItemInt(IDC_BORDERE);
	border = min(border,MAX_BORDER);
	border = max(border,0);
	SetBorderSlider(border);
	if(!m_SelectedSubPic)
		return ;
	m_SelectedSubPic->VMoveSubPicInFrame(border,m_FlushBorders.GetCheck()); 
	m_ChangeRecord.RecordChange(m_SelectedSubPic, ChangeBorder, border);
	ViewSPU(m_SPUSlider.GetPos());
}

void CDVDSubEditDlg::OnKillfocusDuratione() 
{
	char Temp[1024];
	GetDlgItemText(IDC_DURATIONE,Temp,1024);
	float Duration;
	sscanf(Temp,"%f",&Duration);
	if(!m_SelectedSubPic)
		return ;
	m_SelectedSubPic->ChangeDCSQTDelay((int)(Duration*1000));
	m_ChangeRecord.RecordChange(m_SelectedSubPic, ChangeDur, m_SelectedSubPic->GetDCSQTDelay());
	ViewSPU(m_SPUSlider.GetPos());
}

void CDVDSubEditDlg::OnKillfocusGoto() 
{
	int Goto = GetDlgItemInt(IDC_GOTO);
	// Look for this guy in our list!
	int i;
	for(i=0;i<m_PruneLength;i++)
	{
		if((m_SelectedStream == 0 || m_TopStreamID == 0 ? m_PrunedList[i]->m_SPUNumber : m_PrunedList[i]->m_SPUNumberInStream ) == Goto)
		{
			m_SPUSlider.SetPos(i);
			ViewSPU(i);
			return;
		}
	}
	MessageBeep(MB_ICONEXCLAMATION);
}


void CDVDSubEditDlg::OnSelchangePixselect() 
{
	ViewSPU(m_SPUSlider.GetPos());
}

void CDVDSubEditDlg::OnSelchangeStart() 
{
	if(!m_SelectedSubPic)
		return ;
	// Change the start command according to the selector...
	if(m_StartCommand.GetCurSel() == 0)
		m_SelectedSubPic->ChangeCommand(FSTA_DSP,STA_DSP);
	else
		m_SelectedSubPic->ChangeCommand(STA_DSP,FSTA_DSP);
	// Be smart, if se change the start command, we might want to apply to all.
	m_ChangeRecord.RecordChange(m_SelectedSubPic, ChangeStart, m_StartCommand.GetCurSel());
	ViewSPU(m_SPUSlider.GetPos(),false);	
}

void CDVDSubEditDlg::OnSelchangeBorderselect() 
{
	if(!m_SelectedSubPic)
		return ;
	switch(m_BorderSelect.GetCurSel())
	{
	case BORDER_CUSTOM:
		m_SelectedSubPic->VMoveSubPicInFrame(0,m_FlushBorders.GetCheck());
		m_ChangeRecord.RecordChange(m_SelectedSubPic, ChangeBorder, 0);
		ViewSPU(m_SPUSlider.GetPos());
		break;
	case BORDER_SAFE: 
		// I'm using 8% for the safe area.
		m_SelectedSubPic->VMoveSubPicInFrame((VideoFormat == FORMAT_NTSC ? m_Border4_3S_N : m_Border4_3S_P),m_FlushBorders.GetCheck());
		m_ChangeRecord.RecordChange(m_SelectedSubPic, ChangeBorder, (VideoFormat == FORMAT_NTSC ? m_Border4_3S_N : m_Border4_3S_P));
		ViewSPU(m_SPUSlider.GetPos());
		break;
	case BORDER_4_3_L: 
		// 16:9 Letterboxed.
		m_SelectedSubPic->VMoveSubPicInFrame((VideoFormat == FORMAT_NTSC ? m_Border4_3LB_N : m_Border4_3LB_P),m_FlushBorders.GetCheck());
		m_ChangeRecord.RecordChange(m_SelectedSubPic, ChangeBorder, (VideoFormat == FORMAT_NTSC ? m_Border4_3LB_N : m_Border4_3LB_P));
		ViewSPU(m_SPUSlider.GetPos());
		break;
	case BORDER_4_3_LS: 
		// 16:9 letterboxed safe.
		m_SelectedSubPic->VMoveSubPicInFrame((VideoFormat == FORMAT_NTSC ? m_Border4_3LBS_N : m_Border4_3LBS_P),m_FlushBorders.GetCheck());
		m_ChangeRecord.RecordChange(m_SelectedSubPic, ChangeBorder, (VideoFormat == FORMAT_NTSC ? m_Border4_3LBS_N : m_Border4_3LBS_P));
		ViewSPU(m_SPUSlider.GetPos());
		break;
	case BORDER_1_85_L: 
		// 1.85:1 in 16:9 safe.
		m_SelectedSubPic->VMoveSubPicInFrame((VideoFormat == FORMAT_NTSC ? m_Border1_85_N : m_Border1_85_P),m_FlushBorders.GetCheck());
		m_ChangeRecord.RecordChange(m_SelectedSubPic, ChangeBorder, (VideoFormat == FORMAT_NTSC ? m_Border1_85_N : m_Border1_85_P));
		ViewSPU(m_SPUSlider.GetPos());
		break;
	case BORDER_2_39_L: 
		// 2.39:1 in 16:9 safe.
		m_SelectedSubPic->VMoveSubPicInFrame((VideoFormat == FORMAT_NTSC ? m_Border2_39_N : m_Border2_39_P),m_FlushBorders.GetCheck());
		m_ChangeRecord.RecordChange(m_SelectedSubPic, ChangeBorder, (VideoFormat == FORMAT_NTSC ? m_Border2_39_N : m_Border2_39_P));
		ViewSPU(m_SPUSlider.GetPos());
		break;
	}
}



void CDVDSubEditDlg::OnSelchangeDcsqtselect() 
{
	if(!m_SelectedSubPic)
		return ;

	bool All = m_DCSQTSelect.GetCurSel() == m_DCSQTSelect.GetCount()-1;
	int Select = All ? 0 : m_DCSQTSelect.GetCurSel();
	m_SelectedSubPic->SelectDCSQT(Select);
	m_SelectedSubPic->m_ChangeAllDCSQT = All;
	m_ChangeRecord.m_ApplyToAllDCSQTs = All;
	
	int i;
	for(i=0;i<m_PruneLength && All == 0;i++)
		m_PrunedList[i]->SelectDCSQT(Select);

	ViewSPU(m_SPUSlider.GetPos(),true,false);
}


void CDVDSubEditDlg::OnSelchangeDisplaymode() 
{
		
	switch(m_DispModeSelect.GetCurSel())
	{
		case 0: m_AspectRatio = 2; m_WSDisplayMode = WSDispNormal;
			JustFoundVideoFormat(); break ;
		case 1: m_AspectRatio = 3; m_WSDisplayMode = WSDispNormal;
			JustFoundVideoFormat(); break ;
		case 2: m_AspectRatio = 2; m_WSDisplayMode = WSDispLB;
			JustFoundVideoFormat(); break ;
		case 3: m_AspectRatio = 2; m_WSDisplayMode = WSDispPS;
			JustFoundVideoFormat(); break ;
	}
}


void CDVDSubEditDlg::OnDeltaposMoveb(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
	SetMoveVSlider(m_MoveVSlider.GetPos()-pNMUpDown->iDelta*(CSubPic::m_ForceEvenMove?2:1));
	if(m_SelectedSubPic)
	{
		m_SelectedSubPic->VMoveSubPic(m_MoveVSlider.GetPos()-MAX_V_MOVE);
		m_ChangeRecord.RecordChange(m_SelectedSubPic, ChangeVMove, m_MoveVSlider.GetPos()-MAX_V_MOVE);
		ViewSPU(m_SPUSlider.GetPos());
	}
	*pResult = 0;
}

void CDVDSubEditDlg::OnDeltaposMovelrb(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
	SetMoveHSlider(m_MoveHSlider.GetPos()-pNMUpDown->iDelta);
	if(m_SelectedSubPic)
	{
		m_SelectedSubPic->HMoveSubPic(m_MoveHSlider.GetPos()-MAX_H_MOVE);
		m_ChangeRecord.RecordChange(m_SelectedSubPic, ChangeHMove, m_MoveHSlider.GetPos()-MAX_H_MOVE);
		ViewSPU(m_SPUSlider.GetPos());
	}
	*pResult = 0;
}


void CDVDSubEditDlg::OnDeltaposBorderb(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
	SetBorderSlider(m_BorderSlider.GetPos()-pNMUpDown->iDelta);
	if(m_SelectedSubPic)
	{
		m_SelectedSubPic->VMoveSubPicInFrame(m_BorderSlider.GetPos(),m_FlushBorders.GetCheck());
		ViewSPU(m_SPUSlider.GetPos());
		m_ChangeRecord.RecordChange(m_SelectedSubPic, ChangeBorder, m_BorderSlider.GetPos());
	}
	*pResult = 0;
}


void CDVDSubEditDlg::OnFlushBorder() 
{
	if(m_SelectedSubPic && m_BorderSlider.GetPos() != 0 && m_FlushBorders.GetCheck())
	{
		m_SelectedSubPic->VMoveSubPicInFrame(m_BorderSlider.GetPos(),m_FlushBorders.GetCheck());
		ViewSPU(m_SPUSlider.GetPos());
		m_ChangeRecord.RecordChange(m_SelectedSubPic, ChangeBorder, m_BorderSlider.GetPos());
	}
}


void CDVDSubEditDlg::OnDeltaposDscqtb(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
	// TODO: Add your control notification handler code here
	m_DCSQTSelect.SetCurSel(max(0,m_DCSQTSelect.GetCurSel()+pNMUpDown->iDelta));
	OnSelchangeDcsqtselect();
	*pResult = 0;
}

void CDVDSubEditDlg::OnHidesubpic() 
{
	if(!m_SelectedSubPic)
		return;
#ifdef ERASE_WHEN_HIDE
	m_ChangeRecord.RecordChange(m_SelectedSubPic, ChangeHide, 0);
	m_SelectedSubPic->EraseAll();
#else
	m_SelectedSubPic->ChangeTransparency(0,5);
	m_ChangeRecord.RecordChange(m_SelectedSubPic,ChangeTransp,0,0);
	m_ChangeRecord.RecordChange(m_SelectedSubPic,ChangeTransp,0,1);
	m_ChangeRecord.RecordChange(m_SelectedSubPic,ChangeTransp,0,2);
	m_ChangeRecord.RecordChange(m_SelectedSubPic,ChangeTransp,0,3);
#endif
	ViewSPU(m_SPUSlider.GetPos());
}






// __________________________ MENU STUFF _______________________


void CDVDSubEditDlg::OnHelpAbout() 
{
	CAboutDlg dlgAbout;
	dlgAbout.m_Edit = this;
	dlgAbout.DoModal();
}

void CDVDSubEditDlg::OnHelpManual() 
{
	OpenRSCFile(IDR_HELPFILE, ".pdf");
}

void CDVDSubEditDlg::OnHelpOntheweb() 
{
	ShellExecute(NULL, "open", "http://www.videohelp.com/~DVDSubEdit", NULL, NULL, SW_SHOWNORMAL);
}

void CDVDSubEditDlg::OnEditRedo() 
{
	if(m_SelectedSubPic == 0)
		return;
	if(m_ChangeRecord.m_VMove != NO_CHANGE)
		m_SelectedSubPic->VMoveSubPic(m_ChangeRecord.m_VMove);
	if(m_ChangeRecord.m_HMove != NO_CHANGE)
		m_SelectedSubPic->HMoveSubPic(m_ChangeRecord.m_HMove);
	if(m_ChangeRecord.m_Border != 0)
		m_SelectedSubPic->VMoveSubPicInFrame(m_ChangeRecord.m_Border,m_FlushBorders.GetCheck());
	if(m_ChangeRecord.m_TranspPixel != 0)
	{
		m_SelectedSubPic->m_ChangeAllDCSQT = m_ChangeRecord.m_ApplyToAllDCSQTs != 0;
		if(m_SelectedSubPic->m_ChangeAllDCSQT == 0 && m_ChangeRecord.m_DCSQT >= m_SelectedSubPic->m_DCSQT.size())
			return;
		if(m_SelectedSubPic->m_ChangeAllDCSQT == 0) m_SelectedSubPic->SelectDCSQT(m_ChangeRecord.m_DCSQT);
		ChangeTransparencies(m_ChangeRecord.m_Transparency,m_ChangeRecord.m_TranspPixel,m_SelectedSubPic);
	}
	if(m_ChangeRecord.m_ColorPixel != 0)
	{
		m_SelectedSubPic->m_ChangeAllDCSQT = m_ChangeRecord.m_ApplyToAllDCSQTs != 0;
		if(m_SelectedSubPic->m_ChangeAllDCSQT == 0 && m_ChangeRecord.m_DCSQT >= m_SelectedSubPic->m_DCSQT.size())
			return;
		if(m_SelectedSubPic->m_ChangeAllDCSQT == 0) m_SelectedSubPic->SelectDCSQT(m_ChangeRecord.m_DCSQT);
		ChangeColors(m_ChangeRecord.m_Color,m_ChangeRecord.m_ColorPixel,m_SelectedSubPic);
	}
	if(m_ChangeRecord.m_Crop != NO_CHANGE)
		m_SelectedSubPic->CropBackground();
	if(m_ChangeRecord.m_StartCommand != NO_CHANGE)
	{
		if(m_ChangeRecord.m_StartCommand == 0)
			m_SelectedSubPic->ChangeCommand(FSTA_DSP,STA_DSP);
		else
			m_SelectedSubPic->ChangeCommand(STA_DSP,FSTA_DSP);
	}
	if(m_ChangeRecord.m_Duration != NO_CHANGE)
		m_SelectedSubPic->ChangeDCSQTDelay(m_ChangeRecord.m_Duration);
	if(m_ChangeRecord.m_MakeInfinite != NO_CHANGE)
		m_SelectedSubPic->RemoveDCSQT(m_SelectedSubPic->m_DCSQT.size()-1);
	if(m_ChangeRecord.m_AddFadeIn != NO_CHANGE)
	{
		if(m_ChangeRecord.m_AddFadeIn == 1) m_SelectedSubPic->AddFadeIn(); 
		else m_SelectedSubPic->RemoveFadeIn();
	}
	if(m_ChangeRecord.m_Hide != NO_CHANGE)
	{
		m_SelectedSubPic->EraseAll(); 
	}
	ViewSPU(m_SPUSlider.GetPos());
}

void CDVDSubEditDlg::OnExit() 
{
    AfxGetMainWnd()->SendMessage(WM_CLOSE);
}

void CDVDSubEditDlg::OnHelpViewlog() 
{
	ShellExecute(NULL, "open", m_LogPath, NULL, NULL, SW_SHOWNORMAL);
}


void CDVDSubEditDlg::OnEditApplyall() 
{
	if(m_SelectedSubPic == 0)
		return;
	m_ApplyAllDialog->m_ApplyVMove = m_ChangeRecord.m_VMove != NO_CHANGE;
	m_ApplyAllDialog->m_ApplyHMove = m_ChangeRecord.m_HMove != NO_CHANGE;
	m_ApplyAllDialog->m_ApplyBorders = m_ChangeRecord.m_Border != 0;
	m_ApplyAllDialog->m_ApplyStart = m_ChangeRecord.m_StartCommand != NO_CHANGE;
	m_ApplyAllDialog->m_ApplyTransp = m_ChangeRecord.m_TranspPixel != 0;
	m_ApplyAllDialog->m_ApplyColor = m_ChangeRecord.m_ColorPixel != 0;
	m_ApplyAllDialog->m_ApplyDuration = m_ChangeRecord.m_Duration != NO_CHANGE;
	m_ApplyAllDialog->m_ApplyCrop = m_ChangeRecord.m_Crop != NO_CHANGE;
	CString Temp;
	Temp.Format("%.3f s",(float)max(0,m_ChangeRecord.m_Duration)/1000);
	m_ApplyAllDialog->m_Duration = Temp;
	m_ApplyAllDialog->m_ApplyFade = m_ChangeRecord.m_AddFadeIn != NO_CHANGE;
	m_ApplyAllDialog->m_ApplyHide = m_ChangeRecord.m_Hide != NO_CHANGE;
	OnApplytoall();
}


void CDVDSubEditDlg::OnOpenVob() 
{
	if(WarnIfModifs() == false)
		return;
	CFileDialog FileDialog(TRUE, CString("*.vob"), "", OFN_FILEMUSTEXIST, CString("VOB Files (*.vob)|*.vob||"));
	FileDialog.m_ofn.lpstrTitle = "Open VOB file";
	FileDialog.m_ofn.lpstrInitialDir = m_LastVOBDir;
	FileDialog.m_ofn.Flags |= OFN_ALLOWMULTISELECT;
	if (FileDialog.DoModal() == IDOK)
	{
		Reset();
		POSITION pos ( FileDialog.GetStartPosition() );
		while( pos )
			AddFile((char*)(LPCTSTR)FileDialog.GetNextPathName(pos));
		EnableUI(true);
		// I need to do that so preview indeed opens the first file!
		SortInputFiles();
		// Now scan input file...
		m_ScanDialog->DisplayInfo(0,"","");
		m_ScanDialog->ShowWindow(SW_SHOW);
		if(m_AutoOpenVideo) {m_ShowVideo.SetCheck(1);m_DisplayVideo=1;}
		OnShowvideo();
		::ShowWindow(m_PreviewHandle,SW_HIDE);
		m_Preview->LoadFile(m_FileInfos[0].m_FileName);
		StartScan();
		strcpy(m_LastVOBDir,GetDirPath(m_FileInfos[0].m_FileName));
	}
}

bool CDVDSubEditDlg::OpenDomain(char* Path)
{
	Reset();
	int Index = -1;
	int VTS = -1;
	char FileName[1024];
	strcpy(FileName,Path);
	// Get VTS number and index.
	if(sscanf(GetFname(FileName),"%*[^_]_%d_%d",&VTS,&Index) == 2 && Index > 0)
	{
		// This is a vts_nn_x.vob
		for(int i=1;i<10;i++)
		{
			FileName[strlen(FileName)-5]='0'+i;
			if(PathExists(FileName))
				AddFile(FileName);
		}
	}
	else
		AddFile(FileName);
	EnableUI(true);
	// Now scan input file...
	m_ScanDialog->DisplayInfo(0,"","");
	m_ScanDialog->ShowWindow(SW_SHOW);
	m_CanMove = true;
	if(m_AutoOpenVideo) {m_ShowVideo.SetCheck(1);m_DisplayVideo=1;}
	OnShowvideo();
	::ShowWindow(m_PreviewHandle,SW_HIDE);
	m_Preview->LoadFile(m_FileInfos[0].m_FileName);	
	StartScan();
	strcpy(m_LastVOBDir,Path);
	return true;
}

void CDVDSubEditDlg::OnOpenDomain() 
{
	if(WarnIfModifs() == false)
		return;
	CFileDialog FileDialog(TRUE, CString("*.vob"), "", OFN_FILEMUSTEXIST, CString("VOB Files (*.vob)|*.vob||"));
	FileDialog.m_ofn.lpstrTitle = "Open Full Domain";
	FileDialog.m_ofn.lpstrInitialDir = m_LastVOBDir;
	if (FileDialog.DoModal() == IDOK)
		OpenDomain((char*)(LPCTSTR)FileDialog.GetPathName());
}

void CDVDSubEditDlg::OnSavePPM() 
{
	bool ShiftOn = (GetAsyncKeyState(VK_SHIFT) & 0x8000) != 0;
	if(m_SelectedSubPic == 0)
		return;
	CFileDialog FileDialog(FALSE, CString("*.ppm"), "", OFN_OVERWRITEPROMPT, 
		CString("ppm Files (*.ppm)|*.ppm||"));
	FileDialog.m_ofn.lpstrTitle = "Save subpic as PPM file";
	FileDialog.m_ofn.lpstrInitialDir = m_LastPPMDir;
	if (FileDialog.DoModal() == IDOK)
	{
		if(ShiftOn)
		{
			uchar FillPix = 0;
			for(int i=0;i<4;i++)
				if(m_SelectedSubPic->m_NumPix[i] == 0) FillPix = i;
			FillMissingPix(m_SelectedSubPic->m_Pixels,
				m_SelectedSubPic->m_SubPicWidth,m_SelectedSubPic->m_SubPicHeight,
				m_SelectedSubPic->m_BodyPix,m_SelectedSubPic->m_BackgroundPix,FillPix);
		}
		m_SelectedSubPic->SavePicAsPPM((LPCTSTR)FileDialog.GetPathName(),GetCLUT(),1,1,0);

		strcpy(m_LastPPMDir,GetDirPath((LPCTSTR)FileDialog.GetPathName()));
	}
}


class MySaveDlg: public CFileDialog
{
public:
	int m_SaveType;
	MySaveDlg::MySaveDlg(BOOL bOpenFileDialog, LPCTSTR lpszDefExt = NULL, LPCTSTR lpszFileName = NULL, DWORD dwFlags = OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, LPCTSTR lpszFilter = NULL, CWnd* pParentWnd = NULL )
		: CFileDialog(bOpenFileDialog, lpszDefExt , lpszFileName , dwFlags, lpszFilter, pParentWnd)
	{
		m_SaveType = 0;
	}
	void OnTypeChange()
	{
		 const MSG* pMSG= GetCurrentMessage();
		 ASSERT( pMSG );
		 LPOFNOTIFY pNotif= (LPOFNOTIFY)pMSG->lParam;
		 m_SaveType = pNotif->lpOFN->nFilterIndex - 1;
	};
	void SetSaveType(int AType) {m_SaveType = AType; m_ofn.nFilterIndex = AType+1;};
};

void CDVDSubEditDlg::OnSaveBMP() 
{
	if(m_SelectedSubPic == 0)
		return;
	MySaveDlg FileDialog(FALSE, CString("*.bmp"), "", OFN_OVERWRITEPROMPT, 
		CString("24bit bmp Files (*.bmp)|*.bmp|4bit bmp Files (*.bmp)|*.bmp|ppm Files (*.ppm)|*.ppm||"));
	FileDialog.m_ofn.lpstrTitle = "Save subpic as bitmap file";
	FileDialog.m_ofn.lpstrInitialDir = m_LastPPMDir;
	FileDialog.m_ofn.Flags |= OFN_EXPLORER ;
	FileDialog.SetSaveType(m_LastBitmapSaveType);
	char DefaultName[1025];
	sprintf(DefaultName,"%ssubpic_0x%02x_%03d",m_LastPPMDir,32+m_SelectedSubPic->m_StreamID,
		m_SelectedSubPic->m_SPUNumberInStream);
	strcpy(FileDialog.m_ofn.lpstrFile,DefaultName);
	if (FileDialog.DoModal() == IDOK)
	{
		int SaveType = FileDialog.m_SaveType;
		m_LastBitmapSaveType = SaveType;
		if(SaveType == 0) m_SelectedSubPic->SavePicAsBMP((LPCTSTR)FileDialog.GetPathName(),24, GetCLUT(), 0);
		if(SaveType == 1) m_SelectedSubPic->SavePicAsBMP((LPCTSTR)FileDialog.GetPathName(),4,GetCLUT(), 0);
		if(SaveType == 2) m_SelectedSubPic->SavePicAsPPM((LPCTSTR)FileDialog.GetPathName(),GetCLUT(),1,1,0);

		strcpy(m_LastPPMDir,GetDirPath((LPCTSTR)FileDialog.GetPathName()));
	}
}

void CDVDSubEditDlg::OnSaveVideoFrame() 
{
	if(m_SelectedSubPic == 0)
		return;
	CFileDialog FileDialog(FALSE, CString("*.bmp"), "", OFN_OVERWRITEPROMPT, 
		CString("bmp Files (*.bmp)|*.bmp||"));
	FileDialog.m_ofn.lpstrTitle = "Save video frame as bitmap (.bmp) file";
	FileDialog.m_ofn.lpstrInitialDir = m_LastPPMDir;
	if (FileDialog.DoModal() == IDOK)
	{
		m_Preview->SaveBMPFile((char*)(LPCTSTR)FileDialog.GetPathName());
		strcpy(m_LastPPMDir,GetDirPath((LPCTSTR)FileDialog.GetPathName()));
	}
}

void CDVDSubEditDlg::OnLoadBMP() 
{
	if(m_SelectedSubPic == 0)
		return;
	CFileDialog FileDialog(TRUE, CString("*.bmp"), "", NULL, 
		CString("bmp Files (*.bmp)|*.bmp||"));
	FileDialog.m_ofn.lpstrTitle = "Load replacement bitmap (.bmp) file";
	FileDialog.m_ofn.lpstrInitialDir = m_LastPPMDir;
	char DefaultName[1025];
	sprintf(DefaultName,"%ssubpic_0x%02x_%03d",m_LastPPMDir,32+m_SelectedSubPic->m_StreamID,
		m_SelectedSubPic->m_SPUNumberInStream);
	strcpy(FileDialog.m_ofn.lpstrFile,DefaultName);
	if (FileDialog.DoModal() == IDOK)
	{
		m_SelectedSubPic->LoadBMP((LPCTSTR)FileDialog.GetPathName(),GetCLUT(), 0);
		strcpy(m_LastPPMDir,GetDirPath((LPCTSTR)FileDialog.GetPathName()));
		ViewSPU(m_SPUSlider.GetPos());
	}
}

void CDVDSubEditDlg::OnExportImportBMP()
{
	int PrevSel = m_CLUTSelect.GetCurSel();
	if(m_SelectedSubPic == 0)
		return;
	if(m_SetupDialog->m_BitmapEditTool == "")
	{
		ERROR1("You haven't indicated which tool to use to edit the subpic bitmap\nI will open the preference dialog to let you do that");
		m_SetupDialog->DoModal();
		if(m_SetupDialog->m_BitmapEditTool == "")
			return;
	}
	char* TempName = CatPath(getenv("TEMP"),"tempbitmap.bmp");
	if(m_SelectedSubPic->CheckForIdenticalColors(GetCLUT(), 0) == false)
	{
		ERROR1("At least 2 pixel types have the same color!\nThis could create problems when the bitmap is re-imported into DVDSubEdit.\nI will use the \"Automatic Clut\" instead of the IFO Clut");
		m_CLUTSelect.SetCurSel(1);
	}
	if(m_SelectedSubPic->SavePicAsBMP(TempName,m_SetupDialog->m_EditBitmapAs24 ? 24 : 4, GetCLUT(), 0) != 0)
	{
		ERROR1("Couldn't save subpic as BMP! sorry... "); 
		return;	
	}
	if(m_SetupDialog->m_BitmapEditTool != "")
	{
		HANDLE EditHandle = StartApp(m_SetupDialog->m_BitmapEditTool, CatStr(m_SetupDialog->m_BitmapEditToolArgs, " ", TempName));
		while(WaitForSingleObject(EditHandle, 100) == WAIT_TIMEOUT)
		{
			Sleep(100);
		}
		CloseHandle(EditHandle);
	}	
	m_SelectedSubPic->LoadBMP(TempName,GetCLUT(), 0);
	m_CLUTSelect.SetCurSel(PrevSel);
	remove(TempName);
	ViewSPU(m_SPUSlider.GetPos());
}


void CDVDSubEditDlg::OnSaveSup() 
{
	if(m_StreamSelect.GetCurSel() == 0 && m_StreamSelect.GetCount() > 2)
	{
		ERROR1("You did not select a specific subpicture stream.\nSelect one first before saving as sup file.");
		return;
	}
	CFileDialog FileDialog(FALSE, CString("*.sup"), "", OFN_OVERWRITEPROMPT, 
		CString("sup Files (*.sup)|*.sup||"));
	FileDialog.m_ofn.lpstrTitle = "Save subpics as .sup file";
	FileDialog.m_ofn.lpstrInitialDir = m_LastSaveSUPDir;

	if (FileDialog.DoModal() == IDOK)
	{
		SaveSelectedAsSupFile((LPCTSTR)FileDialog.GetPathName());
		strcpy(m_LastSaveSUPDir,GetDirPath((LPCTSTR)FileDialog.GetPathName()));
	}
}

void CDVDSubEditDlg::OnOpenSup() 
{
	if(WarnIfModifs() == false)
		return;
	CFileDialog FileDialog(TRUE, CString("*.sup"), "", OFN_FILEMUSTEXIST, 
		CString("sup Files (*.sup)|*.sup||"));
	FileDialog.m_ofn.lpstrTitle = "Load .sup file";
	FileDialog.m_ofn.lpstrInitialDir = m_LastOpenSUPDir;
	if (FileDialog.DoModal() == IDOK)
	{
		::ShowWindow(m_PreviewHandle,SW_HIDE);
		if(LoadSUPFile((LPCTSTR)FileDialog.GetPathName()) != 0)
			return;
		m_ShowModified.SetCheck(false);
		OnSelchangeStreamselect();
		m_ShowVideo.SetCheck(0);m_DisplayVideo=0;
		EnableUI(true);
		SetWindowText((LPCTSTR)FileDialog.GetPathName());
		m_Preview->SetDisplayMode(-1);
		m_CLUTSelect.SetCurSel(1);
		strcpy(m_DefIFOPath,GetDirPath((LPCTSTR)FileDialog.GetPathName()));
		strcpy(m_LastOpenSUPDir,GetDirPath((LPCTSTR)FileDialog.GetPathName()));
		if(m_SetupDialog->m_AskForIFO)
			OnOpenIFO();
	}
}

void CDVDSubEditDlg::OnSaveOCR() 
{
	CFileDialog FileDialog(FALSE, CString("*.txt"), "", OFN_OVERWRITEPROMPT, 
		CString("(*.srt)|*.srt||"));
	FileDialog.m_ofn.lpstrTitle = "Save OCR output as .srt file";
	FileDialog.m_ofn.lpstrInitialDir = m_LastSRTDir;
	if (FileDialog.DoModal() == IDOK)
	{
		FILE* fd = fopen((LPCTSTR)FileDialog.GetPathName(),"w");
		if(fd == 0)
		{
			ERROR1("Couldn't open file for writing!");
			return;
		}
		int i,j;
		for(i=0;i<m_PruneLength;i++)
		{
			CSubPic* Temp = m_PrunedList[i];
			if(Temp->m_OCRLines[0][0] == '\0' || Temp->GetMeanTransparency() == 0)
				continue;
			// srt files start at 1 apparently.
			fprintf(fd,"%d\n%s\n",i+1,PrintPTS(Temp->m_TruePTS,Temp->GetDCSQTDelay()));
			for(j=0;j<32;j++)
			{
				if(Temp->m_OCRLines[j][0] == '\0')
					break;
				fprintf(fd,"%s\n",Temp->m_OCRLines[j]);
			}
			fprintf(fd,"\n");
		}
		fclose(fd);
		strcpy(m_LastSRTDir,GetDirPath((LPCTSTR)FileDialog.GetPathName()));
	}
}

void CDVDSubEditDlg::OnTimingSingle() 
{
	static bool First = true;
	m_MoveDialog->m_RetimeAll = false;
	if(!m_CanMove)
	{
		AfxMessageBox("Retiming subtitles is only possible in the title domain\nand if the entire domain is loaded.\
\nNot available for .sup files, or if you used the MRU list to open the files.");
		return;
	}
	m_ShowEndFrame.SetCheck(0);
	if(m_SelectedSubPic == 0 || m_MoveDialog->DoModal() != IDOK)
		return;
	if(First && AfxMessageBox("There is NO undo for this\nThe VOB file will be modified. Proceed?",MB_OKCANCEL) != IDOK)
		return;
	First = false;
	if(m_MoveDialog->m_CurSel == 0)
		MoveThisOne(m_SelectedSubPic->m_SPUNumber, m_MoveDialog->m_ChangeTimingBy);
	else if(m_MoveDialog->m_CurSel == 1)
	{
		float DeltaSeconds = m_MoveDialog->m_ChangeTimingBy - PTSToSecs(m_SelectedSubPic->m_PTS);
		MoveThisOne(m_SelectedSubPic->m_SPUNumber, DeltaSeconds);
	}
	UpdateSPUList(true);
	CloseOpenAllFiles();
	AddFilesToMRU();
}


void CDVDSubEditDlg::OnInitMenu( CMenu* pMenu )
{
	HMENU HMenu = pMenu->GetSafeHmenu( );

	
	EnableMenuItem(HMenu,ID_FILE_OPENIFO,(m_MenuEnabled && m_SupFileMode)?MF_ENABLED:MF_GRAYED);
	EnableMenuItem(HMenu,ID_FILE_OPENVOB,m_MenuEnabled?MF_ENABLED:MF_GRAYED);
	EnableMenuItem(HMenu,ID_EDIT_REVERTTHIS,(m_MenuEnabled && m_CanReload)?MF_ENABLED:MF_GRAYED);
	EnableMenuItem(HMenu,ID_EDIT_REVERTALL,(m_MenuEnabled && m_CanReload)?MF_ENABLED:MF_GRAYED);
	EnableMenuItem(HMenu,ID_EDIT_APPLYALL, m_MenuEnabled?MF_ENABLED:MF_GRAYED);
	EnableMenuItem(HMenu,ID_FILE_OPENVOB,m_MenuEnabled?MF_ENABLED:MF_GRAYED);
	EnableMenuItem(HMenu,ID_FILE_OPENDOMAIN,m_MenuEnabled?MF_ENABLED:MF_GRAYED);
	EnableMenuItem(HMenu,ID_FILE_OPENSUP,m_MenuEnabled?MF_ENABLED:MF_GRAYED);
	EnableMenuItem(HMenu,ID_FILE_SAVE,(m_MenuEnabled && m_CanSave) ?MF_ENABLED:MF_GRAYED);
	EnableMenuItem(HMenu,ID_FILE_OPENLAST,(m_MenuEnabled && m_CanSave) ?MF_ENABLED:MF_GRAYED);
	EnableMenuItem(HMenu,ID_FILE_SAVESUP,(m_MenuEnabled) ?MF_ENABLED:MF_GRAYED);
	EnableMenuItem(HMenu,ID_OCR_ONE,m_MenuEnabled?MF_ENABLED:MF_GRAYED);
	EnableMenuItem(HMenu,ID_OCR_REMOVEH,m_MenuEnabled?MF_ENABLED:MF_GRAYED);

	int i;
	for(i=0;i<m_NumMRU;i++)
	{
		EnableMenuItem(HMenu,ID_FILE_MRU0+i,m_MenuEnabled?MF_ENABLED:MF_GRAYED);
	}

}

void CDVDSubEditDlg::OnFindLeft() 
{
	FindNext(-1);
}

void CDVDSubEditDlg::OnFindRight() 
{
	FindNext(1);
}

void CDVDSubEditDlg::FindNext(int Inc)
{
	char Find[1024];
	GetDlgItemText(IDC_SEARCH,Find,1024);
	int i = m_SPUSlider.GetPos()+Inc;
	int k=0;
start:
	for(;i<m_PruneLength && i >= 0;i+=Inc)
	{
		// Ask SPU if it has that string.
		if(m_PrunedList[i]->FindString(Find,m_SetupDialog->m_CaseSensitive!=0))
		{
			// Yes!
			m_SPUSlider.SetPos(i);
			ViewSPU(i);
			return ;
		}
	}
	if(k++ == 0)
	{
		// We didn't find the search string, and this is the first time. Let's go back to start
		// or end, and search again.
		i=(Inc==1)?0:m_PruneLength-1;
		goto start;
	}
	MessageBeep(MB_ICONEXCLAMATION);
    return ;
}

// A 3-line class so I can set the title of the dialog the way I want it! GEEE!
class MyCColorDialog : public CColorDialog
{
public:
	char* m_Title;
	MyCColorDialog(COLORREF clrInit) : CColorDialog(clrInit) {};
	BOOL OnInitDialog() { CColorDialog::OnInitDialog(); SetWindowText(m_Title); return true;}
};

void CDVDSubEditDlg::OnClutClick(UINT nID) 
{
	if(m_SelectedSubPic == 0) return;
	int i = nID-IDC_COLB;
	static CPickColorDlg PickCol;
	char Temp[512];
	char Foo[4][3]={"b","p","e1","e2"};

	int SelClut = m_CLUTSelect.GetCurSel();
	if(SelClut == 0)
	{
		// Here we're picking which tile in the CLUT to use for pixel i.
		sprintf(Temp,"Pick color for pixel %s",Foo[i]);
		PickCol.m_WindowTitle = Temp;
		PickCol.m_Engine = this;
		PickCol.m_WhichPixel = i;
		// Set the currently selected tile for this pixel.
		PickCol.m_SelectedCol = m_SelectedSubPic->m_ColorPtr[i];
		if(PickCol.DoModal() == IDOK)
		{
			// Change the color for this pixel
			m_SelectedSubPic->ChangeColor(PickCol.m_SelectedCol,i);
			ViewSPU(m_SPUSlider.GetPos());
			// And record the change. Set the right bit in Pixel and set the right Byte in Color.
			m_ChangeRecord.RecordChange(m_SelectedSubPic, ChangeColor, PickCol.m_SelectedCol, i);
		}
	}
	else if(0 && m_SelectedSubPic->m_NumButtons > 0)
	{
		sprintf(Temp,"Pick color for pixel %s",Foo[i]);
		COLORREF *Color;
		switch(i)
		{
		case 0 : Color = &m_DefAutoPixColor; break;
		case 1 : Color = &m_DefAutoBckgColor; break;
		case 2 : Color = &m_DefAutoAA1Color; break;
		case 3 : Color = &m_DefAutoAA2Color; break;
		}
		MyCColorDialog TheColDialog(*Color); 
		TheColDialog.m_Title = Temp;
		if(TheColDialog.DoModal() == IDOK)
		{
			*Color = TheColDialog.GetColor();
			LOADCOLOR(CSubPic::m_DefColors,m_DefAutoPixColor);
			LOADCOLOR(CSubPic::m_DefColors+3,m_DefAutoBckgColor);
			LOADCOLOR(CSubPic::m_DefColors+6,m_DefAutoAA1Color);
			LOADCOLOR(CSubPic::m_DefColors+9,m_DefAutoAA2Color);
			ViewSPU(m_SPUSlider.GetPos());
			return;
		}
	}
	else
	{
		// Here, we're picking the color for the auto clut.
		COLORREF *Color;
		if(i==m_SelectedSubPic->m_BodyPix)
		{
			sprintf(Temp,"Pick color for text pixel. ");
			Color = &m_DefAutoPixColor;
		}
		else if(i==m_SelectedSubPic->m_BackgroundPix)
		{
			sprintf(Temp,"Pick color for background pixel");
			Color = &m_DefAutoBckgColor;
		}
		else if(i==m_SelectedSubPic->m_AntiAliasPixel1)
		{
			sprintf(Temp,"Pick color for anti-alias pixel 1");
			Color = &m_DefAutoAA1Color;
		}
		else
		{
			sprintf(Temp,"Pick color for anti-alias pixel 2");
			Color = &m_DefAutoAA2Color;
		}
		MyCColorDialog TheColDialog(*Color); 
		TheColDialog.m_Title = Temp;
		if(TheColDialog.DoModal() == IDOK)
		{
			*Color = TheColDialog.GetColor();
			LOADCOLOR(CSubPic::m_DefColors,m_DefAutoPixColor);
			LOADCOLOR(CSubPic::m_DefColors+3,m_DefAutoBckgColor);
			LOADCOLOR(CSubPic::m_DefColors+6,m_DefAutoAA1Color);
			LOADCOLOR(CSubPic::m_DefColors+9,m_DefAutoAA2Color);
			ViewSPU(m_SPUSlider.GetPos());
			return;
		}
	}
}

void CDVDSubEditDlg::ChangeColors(uchar* Colors, int Pixels, CSubPic* TheSubpic)
{
	int i;
	for(i=0;i<4;i++)
	{
		// Iterate over all pixels and if the corresponding bit is set in Pixels, change the color.
		if(Pixels & ((int)1<<i))
			TheSubpic->ChangeColor(Colors[i],i);
	}
}

void CDVDSubEditDlg::ChangeTransparencies(uchar* Transparencies, int Pixels, CSubPic* TheSubpic)
{
	int i;
	for(i=0;i<5;i++)
	{
		// Iterate over all pixels and if the corresponding bit is set in Pixels, change the transparency.
		if(Pixels & ((int)1<<i))
			TheSubpic->ChangeTransparency(Transparencies[i],i);
	}
}


#define MOVEWIN(A,DeltaX,DeltaY) 	{ CRect Tmp; CWnd* Wn = GetDlgItem(A); if(Wn != 0) {Wn->GetWindowRect(&Tmp); ScreenToClient(&Tmp); Wn->SetWindowPos(0,Tmp.left+DeltaX,Tmp.top+DeltaY,0,0,SWP_NOSIZE|SWP_NOZORDER|SWP_NOREDRAW);}}
#define MOVEWINS(A,DeltaY,DeltaW) 	{ CRect Tmp; CWnd* Wn = GetDlgItem(A); if(Wn != 0) {Wn->GetWindowRect(&Tmp); ScreenToClient(&Tmp); Wn->SetWindowPos(0,Tmp.left,Tmp.top+DeltaY,Tmp.Width()+DeltaW,Tmp.Height(),SWP_NOZORDER|SWP_NOREDRAW);}}
#define SIZEWIN(A,DeltaW,DeltaH) 	{ CRect Tmp; CWnd* Wn = GetDlgItem(A); if(Wn != 0) {Wn->GetWindowRect(&Tmp); ScreenToClient(&Tmp); Wn->SetWindowPos(0,Tmp.left,Tmp.top+DeltaY,Tmp.Width()+DeltaW,Tmp.Height()+DeltaH,SWP_NOMOVE|SWP_NOZORDER|SWP_NOREDRAW);}}

void CDVDSubEditDlg::OnSize(UINT nType, int cx, int cy) 
{
	// Avoid doing anything when minimizing!!!
	if(m_InitWindowWidth == -1 || nType == SIZE_MINIMIZED)
		return ;
	CRect NewRect;
	GetWindowRect(&NewRect);
	// Move whatever needs to be moved.
	static int LastWidth = m_InitWindowWidth;
	static int LastHeight = m_InitWindowHeight;
	int DeltaX = max(m_InitWindowWidth,NewRect.Width()) - LastWidth;
	int DeltaY = max(NewRect.Height(),m_InitWindowHeight) - LastHeight;
	LastWidth = max(m_InitWindowWidth,NewRect.Width());
	LastHeight = max(NewRect.Height(),m_InitWindowHeight);

	int i;
	for(i=IDC_RUN;i<=IDC_AUDIOSEL;i++)
	{
		if(i == IDC_COMMANDS || i == IDC_FRAME7 || i == IDC_SPUSLIDER || i == IDC_VIDEOSELECT)
		{
			MOVEWINS(i,DeltaY,DeltaX);
		}
		else if (i != IDC_VIDEOBACK)
		{
			if(i == IDC_DISPLAYMODE)
			{
				MOVEWIN(i,0,DeltaY);
			}
			else
			{
				MOVEWIN(i,DeltaX,DeltaY);
			}
		}
	}
	// We handle IDC_VIDEOBACK (the video fake window) differently.
	// We don't want IDC_VIDEOBACK to extend past IDC_DEFBUTTON
	GetDlgItem(IDC_VIDEOBACK)->GetWindowRect(&m_VideoRect);
	ScreenToClient(&m_VideoRect);
	CRect BotRight;
	GetDlgItem(IDC_DEFBUTTON)->GetWindowRect(&BotRight);
	ScreenToClient(&BotRight);
	int NewVidWidth = BotRight.left - m_VideoRect.left;
	int NewVidHeight = BotRight.top - m_VideoRect.top;
	// Resize that guy.
	GetDlgItem(IDC_VIDEOBACK)->SetWindowPos(0,0,0,NewVidWidth,NewVidHeight,
		SWP_NOMOVE|SWP_NOZORDER|SWP_NOREDRAW);
	GetDlgItem(IDC_VIDEOBACK)->GetWindowRect(&m_VideoRect);
	ScreenToClient(&m_VideoRect);
	m_DisplayHeight = m_VideoRect.Height();
	JustFoundVideoFormat();
	CDialog::OnSize(nType, cx, cy);
}

void CDVDSubEditDlg::OnGetMinMaxInfo( MINMAXINFO FAR* lpMMI )
{
	// This is called when the window is about to be resized.
	POINT Pt={m_InitWindowWidth,m_InitWindowHeight};
	lpMMI->ptMinTrackSize = Pt;
}

void CDVDSubEditDlg::OnActivate( UINT nState, CWnd* pWndOther, BOOL bMinimized )
{
	// I have to override this to NOT show the scan window if the other thread is done.
	if(!m_Running && m_ScanDialog->m_hWnd)
		m_ScanDialog->ShowWindow(SW_HIDE);
}

bool CDVDSubEditDlg::DlgPtInRect(UINT ID, CPoint point)
{
	CRect Rect;
	GetDlgItem(ID)->GetWindowRect(&Rect);
	ScreenToClient(&Rect);
	return ::PtInRect(Rect, (POINT) point) != 0;
}

static bool Tracking = false;
POINT DownPoint;
int LastVMove,LastHMove;
void CDVDSubEditDlg::OnLButtonDown(UINT nFlags, CPoint point) 
{
	if(m_SelectedSubPic && DlgPtInRect(IDC_Z9, point))
	{
		CSubPic::m_PixTempAlpha = 1;
		ViewSPU(m_SPUSlider.GetPos());
	}
	if(m_SelectedSubPic && DlgPtInRect(IDC_Z11, point))
	{
		CSubPic::m_PixTempAlpha = 2;
		ViewSPU(m_SPUSlider.GetPos());
	}
	if(m_SelectedSubPic && DlgPtInRect(IDC_Z10, point))
	{
		CSubPic::m_PixTempAlpha = 4;
		ViewSPU(m_SPUSlider.GetPos());
	}
	if(m_SelectedSubPic && DlgPtInRect(IDC_Z12, point))
	{
		CSubPic::m_PixTempAlpha = 8;
		ViewSPU(m_SPUSlider.GetPos());
	}
	if(m_SelectedSubPic && PtInRect(&m_VideoRect, (POINT) point))
	{
		DownPoint = (POINT)point;
		LastVMove = m_SelectedSubPic->m_LastVMove;
		LastHMove = m_SelectedSubPic->m_LastHMove;
		Tracking = true;
	}
	else if(m_SelectedSubPic && ChildWindowFromPoint((POINT) point) == &m_DurationSlider && !m_DurationSlider.IsWindowEnabled())
	{
		if(AfxMessageBox("This subpic does not have a timed STOP command\nTherefore it will be displayed forever (or until the next subpic is displayed).\nTo change its duration, do you want to add a STOP command?",MB_YESNO) == IDYES)
		{
			m_SelectedSubPic->AddStop();
			ViewSPU(m_SPUSlider.GetPos());
		}
	}
	else if(m_SelectedSubPic && ChildWindowFromPoint((POINT) point) == &m_ShowVideo && !m_IsMPEG2 && !m_SupFileMode)
	{
		AfxMessageBox("The background video seems to be of type MPEG-1\nDVDSubEdit can't display MPEG-1 video at this point");
	}
	CDialog::OnLButtonDown(nFlags, point);
}

void CDVDSubEditDlg::OnMouseMove(UINT nFlags, CPoint point) 
{
	if(PtInRect(&m_VideoRect, (POINT) point) && m_SelectedSubPic != NULL)
	{
		if(m_SelectedSubPic->CanHMove())
			SetCursor(AfxGetApp()->LoadStandardCursor(IDC_SIZEALL));
		else
			SetCursor(AfxGetApp()->LoadStandardCursor(IDC_SIZENS));
	}
	else
		SetCursor(AfxGetApp()->LoadStandardCursor(IDC_ARROW));

	if(Tracking && PtInRect(&m_VideoRect, (POINT) point))
	{
		unsigned short Mask = 0x8000;
		BOOL AltOn = (GetAsyncKeyState(VK_MENU) & Mask);
		BOOL CtrlOn = (GetAsyncKeyState(VK_CONTROL) & Mask);
		POINT TheMouse = (POINT) point;

#if 0
		// Get the coordinates relative to the 
		ClientToScreen(&TheMouse);
		GetDlgItem(IDC_VIDEOBACK)->ScreenToClient(&TheMouse);
#endif
		int MoveX = TheMouse.x - DownPoint.x;
		int MoveY = TheMouse.y - DownPoint.y;
		MoveX = (int)(MoveX / m_VideoScalingX);
		MoveY = (int)(MoveY / m_VideoScalingY);
		if(m_SelectedSubPic != NULL)
		{
			if(!AltOn)
			{
				m_SelectedSubPic->VMoveSubPic(-MoveY+LastVMove);
				m_ChangeRecord.RecordChange(m_SelectedSubPic, ChangeVMove, -MoveY+LastVMove);
			}
			if(!CtrlOn)
			{
				m_SelectedSubPic->HMoveSubPic(MoveX+LastHMove);
				m_ChangeRecord.RecordChange(m_SelectedSubPic, ChangeHMove, MoveX+LastHMove);
			}
			ViewSPU(m_SPUSlider.GetPos());
		}
		TRACE("TRACKING %d %d\n",MoveX, MoveY);
	}
	CDialog::OnMouseMove(nFlags, point);
}

void CDVDSubEditDlg::OnLButtonUp(UINT nFlags, CPoint point) 
{
	Tracking = false;
	CSubPic::m_PixTempAlpha &= 0xF0;
	ViewSPU(m_SPUSlider.GetPos());
	CDialog::OnLButtonUp(nFlags, point);
}


void CDVDSubEditDlg::TestOCR()
{
	// This is to run the OCR on a bunch of sup files and save the srt, for comparisons with previous
	// versions.
	FILE* fd = fopen("C:/temp/ocrtest.txt","r");
	char SaveTo[1024];
	char Ver[64];
	strcpy(Ver,m_Version+22);Ver[5]=0;
	strcpy(SaveTo,CatStr("C:/temp/OCRV",Ver));
	mkdir(SaveTo);
	char Temp[1024];
::SetThreadPriority(GetCurrentThread(),THREAD_PRIORITY_BELOW_NORMAL);
	while(1)
	{
		if(fgets(Temp,1024,fd) == 0)
			break;
		if(Temp[strlen(Temp)-1]=='\n')Temp[strlen(Temp)-1]=0;
		LoadSUPFile(Temp);
		OnSelchangeStreamselect();
		RunOCRFunc(this);
		{
			char *TempSrt = CatPath(SaveTo,CatStr(GetFname(Temp),".srt"));
			FILE* srtfd = fopen(TempSrt,"w");
			if(srtfd == 0)
				break;
			int i,j;
			for(i=0;i<m_PruneLength;i++)
			{
				CSubPic* Temp = m_PrunedList[i];
				if(Temp->m_OCRLines[0][0] == '\0' || Temp->GetMeanTransparency() == 0)
					continue;
				// srt files start at 1 apparently.
				fprintf(srtfd,"%d\n%s\n",i+1,PrintPTS(Temp->m_TruePTS,Temp->GetDCSQTDelay()));
				for(j=0;j<32;j++)
				{
					if(Temp->m_OCRLines[j][0] == '\0')
						break;
					fprintf(srtfd,"%s\n",Temp->m_OCRLines[j]);
				}
				fprintf(srtfd,"\n");
			}
			fclose(srtfd);
		}
	}
}


void CDVDSubEditDlg::MakeInfinite() 
{
	if(m_SelectedSubPic == NULL) return;
	if(m_SelectedSubPic->FindCommand(STP_DSP,m_SelectedSubPic->m_DCSQT.size()-1) != -1)
	{
		m_SelectedSubPic->RemoveDCSQT(m_SelectedSubPic->m_DCSQT.size()-1);
		m_ChangeRecord.RecordChange(m_SelectedSubPic, ChangeToInfinite, 0);
		ViewSPU(m_SPUSlider.GetPos());
	}
		
}

void CDVDSubEditDlg::OnAddfadein() 
{
	if(m_SelectedSubPic == NULL) return;
	m_SelectedSubPic->AddFadeIn();
	m_ChangeRecord.RecordChange(m_SelectedSubPic, ChangeAddFadeIn, 1);
	ViewSPU(m_SPUSlider.GetPos());
}


void CDVDSubEditDlg::OnRemovefadein()
{
	if(m_SelectedSubPic == NULL) return;
	m_SelectedSubPic->RemoveFadeIn();
	m_ChangeRecord.RecordChange(m_SelectedSubPic, ChangeAddFadeIn, -1);
	ViewSPU(m_SPUSlider.GetPos());
}

// Context menu handling.
void CDVDSubEditDlg::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	if(m_SelectedSubPic && pWnd == &m_CommandDisplay)
	{
		CMenu contextMenu; 
		CMenu* tracker; 
		contextMenu.LoadMenu(IDR_CONTEXT_COMMANDS); 
		tracker = contextMenu.GetSubMenu(0); 
		//show the context menu
		tracker->TrackPopupMenu(TPM_LEFTALIGN | TPM_TOPALIGN | TPM_LEFTBUTTON | TPM_RIGHTBUTTON, 
		  point.x , point.y , this); 
	}
	CDialog::OnContextMenu(pWnd, point); 
}

// Callback function used to copy from the command info RichEditCtrl into a file.
static DWORD CALLBACK EditStreamOut(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb)
{
	FILE *fd = (FILE *)dwCookie;
	pbBuff[cb]=0;
	int i;
	for(i=0;i<cb;i++)
	{
		// Don't keep \r\n as it will create two CR in the output file.
		if(pbBuff[i]=='\r' && pbBuff[i+1]=='\n')
			pbBuff[i] = ' ';
	}
	fprintf(fd,"%s",(char*)pbBuff);
	*pcb = cb;
	return 0;
}

void CDVDSubEditDlg::OnCommandsSave() 
{
	// Save all the subpic info in the command display into a file.
	CFileDialog FileDialog(FALSE, CString("*.txt"), "", OFN_OVERWRITEPROMPT, 
		CString("(*.txt)|*.txt||"));
	FileDialog.m_ofn.lpstrTitle = "Save all selected subpic info as a text file";
	if (FileDialog.DoModal() == IDOK)
	{
		FILE* fd = fopen((LPCTSTR)FileDialog.GetPathName(),"w");
		int i;
		int pos = m_SPUSlider.GetPos();
		if(fd == 0)
		{
			ERROR1("Couldn't open file for writing!");
			return;
		}
		CWaitCursor Cursor;
		EDITSTREAM es = {(DWORD)fd, 0, EditStreamOut};
		for(i=0;i<m_PruneLength;i++)
		{
			m_SPUSlider.SetPos(i);
			ViewSPU(i,0,1);
			// Read from m_Commands and copy into file... Then add one more \n
			m_CommandDisplay.StreamOut(SF_TEXT, es);
			fprintf(fd,"\n");
		}
		fclose(fd);
		// Go back where we were!
		m_SPUSlider.SetPos(pos);
		ViewSPU(pos);
	}
}

void CDVDSubEditDlg::OnSelendokZoomsel() 
{
	if(!m_ZoomDialog->IsWindowVisible()) m_RealSizeDisplay = 0;
	m_RealSizeDisplay = m_ZoomSelect.GetCurSel();
	m_ZoomDialog->ShowWindow(m_RealSizeDisplay ? SW_SHOW : SW_HIDE);
	SetActiveWindow();
	ViewSPU(m_SPUSlider.GetPos());	
}


void CDVDSubEditDlg::OnSelendokClutselect() 
{
	if(m_SelectedSubPic != NULL && m_CLUTSelect.GetCurSel() == 0 && GetCLUT() == NULL)
	{
		if(m_SupFileMode)
		{
			ERROR1("No CLUT! You loaded a SUP file and didn't load an IFO\nso there's no CLUT to be found.\nEither load the IFO or use the automatic CLUT");
		}
		else
		{
			ERROR1("There is no IFO associated with the VOB file you loaded.\nReverting to the automatic CLUT");
		}
		m_CLUTSelect.SetCurSel(1);
	}
	ViewSPU(m_SPUSlider.GetPos());
}

void CDVDSubEditDlg::OnPlay() 
{
	if(m_Preview->IsVideoRunning())
	{
		AbortPlayback();
		return;
	}
	m_Play.SetIcon(AfxGetApp()->LoadIcon(IDI_STOP));
	m_Loop.EnableWindow(false);
	if(m_AudioStreamSel.GetCount() > 0)
		m_Preview->SetPlayAudioStream(m_AudioStreamSel.GetItemData(m_AudioStreamSel.GetCurSel()));
	m_StreamSelect.SetCurSel(m_SelectedSubPic ? m_SelectedSubPic->m_StreamID+1 : 1);
	OnSelchangeStreamselect();
	m_Preview->Play();	
}

void CDVDSubEditDlg::OnLoop() 
{
	if(m_Preview->IsVideoRunning())
	{
		AbortPlayback();
		return;
	}
	m_Loop.SetIcon(AfxGetApp()->LoadIcon(IDI_STOP));
	m_Preview->SetStopPTS(-1);
	m_Play.EnableWindow(false);
	if(m_AudioStreamSel.GetCount() > 0)
		m_Preview->SetPlayAudioStream(m_AudioStreamSel.GetItemData(m_AudioStreamSel.GetCurSel()));
	m_StreamSelect.SetCurSel(m_SelectedSubPic ? m_SelectedSubPic->m_StreamID+1 : 1);
	OnSelchangeStreamselect();
	if(m_SubpicAtStartPlayback == -1) 
		m_SubpicAtStartPlayback = m_SPUSlider.GetPos();
	else {
		m_SelectedSubPic = m_PrunedList[m_SubpicAtStartPlayback];
	}

	if(m_SelectedSubPic == 0) return;
	m_Preview->GoToSector(max(0,m_SelectedSubPic->m_NavPackLBA - 300),0);
	m_Preview->WaitForThread();
//	m_Preview->SetStopSector(m_SelectedSubPic->m_NavPackLBA + 1000);
	int EndPTS = m_SelectedSubPic->GetEndPTS();
	if(EndPTS != -1) m_Preview->SetStopPTS(EndPTS);
	m_Preview->Play();	
}

void CDVDSubEditDlg::OnPlaybackStopped()
{
	m_Play.SetIcon(AfxGetApp()->LoadIcon(IDI_PLAY));
	m_Loop.SetIcon(AfxGetApp()->LoadIcon(IDI_LOOP));
	m_Loop.EnableWindow(true);
	m_Play.EnableWindow(true);
}

void CDVDSubEditDlg::AbortPlayback(bool NoReturn)
{
	bool Looping = m_Preview->GetStopPTS() != -1;
	m_Preview->SetStopPTS(-1);
	m_Preview->Stop(false);
	m_Preview->WaitForThread();
	if(Looping && !NoReturn)
	{
		// Go back!
		m_SPUSlider.SetPos(m_SubpicAtStartPlayback);
		ViewSPU(m_SPUSlider.GetPos());
	}
	m_SubpicAtStartPlayback = -1;
}


void CDVDSubEditDlg::OnSelendokAudiosel() 
{
	m_Preview->SetPlayAudioStream(m_AudioStreamSel.GetItemData(m_AudioStreamSel.GetCurSel()));
}
